//
//  PSMWebSpiderPlugin.m
//  Pandora
//
//  Created by John Pannell on 2/16/06.
//  Copyright 2006 Positive Spin Media. All rights reserved.
//

#import "PSMConsecutiveNumbersPlugin.h"

@implementation PSMConsecutiveNumbersPlugin

#pragma mark -
#pragma mark Creation/Destruction

- (id)init
{
    if(self = [super initWithWindowNibName:@"PSMConsecutiveConfig"]){
        _maxFileSize = 0;
        _minFileSize = 0;
    }
    return self;
}

- (void)awakeFromNib
{
    [icon setImage:[[self class] icon]];
}

#pragma mark -
#pragma mark Identification

+ (NSImage *)icon
{
    return [[NSWorkspace sharedWorkspace] iconForFile:[[NSBundle bundleForClass:[self class]] bundlePath]];
}

+ (NSString *)name
{
    return [[[[NSBundle bundleForClass:[self class]] bundlePath] lastPathComponent] stringByDeletingPathExtension];
}

+ (NSString *)description
{
    return @"Finds files by creating URLs from a numerical pattern.";
}

+ (NSString *)category
{
    return @"Spiders";
}

+ (NSString *)prompt
{
    return @"Type a URL here, use square brackets for pattern";
}

#pragma mark -
#pragma mark Accessors

- (void)setMIMETypes:(NSSet *)types
{
    [types retain];
    [_MIMETypes release];
    _MIMETypes = types;
}

- (void)setMinFileSize:(int)kilobytes
{
    _minFileSize = kilobytes;
}

- (void)setMaxFileSize:(int)kilobytes
{
    _maxFileSize = kilobytes;
}

#pragma mark -
#pragma mark UI

- (IBAction)endConfig:(id)sender
{
    [NSApp endSheet:[self window]];
    [[self window] orderOut:self];
    [NSApp stopModal];
    [self release];
}

#pragma mark -
#pragma mark Spider

- (BOOL)contentFromDifferentServers
{
    // return YES if your plugin rewrites returned links to fetch varous contents from different servers, otherwise return NO.  This would only be YES if you were scraping search engine result pages and returning links to source content.  It should definitely be NO for a spider that moves from link to link on the same server.
    return NO;
}

- (int)linkDepth
{
    // you can keep the spider limited to a specific link depth if desired.  Return -1 if there is no restriction.
    return 0;
}

- (NSMutableArray *)preprocessURLOrKeyword:(NSString *)string
{
    // find number pattern
    int start = [string rangeOfString:@"["].location;
    int end = [string rangeOfString:@"]"].location;
    int length = end - start;
    NSString *prefix = [string substringToIndex:start];
    NSString *suffix = [string substringFromIndex:(end + 1)];
    NSString *pattern = [string substringWithRange:NSMakeRange((start + 1), (length - 1))];
    NSArray *pieces = [pattern componentsSeparatedByString:@"-"];
    int begin, finish;
    if([pieces count] > 1){
        begin = [[pieces objectAtIndex:0] intValue];
        finish = [[pieces objectAtIndex:1] intValue];
    } else {
        NSLog(@"Consecutive Number Plugin: couldn't create sequence from pattern");
        return nil;
    }
    
    int padLength = [[pieces objectAtIndex:0] length];
    
    NSMutableArray *returnArray = [NSMutableArray array];
    int i;
    
    for(i = begin; i <= finish; i++){
        NSString *preString = [NSString stringWithFormat:@"0000000000000000000%d", i]; // leading zeros
        NSString *currentImageNumber = [preString substringFromIndex:([preString length] - padLength)];
        NSString *newURL = [NSString stringWithFormat:@"%@%@%@", prefix, currentImageNumber, suffix];
        NSDictionary *dict = [NSDictionary dictionaryWithObjectsAndKeys:newURL, @"URL", @"identify", @"action", nil];
        [returnArray addObject:dict];
    }
    
    return returnArray;
}

- (BOOL)shouldPursueResourceWithURL:(NSString *)url MIMEType:(NSString *)mime andExpectedFileSize:(long long)size withReason:(NSString **)reason
{
    if(![_MIMETypes containsObject:mime]){
        *reason = @"Wrong MIME Type";
        return NO;
    }
    
    if((_minFileSize != 0) && (size < (_minFileSize * 1024))){
        *reason = @"File is too small";
        return NO;
    }
    
    if((_maxFileSize != 0) && (size > (_maxFileSize * 1024))){
        *reason = @"File is too large";
        return NO;
    }
    
    return YES;
}

- (NSDictionary *)postprocessURL:(NSString *)url
{
    // given a URL, you can determine not to use it (return nil), to alter it (return another string)
    // or pass it back unchanged.  The URLs are fed to this function right after they are discovered on
    // a parsed HTML web page.  You must package the returned string into a dictionary... the dictionary
    // currently supports two keys: source and parent - both should be strings representing URLs.  The
    // source key is for the URL that should be fetched, and the "parent" is metadata, so the file knows
    // what web page it came from.  The parent key is optional, and you would only know it in the case where
    // you are scraping a search engine or something.
    
    return [NSDictionary dictionaryWithObjectsAndKeys:url, @"source", nil];
}

#pragma mark -
#pragma mark Authentication

- (NSArray *)authenticationInfo
{
    // the return array has two objects, an array of keys, and an array of corresponding values.
    // these keys and values are used to fill out a web form and submit it, or to 
    // provide to HTTP authentication.
    
    // For web forms, the keys are the names of the input elements that need to be filled out.
    // note that the keys and values are case sensitive!
    
    // if providing info for HTTP authentication, provide keys "username" and "password", in that order.
    
    // return [NSArray arrayWithObjects:[NSArray arrayWithObjects:@"username", @"password", nil], [NSArray arrayWithObjects:@"myUsername", @"myPassword", nil], nil];
    
    return nil;
}

- (BOOL)parseAuthSuccessPage
{
    return NO;
}

- (NSString *)authSuccessURLFragment
{
    return nil;
}

@end
