//
//  PSMGoogleImageSearchPlugin.m
//  Pandora
//
//  Created by John Pannell on 4/3/06.
//  Copyright 2006 Positive Spin Media. All rights reserved.
//

#import "PSMFlickrSearchPlugin.h"

@interface PSMFlickrSearchPlugin (Private)
- (NSMutableDictionary *)preferences;
@end

@implementation PSMFlickrSearchPlugin

#pragma mark -
#pragma mark Creation/Destruction

+ (void)initialize
{
    // if preferences do not yet exist, I must register some defaults
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    if([[defaults persistentDomainNames] containsObject:[[NSBundle bundleForClass:[self class]] bundleIdentifier]]){
        return; // already exists
    }
    
    // create dictionary
    NSMutableDictionary *defaultValues = [NSMutableDictionary dictionary];
    
    // set values
    [defaultValues setObject:[NSNumber numberWithInt:0] forKey:@"ID type"];
    [defaultValues setObject:[NSString stringWithString:@""] forKey:@"username"];
    [defaultValues setObject:[NSString stringWithString:@""] forKey:@"password"];
    [defaultValues setObject:[NSNumber numberWithInt:0] forKey:@"look in"];
    [defaultValues setObject:[NSNumber numberWithInt:0] forKey:@"match"];
    [defaultValues setObject:[NSString stringWithString:@""] forKey:@"dont match"];
    [defaultValues setObject:[NSNumber numberWithInt:0] forKey:@"search type"];
    [defaultValues setObject:[NSNumber numberWithInt:0] forKey:@"Find photos"];
    [defaultValues setObject:[NSString stringWithString:@""] forKey:@"after date"];
    [defaultValues setObject:[NSString stringWithString:@""] forKey:@"before date"];
    [defaultValues setObject:[NSNumber numberWithBool:NO] forKey:@"Only creative commons"];
    [defaultValues setObject:[NSNumber numberWithBool:NO] forKey:@"commons commercial"];
    [defaultValues setObject:[NSNumber numberWithBool:NO] forKey:@"commons modify"];
    
    
    // write to disk
    [defaults setPersistentDomain:defaultValues forName:[[NSBundle bundleForClass:[self class]] bundleIdentifier]];
}


- (id)init
{
    if(self = [super initWithWindowNibName:@"FlickrSearchConfig"]){
        _maxFileSize = 0;
        _minFileSize = 0;
        _preferences = [[[NSUserDefaults standardUserDefaults] persistentDomainForName:[[NSBundle bundleForClass:[self class]] bundleIdentifier]] mutableCopy];
    }
    return self;
}

- (void)awakeFromNib
{
    [icon setImage:[[self class] icon]];
    if([[_preferences objectForKey:@"ID type"] intValue] == 0)
        [usernameLabel setStringValue:@"E-mail:"];
    if([[_preferences objectForKey:@"ID type"] intValue] == 1)
        [usernameLabel setStringValue:@"Yahoo ID:"];
}

#pragma mark -
#pragma mark Identification

+ (NSImage *)icon
{
    return [[NSWorkspace sharedWorkspace] iconForFile:[[NSBundle bundleForClass:[self class]] bundlePath]];
}

+ (NSString *)name
{
    return [[[[NSBundle bundleForClass:[self class]] bundlePath] lastPathComponent] stringByDeletingPathExtension];
}

+ (NSString *)description
{
    return @"Finds images using keywords and the Flickr website.";
}

+ (NSString *)category
{
    return @"Agents";
}

+ (NSString *)prompt
{
    return @"Type keywords here";
}

#pragma mark -
#pragma mark UI

- (IBAction)idRadioAction:(id)sender
{
    if([[sender selectedCell] tag] == 0)
        [usernameLabel setStringValue:@"E-mail:"];
    if([[sender selectedCell] tag] == 1)
        [usernameLabel setStringValue:@"Yahoo ID:"];
}

- (IBAction)useCCAction:(id)sender
{
    // must shut off other CC actions if this is being turned off
    if([sender state] == NSOffState){
        [prefController setValue:[NSNumber numberWithBool:NO] forKeyPath:@"selection.commons commercial"];
        [prefController setValue:[NSNumber numberWithBool:NO] forKeyPath:@"selection.commons modify"];
    }
}
        

#pragma mark -
#pragma mark Accessors

- (NSMutableDictionary *)preferences
{
    return _preferences;
}

- (void)setMIMETypes:(NSSet *)types
{
    [types retain];
    [_MIMETypes release];
    _MIMETypes = types;
}

- (void)setMinFileSize:(int)kilobytes
{
    _minFileSize = kilobytes;
}

- (void)setMaxFileSize:(int)kilobytes
{
    _maxFileSize = kilobytes;
}

#pragma mark -
#pragma mark Preferences

- (IBAction)recordPreferences:(id)sender
{
    // end editing for various fields so values are picked up with bindings
    [[usernameField cell] endEditing:[[self window] fieldEditor:NO forObject:usernameField]];
    [[badWordField cell] endEditing:[[self window] fieldEditor:NO forObject:badWordField]];
    [[afterField cell] endEditing:[[self window] fieldEditor:NO forObject:afterField]];
    [[beforeField cell] endEditing:[[self window] fieldEditor:NO forObject:beforeField]];
    [[passwordField cell] endEditing:[[self window] fieldEditor:NO forObject:passwordField]];
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    [defaults setPersistentDomain:_preferences forName:[[NSBundle bundleForClass:[self class]] bundleIdentifier]];
    [defaults synchronize];
    [NSApp endSheet:[self window]];
    [[self window] orderOut:self];
    [NSApp stopModal];
    [self release];
}


#pragma mark -
#pragma mark Spider

- (BOOL)contentFromDifferentServers
{
    // return YES if your plugin rewrites returned links to fetch various contents from different servers, otherwise return NO.  This would only be YES if you were scraping search engine result pages and returning links to source content.  It should definitely be NO for a spider that moves from link to link on the same server.
    return NO;
}

- (int)linkDepth
{
    // you can keep the spider limited to a specific link depth if desired.  Return -1 if there is no restriction.
    return -1;
}

- (NSMutableArray *)preprocessURLOrKeyword:(NSString *)string
{
    // create a URL for flickr search with proper parameters
    // where to look
    NSString *lookInCondition;
    if([[_preferences objectForKey:@"look in"] intValue] == 0)
        lookInCondition = @"w=all";
    if([[_preferences objectForKey:@"look in"] intValue] == 1)
        lookInCondition = @"w=me";
    
    // query string
    NSString *queryString;
    if([[_preferences objectForKey:@"match"] intValue] == 0)
        queryString = [NSString stringWithFormat:@"%@", string];
    if([[_preferences objectForKey:@"match"] intValue] == 1)
        queryString = [NSString stringWithFormat:@"\"%@\"", string];
    if([[_preferences objectForKey:@"match"] intValue] == 2){
        NSMutableString *tempString = [string mutableCopy];
        [tempString replaceOccurrencesOfString:@" " withString:@" OR " options:NSBackwardsSearch range:NSMakeRange(0, [tempString length])];
        queryString = [NSString stringWithFormat:@"%@", tempString];
    }
    
    // none of these words
    if(![[_preferences objectForKey:@"dont match"] isEqualToString:@""] && ([_preferences objectForKey:@"dont match"] != NULL)){
        NSMutableString *negationString = [[_preferences objectForKey:@"dont match"] mutableCopy];
        [negationString replaceOccurrencesOfString:@" " withString:@" -" options:NSBackwardsSearch range:NSMakeRange(0, [negationString length])];
        queryString = [NSString stringWithFormat:@"(%@) -%@", queryString, negationString];
    }
    
    NSString *searchType;
    if([[_preferences objectForKey:@"search type"] intValue] == 0)
        searchType = [NSString stringWithString:@"m=text"];
    if([[_preferences objectForKey:@"search type"] intValue] == 1)
        searchType = [NSString stringWithString:@"m=tags"];
    
    // date
    NSMutableString *dateCondition = [NSMutableString stringWithString:@"d="];
    if(!([[_preferences objectForKey:@"after date"] isEqualToString:@""] || ([_preferences objectForKey:@"after date"] == NULL))  || !([[_preferences objectForKey:@"before date"] isEqualToString:@""] || ([_preferences objectForKey:@"before date"] == NULL))){
        if([[_preferences objectForKey:@"Find photos"] intValue] == 0)
            [dateCondition appendString:@"taken"];
        if([[_preferences objectForKey:@"Find photos"] intValue] == 1)
            [dateCondition appendString:@"posted"];
        if(!([[_preferences objectForKey:@"after date"] isEqualToString:@""] || ([_preferences objectForKey:@"after date"] == NULL))){
            NSString *afterString = [NSString stringWithFormat:@"%@%@%@", [[_preferences objectForKey:@"after date"] lastPathComponent], [[_preferences objectForKey:@"after date"] substringWithRange:NSMakeRange(0, 2)], [[_preferences objectForKey:@"after date"] substringWithRange:NSMakeRange(3, 2)]];
            [dateCondition appendFormat:@"-%@", afterString];
        } else {
            [dateCondition appendString:@"-"];
        }
        if(!([[_preferences objectForKey:@"before date"] isEqualToString:@""] || ([_preferences objectForKey:@"before date"] == NULL))){
            NSString *beforeString = [NSString stringWithFormat:@"%@%@%@", [[_preferences objectForKey:@"before date"] lastPathComponent], [[_preferences objectForKey:@"before date"] substringWithRange:NSMakeRange(0, 2)], [[_preferences objectForKey:@"before date"] substringWithRange:NSMakeRange(3, 2)]];
            [dateCondition appendFormat:@"-%@", beforeString];
        } else {
            [dateCondition appendString:@"-"];
        }
    }
    if([dateCondition isEqualToString:@"d="])
        dateCondition = @"";
    
    // creative commons
    NSMutableString *ccCondition = @"";
    if([[_preferences objectForKey:@"Only creative commons"] boolValue])
        ccCondition = @"l=cc";
    if([[_preferences objectForKey:@"commons commercial"] boolValue])
        ccCondition = @"l=comm";
    if([[_preferences objectForKey:@"commons modify"] boolValue])
        ccCondition = @"l=deriv";
    if([[_preferences objectForKey:@"commons commercial"] boolValue] && [[_preferences objectForKey:@"commons modify"] boolValue])
        ccCondition = @"l=commderiv";
    
    NSString *flickrString = [NSString stringWithFormat:@"http://www.flickr.com/search/?%@&q=%@&%@&%@&%@&z=t", lookInCondition, [queryString stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding], ccCondition, dateCondition, searchType];
    
    // should I authenticate first?
    NSMutableArray *returnArray;
    if(![[_preferences objectForKey:@"username"] isEqualToString:@""] && ![[_preferences objectForKey:@"password"] isEqualToString:@""] && ([_preferences objectForKey:@"username"] != NULL) && ([_preferences objectForKey:@"password"] != NULL)){
        // load proper login URL
        if([[_preferences objectForKey:@"ID type"] intValue] == 0){
            // Flickr
            NSString *flickrLogin = [NSString stringWithString:@"http://www.flickr.com/signin/flickr/"];
            NSDictionary *loginDict = [NSDictionary dictionaryWithObjectsAndKeys:flickrLogin, @"URL", @"parse", @"action", nil];
            NSDictionary *dict = [NSDictionary dictionaryWithObjectsAndKeys:flickrString, @"URL", @"parse", @"action", nil];
            returnArray = [NSMutableArray arrayWithObjects:loginDict, dict, nil];
        } else {
            // Yahoo
            NSString *yahooLogin = [NSString stringWithString:@"https://login.yahoo.com/config/login?.intl=us&.partner=&.last=&.src=flkctx&.done=https%3a//login.yahoo.com/config/validate%3f.src=flkctx%26.pc=5134%26.done=http%253A%252F%252Fwww.flickr.com%252Fsignin%252Fyahoo%252F"];
            NSDictionary *loginDict = [NSDictionary dictionaryWithObjectsAndKeys:yahooLogin, @"URL", @"parse", @"action", nil];
            NSDictionary *dict = [NSDictionary dictionaryWithObjectsAndKeys:flickrString, @"URL", @"parse", @"action", nil];
            returnArray = [NSMutableArray arrayWithObjects:loginDict, dict, nil];
        }
    } else {
        // no auth
        NSDictionary *dict = [NSDictionary dictionaryWithObjectsAndKeys:flickrString, @"URL", @"parse", @"action", nil];
        returnArray = [NSMutableArray arrayWithObject:dict];
    }
    return returnArray;
}

- (BOOL)shouldPursueResourceWithURL:(NSString *)url MIMEType:(NSString *)mime andExpectedFileSize:(long long)size withReason:(NSString **)reason
{
    if(![_MIMETypes containsObject:mime]){
        *reason = @"Wrong MIME Type";
        return NO;
    }
    
    if((_minFileSize != 0) && (size < (_minFileSize * 1024))){
        *reason = @"File is too small";
        return NO;
    }
    
    if((_maxFileSize != 0) && (size > (_maxFileSize * 1024))){
        *reason = @"File is too large";
        return NO;
    }
    
    return YES;
}

- (NSDictionary *)postprocessURL:(NSString *)url
{
    // or pass it back unchanged.  The URLs are fed to this function right after they are discovered on
    // a parsed HTML web page.  You must package the returned string into a dictionary... the dictionary
    // currently supports two keys: source and parent - both should be strings representing URLs.  The
    // source key if for the URL that should be fetched, and the "parent" is metadata, so the file knows
    // what web page it came from.  The parent key is optional, and you would only know it in the case where
    // you are scraping a search engine or something.
    if([url rangeOfString:@"&page="].length != 0){
        // link to more results - follow
        return [NSDictionary dictionaryWithObjectsAndKeys:url, @"source", nil];
    }
    
    // I'll consider Jpegs :-)
    if([[url pathExtension] isEqualToString:@"jpg"] || [[url pathExtension] isEqualToString:@"jpeg"]){
        if([url rangeOfString:@"_t.jp"].length != 0){ // take thumbnail, and...
            NSMutableString *newURL = [[url mutableCopy] autorelease];
            [newURL replaceOccurrencesOfString:@"_t.jp" withString:@".jp" options:NSBackwardsSearch range:NSMakeRange(0, [newURL length])];
            return [NSDictionary dictionaryWithObjectsAndKeys:newURL, @"source", nil];
        }
    }
    
    
    // skip everything else
    return nil;
}

#pragma mark -
#pragma mark Authentication

- (NSArray *)authenticationInfo
{
    // the return array has two objects, an array of keys, and an array of corresponding values.
    // these keys and values are used to fill out a web form and submit it, or to 
    // provide to HTTP authentication.
    
    // For web forms, the keys are the names of the input elements that need to be filled out.
    // note that the keys and values are case sensitive!
    
    // if providing info for HTTP authentication, provide keys "username" and "password", in that order.
    
    if(![[_preferences objectForKey:@"username"] isEqualToString:@""] && ![[_preferences objectForKey:@"password"] isEqualToString:@""] && ([_preferences objectForKey:@"username"] != NULL) && ([_preferences objectForKey:@"password"] != NULL)){
        if([[_preferences objectForKey:@"ID type"] intValue] == 0){
            // flickr
            return [NSArray arrayWithObjects:[NSArray arrayWithObjects:@"email", @"password", nil], [NSArray arrayWithObjects:[_preferences objectForKey:@"username"], [_preferences objectForKey:@"password"], nil], nil];
        } else {
            // Yahoo
            return [NSArray arrayWithObjects:[NSArray arrayWithObjects:@"login", @"passwd", nil], [NSArray arrayWithObjects:[_preferences objectForKey:@"username"], [_preferences objectForKey:@"password"], nil], nil];
        }
    }
    
    return nil;
}

- (BOOL)parseAuthSuccessPage
{
    return NO;
}

- (NSString *)authSuccessURLFragment
{
    return @"http://www.flickr.com/";
}

@end
