//
//  PSMYahooImageSearchPlugin.m
//  Pandora
//
//  Created by John Pannell on 4/3/06.
//  Copyright 2006 Positive Spin Media. All rights reserved.
//

#import "PSMYahooImageSearchPlugin.h"

@interface PSMYahooImageSearchPlugin (Private)
- (NSMutableDictionary *)preferences;
@end

@implementation PSMYahooImageSearchPlugin

#pragma mark -
#pragma mark Creation/Destruction

+ (void)initialize
{
    // if preferences do not yet exist, I must register some defaults
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    if([[defaults persistentDomainNames] containsObject:[[NSBundle bundleForClass:[self class]] bundleIdentifier]]){
        return; // already exists
    }
    
    // create dictionary
    NSMutableDictionary *defaultValues = [NSMutableDictionary dictionary];
    
    // set values
    [defaultValues setObject:[NSNumber numberWithInt:0] forKey:@"image size"];
    [defaultValues setObject:[NSNumber numberWithInt:0] forKey:@"coloration"];
    [defaultValues setObject:[NSNumber numberWithInt:0] forKey:@"safe search"];
    [defaultValues setObject:@"" forKey:@"site"];
    
    // write to disk
    [defaults setPersistentDomain:defaultValues forName:[[NSBundle bundleForClass:[self class]] bundleIdentifier]];
}


- (id)init
{
    if(self = [super initWithWindowNibName:@"YahooImageSearchConfig"]){
        _maxFileSize = 0;
        _minFileSize = 0;
        _preferences = [[[NSUserDefaults standardUserDefaults] persistentDomainForName:[[NSBundle bundleForClass:[self class]] bundleIdentifier]] mutableCopy];
    }
    return self;
}

- (void)awakeFromNib
{
    [icon setImage:[[self class] icon]];
}

#pragma mark -
#pragma mark Identification

+ (NSImage *)icon
{
    return [[NSWorkspace sharedWorkspace] iconForFile:[[NSBundle bundleForClass:[self class]] bundlePath]];
}

+ (NSString *)name
{
    return [[[[NSBundle bundleForClass:[self class]] bundlePath] lastPathComponent] stringByDeletingPathExtension];
}

+ (NSString *)description
{
    return @"Finds images using keywords and the Yahoo/AltaVista Image Search website.";
}

+ (NSString *)category
{
    return @"Agents";
}

+ (NSString *)prompt
{
    return @"Type keywords here";
}

#pragma mark -
#pragma mark Accessors

- (NSMutableDictionary *)preferences
{
    return _preferences;
}

- (void)setMIMETypes:(NSSet *)types
{
    [types retain];
    [_MIMETypes release];
    _MIMETypes = types;
}

- (void)setMinFileSize:(int)kilobytes
{
    _minFileSize = kilobytes;
}

- (void)setMaxFileSize:(int)kilobytes
{
    _maxFileSize = kilobytes;
}

#pragma mark -
#pragma mark Preferences

- (IBAction)recordPreferences:(id)sender
{
    // end editing for link depth field so value is picked up with bindings
    [[siteField cell] endEditing:[[self window] fieldEditor:NO forObject:siteField]];
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    [defaults setPersistentDomain:_preferences forName:[[NSBundle bundleForClass:[self class]] bundleIdentifier]];
    [defaults synchronize];
    [NSApp endSheet:[self window]];
    [[self window] orderOut:self];
    [NSApp stopModal];
    [self release];
}


#pragma mark -
#pragma mark Spider

- (BOOL)contentFromDifferentServers
{
    // return YES if your plugin rewrites returned links to fetch various contents from different servers, otherwise return NO.  This would only be YES if you were scraping search engine result pages and returning links to source content.  It should definitely be NO for a spider that moves from link to link on the same server.
    return YES;
}

- (int)linkDepth
{
    // you can keep the spider limited to a specific link depth if desired.  Return -1 if there is no restriction.
    return -1;
}

- (NSMutableArray *)preprocessURLOrKeyword:(NSString *)string
{
    //NSLog(@"preprocess. Input string: %@", string);
    // create a URL for yahoo image search with proper parameters
    // image size
    NSString *imageSizeCondition;
    if([[_preferences objectForKey:@"image size"] intValue] == 0)
        imageSizeCondition = @"imgsz=";
    if([[_preferences objectForKey:@"image size"] intValue] == 1)
        imageSizeCondition = @"imgsz=wallpaper";
    if([[_preferences objectForKey:@"image size"] intValue] == 2)
        imageSizeCondition = @"imgsz=small";
    if([[_preferences objectForKey:@"image size"] intValue] == 3)
        imageSizeCondition = @"imgsz=medium";
    if([[_preferences objectForKey:@"image size"] intValue] == 4)
        imageSizeCondition = @"imgsz=large";
    //NSLog(@"preprocess. imageSizeCondition string: %@", imageSizeCondition);
    
    // coloration
    NSString *colorationCondition;
    if([[_preferences objectForKey:@"coloration"] intValue] == 0)
        colorationCondition = @"imgc=";
    if([[_preferences objectForKey:@"coloration"] intValue] == 1)
        colorationCondition = @"imgc=bw";
    if([[_preferences objectForKey:@"coloration"] intValue] == 2)
        colorationCondition = @"imgc=color";
    //NSLog(@"preprocess. colorationCondition string: %@", colorationCondition);
    
    // safesearch
    NSString *safeSearchCondition;
    if([[_preferences objectForKey:@"safe search"] intValue] == 0)
        safeSearchCondition = @"safe=off";
    if([[_preferences objectForKey:@"safe search"] intValue] == 1)
        safeSearchCondition = @"safe=images";
    if([[_preferences objectForKey:@"safe search"] intValue] == 2)
        safeSearchCondition = @"safe=active";
    //NSLog(@"preprocess. safeSearchCondition string: %@", safeSearchCondition);
    
    // site
    NSString *siteSearchCondition;
    if([_preferences objectForKey:@"site"]){
        siteSearchCondition = [NSString stringWithFormat:@"vs=%@", [_preferences objectForKey:@"site"]];
    } else {
        siteSearchCondition = @"vs=";
    }
    //NSLog(@"preprocess. siteSearchCondition string: %@", siteSearchCondition);
    
    // query
    NSString *query = [string stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding];
    //NSLog(@"preprocess. query string: %@", query);
    
    NSString *yahooString = [NSString stringWithFormat:@"http://images.search.yahoo.com/search/images?_adv_prop=images&x=op&ei=UTF-8&fr2=op&js=0&va=%@&%@&%@&%@", query, imageSizeCondition, colorationCondition, siteSearchCondition, safeSearchCondition];
    //NSLog(@"preprocess. googleString string: %@", googleString);
    
    // DEBUG - write contents of this page to file
    /*
    if(![[NSFileManager defaultManager] fileExistsAtPath:[NSHomeDirectory() stringByAppendingPathComponent:@"GoogleResults.html"]]){
        [[NSString stringWithContentsOfURL:[NSURL URLWithString:googleString]] writeToFile:[NSHomeDirectory() stringByAppendingPathComponent:@"GoogleResults.html"] atomically:YES encoding:NSUTF8StringEncoding error:NULL];
    }
     */
    
    NSDictionary *dict = [NSDictionary dictionaryWithObjectsAndKeys:yahooString, @"URL", @"parse", @"action", nil];
    NSMutableArray *returnArray = [NSMutableArray arrayWithObject:dict];
    //NSLog(@"preprocess. returnArray: %@", returnArray);
    
    return returnArray;
}

- (BOOL)shouldPursueResourceWithURL:(NSString *)url MIMEType:(NSString *)mime andExpectedFileSize:(long long)size withReason:(NSString **)reason
{
    if(![_MIMETypes containsObject:mime]){
        *reason = @"Wrong MIME Type";
        return NO;
    }
    
    if((_minFileSize != 0) && (size < (_minFileSize * 1024))){
        *reason = @"File is too small";
        return NO;
    }
    
    if((_maxFileSize != 0) && (size > (_maxFileSize * 1024))){
        *reason = @"File is too large";
        return NO;
    }
    
    return YES;
}

- (NSDictionary *)postprocessURL:(NSString *)url
{
    // or pass it back unchanged.  The URLs are fed to this function right after they are discovered on
    // a parsed HTML web page.  You must package the returned string into a dictionary... the dictionary
    // currently supports two keys: source and parent - both should be strings representing URLs.  The
    // source key if for the URL that should be fetched, and the "parent" is metadata, so the file knows
    // what web page it came from.  The parent key is optional, and you would only know it in the case where
    // you are scraping a search engine or something.
    if([url rangeOfString:@"imgurl="].length != 0){
        // link to image - de-urlencode
        NSString *sourceURL = [NSString string];
        NSString *parentURL = [NSString string];
        NSScanner *imageScanner = [NSScanner scannerWithString:url];
        [imageScanner scanUpToString:@"imgurl=" intoString:nil];
        [imageScanner scanString:@"imgurl=" intoString:nil];
        [imageScanner scanUpToString:@"2526rurl=" intoString:&sourceURL];
        [imageScanner scanString:@"2526rurl=" intoString:nil];
        [imageScanner scanUpToString:@"2526" intoString:&parentURL];
        
        // chop off last % remaining
        NSMutableString *sourceURL2 = [[[NSMutableString stringWithString:sourceURL] substringToIndex:([sourceURL length] - 1)] mutableCopy];
        NSMutableString *parentURL2 = [[[NSMutableString stringWithString:parentURL] substringToIndex:([parentURL length] - 1)] mutableCopy];
        
        // add http:// to source URL
        [sourceURL2 insertString:@"http://" atIndex:0];
        
        // double decode the %'s
        [sourceURL2 replaceOccurrencesOfString:@"%25" withString:@"%" options:NSBackwardsSearch range:NSMakeRange(0,[sourceURL2 length])];
        [sourceURL2 replaceOccurrencesOfString:@"%25" withString:@"%" options:NSBackwardsSearch range:NSMakeRange(0,[sourceURL2 length])];
        [parentURL2 replaceOccurrencesOfString:@"%25" withString:@"%" options:NSBackwardsSearch range:NSMakeRange(0,[parentURL2 length])];
        [parentURL2 replaceOccurrencesOfString:@"%25" withString:@"%" options:NSBackwardsSearch range:NSMakeRange(0,[parentURL2 length])];
        
        NSMutableString *finalSourceURL = (NSMutableString *)CFURLCreateStringByReplacingPercentEscapesUsingEncoding(kCFAllocatorDefault, (CFStringRef)sourceURL2, CFSTR(""), kCFStringEncodingUTF8);
        NSMutableString *finalParentURL = (NSMutableString *)CFURLCreateStringByReplacingPercentEscapesUsingEncoding(kCFAllocatorDefault, (CFStringRef)parentURL2, CFSTR(""), kCFStringEncodingUTF8);

        [finalSourceURL autorelease];
        [finalParentURL autorelease];
        [parentURL2 release];
        [sourceURL2 release];

        return [NSDictionary dictionaryWithObjectsAndKeys:finalSourceURL, @"source", finalParentURL, @"parent", nil];
    }
    
    
    if(([url rangeOfString:@"images.search.yahoo.com/search/images"].length != 0) && ([url rangeOfString:@"b="].length != 0) && ([url rangeOfString:@"preferences"].length == 0)){
        // link to more results - must tease it out
        NSString *linkString = [NSString string];
        NSScanner *linkScanner = [NSScanner scannerWithString:url];
        [linkScanner scanUpToString:@"/**" intoString:nil];
        [linkScanner scanString:@"/**" intoString:nil];
        [linkScanner scanUpToString:@"" intoString:&linkString];
         
         
         NSString *imageSizeCondition;
         if([[_preferences objectForKey:@"image size"] intValue] == 0)
         imageSizeCondition = @"imgsz=";
         if([[_preferences objectForKey:@"image size"] intValue] == 1)
         imageSizeCondition = @"imgsz=wallpaper";
         if([[_preferences objectForKey:@"image size"] intValue] == 2)
         imageSizeCondition = @"imgsz=small";
         if([[_preferences objectForKey:@"image size"] intValue] == 3)
         imageSizeCondition = @"imgsz=medium";
         if([[_preferences objectForKey:@"image size"] intValue] == 4)
         imageSizeCondition = @"imgsz=large";
         //NSLog(@"preprocess. imageSizeCondition string: %@", imageSizeCondition);
         
         // coloration
         NSString *colorationCondition;
         if([[_preferences objectForKey:@"coloration"] intValue] == 0)
         colorationCondition = @"imgc=";
         if([[_preferences objectForKey:@"coloration"] intValue] == 1)
         colorationCondition = @"imgc=bw";
         if([[_preferences objectForKey:@"coloration"] intValue] == 2)
         colorationCondition = @"imgc=color";
         //NSLog(@"preprocess. colorationCondition string: %@", colorationCondition);
         
         // safesearch
         NSString *safeSearchCondition;
         if([[_preferences objectForKey:@"safe search"] intValue] == 0)
         safeSearchCondition = @"safe=off";
         if([[_preferences objectForKey:@"safe search"] intValue] == 1)
         safeSearchCondition = @"safe=images";
         if([[_preferences objectForKey:@"safe search"] intValue] == 2)
         safeSearchCondition = @"safe=active";
         //NSLog(@"preprocess. safeSearchCondition string: %@", safeSearchCondition);
         
         // site
         NSString *siteSearchCondition;
         if([_preferences objectForKey:@"site"]){
             siteSearchCondition = [NSString stringWithFormat:@"vs=%@", [_preferences objectForKey:@"site"]];
         } else {
             siteSearchCondition = @"vs=";
         }
         //NSLog(@"preprocess. siteSearchCondition string: %@", siteSearchCondition);
         
         NSString *linkURL = [NSString stringWithFormat:@"%@&%@&%@&%@", [[linkString stringByReplacingPercentEscapesUsingEncoding:NSUTF8StringEncoding] stringByReplacingPercentEscapesUsingEncoding:NSUTF8StringEncoding], imageSizeCondition, colorationCondition, siteSearchCondition, safeSearchCondition];
        
        return [NSDictionary dictionaryWithObjectsAndKeys:linkURL, @"source", nil];
    }
    
    // skip everything else
    return nil;
}

#pragma mark -
#pragma mark Authentication

- (NSArray *)authenticationInfo
{
    // the return array has two objects, an array of keys, and an array of corresponding values.
    // these keys and values are used to fill out a web form and submit it, or to 
    // provide to HTTP authentication.
    
    // For web forms, the keys are the names of the input elements that need to be filled out.
    // note that the keys and values are case sensitive!
    
    // if providing info for HTTP authentication, provide keys "username" and "password", in that order.
    
    // return [NSArray arrayWithObjects:[NSArray arrayWithObjects:@"username", @"password", nil], [NSArray arrayWithObjects:@"myUsername", @"myPassword", nil], nil];
    
    return nil;
}

- (BOOL)parseAuthSuccessPage
{
    return NO;
}

- (NSString *)authSuccessURLFragment
{
    return nil;
}

// Yahoo safe search pref setting
- (IBAction)setSafeSearchPref:(id)sender
{
    // launch Yahoo in web browser
    [[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:@"http://search.yahoo.com/preferences/preferences?page=filters"]];
}

@end
