//
//  FileController.m
//  NetScrape
//
//  Created by John Pannell on Sun Oct 24 2004.
//  Copyright (c) 2004 Positive Spin Media. All rights reserved.
//

#import "FileController.h"


@implementation FileController

#pragma mark -
#pragma mark Constructor/destructor

// designated initializer
- (id)initWithString:(NSString *)theString scrapeSubdirectories:(BOOL)subDirs
{
    if(self == [super init]){
        _objectController = [[NSObjectController alloc] initWithContent:self];
        fileNames = [[NSMutableArray alloc] init];
        _scrapedFiles = [[NSMutableArray alloc] init];
        myString = theString;
        _scrapeSubdirectories = subDirs;
        _isDone = NO;
        _userPaused = NO;
        _sleepPaused = NO;
    }
    return self;
}

#pragma mark -
#pragma mark Accessors

- (NSString *)rootString
{
    return myString;
}

- (NSString *)pluginName
{
    return @"Folder Contents";
}

- (ThumbnailMatrix *)thumbnailMatrix
{
    return myThumbnailMatrix;
}

- (void)setThumbnailMatrix:(ThumbnailMatrix *)matrix
{
    [matrix retain];
    [myThumbnailMatrix release];
    myThumbnailMatrix = matrix;
}

- (BOOL)isProcessing
{
    return _isProcessing;
}

- (void)setIsProcessing:(BOOL)value
{
    _isProcessing = value;
}

- (BOOL)isDone
{
    return _isDone;
}

- (void)setIsDone:(BOOL)flag
{
    _isDone = flag;
}

- (int)objectCount
{
    return [_scrapedFiles count];
}

- (void)setObjectCount:(int)value
{
    _objectCount = value;
}

- (NSImage *)icon
{
    return _icon;
}

- (void)setIcon:(NSImage *)icon
{
    [icon retain];
    [_icon release];
    _icon = icon;
}

- (NSMutableArray *)scrapedFiles
{
    return _scrapedFiles;
}

- (void)setScrapedFiles:(NSMutableArray *)array
{
    [array retain];
    [_scrapedFiles release];
    _scrapedFiles = array;
}

- (NSObjectController *)objectController
{
    return _objectController;
}

- (NSString *)myString
{
    return myString;
}

#pragma mark -
#pragma mark Session compatibility

- (void)pause:(id)sender
{
    _userPaused = YES;
    [_objectController setValue:[NSNumber numberWithBool:NO] forKeyPath:@"selection.isProcessing"];
}

- (void)resume:(id)sender
{
    _userPaused = NO;
    [_objectController setValue:[NSNumber numberWithBool:YES] forKeyPath:@"selection.isProcessing"];
    [self loadFile];
}

- (void)pauseEngine
{
    _sleepPaused = YES;
    [_objectController setValue:[NSNumber numberWithBool:NO] forKeyPath:@"selection.isProcessing"];
}

- (void)resumeEngine
{
    _sleepPaused = NO;
    if(!_userPaused){
        [_objectController setValue:[NSNumber numberWithBool:YES] forKeyPath:@"selection.isProcessing"];
        [self loadFile];
    }
}

- (void)stopEngine
{
    // normally would remove spider and release everything
    [self stopScript];
}

#pragma mark -
#pragma mark Functionality

- (void)startScript
{
    [fileNames removeAllObjects];
    [_objectController setValue:[NSNumber numberWithBool:YES] forKeyPath:@"selection.isProcessing"];
    [self getFileListAtPath:myString];
    [self loadFile];
}

- (void)stopScript
{
    // note that we are no longer running a task
    [_objectController setValue:[NSNumber numberWithBool:NO] forKeyPath:@"selection.isProcessing"];
}

- (void)getFileListAtPath:(NSString *)thePath
{
    NSFileManager *myFileManager = [NSFileManager defaultManager];
    BOOL isDirectory = NO;
    
    // Make sure thePath is really a readable directory
    if ([myFileManager fileExistsAtPath:thePath isDirectory:&isDirectory] && isDirectory)
    {
        NSDirectoryEnumerator *myDirectoryEnumerator = [myFileManager enumeratorAtPath:thePath];
        NSString *myFile = nil;
        
        while (myFile = [myDirectoryEnumerator nextObject]) 
        {
            NSString *myCompletePath = [NSString stringWithFormat:@"%@/%@", thePath, myFile];
            // subdirectories?
            if(!_scrapeSubdirectories){
                [myDirectoryEnumerator skipDescendents];
            }
            
            // We only care about bitmaps at this point
            if ([[NSBitmapImageRep imageUnfilteredFileTypes] containsObject:[myFile pathExtension]])
            {
                [fileNames addObject:myCompletePath];
            }
        }
    }
}

- (void)loadFile
{
    if(_userPaused || _sleepPaused)
        return;
    
    // create scrapedFile object for first item in array
    PSMWebFile *scrapedFile = [[PSMWebFile alloc] initWithSourceURL:[fileNames objectAtIndex:0]];
    [scrapedFile setStoragePath:[fileNames objectAtIndex:0]];
    [scrapedFile setParentWebPage:nil];
    
    // remove from list
    [fileNames removeObjectAtIndex:0];
    
    // load it up
    [NSThread detachNewThreadSelector:@selector(loadFileData:) toTarget:scrapedFile withObject:self];
}

- (void)addScrapedFile:(PSMWebFile *)object
{
    if(object){
        // keep it around
        [_scrapedFiles addObject:object];
        [_objectController setValue:[NSNumber numberWithInt:[self objectCount]+1] forKeyPath:@"selection.objectCount"];
        
        // add the image to the view
        if(myThumbnailMatrix){
            [myThumbnailMatrix addScrapedFile:object];
        }
    }
    
    // finished?
    if([fileNames count] == 0)
        [self setIsDone:YES];
    
    // if there are still more files, load another
    if(([fileNames count] > 0) && (_isProcessing)){
        [self loadFile];
    } else {
        [self stopScript];
    }
}

- (NSMutableArray *)fileNames
{
    // return the fileNames array for examination
    return fileNames;
}

@end
