//
//  PSMAppController.h
//  Pandora
//
//  Created by John Pannell on 12/10/05.
//  Copyright 2005 Positive Spin Media. All rights reserved.
//

/*
There is a single instance of this class, instantiated in the nib.  It is the delegate to the NSApp object (also hooked up in the nib file).
 
*/

#import <Cocoa/Cocoa.h>
#include <SystemConfiguration/SystemConfiguration.h>

@class PSMPaymentController;
@class PSMPreferenceController;
@class PSMRegistrationController;

@interface PSMAppController : NSObject 
{
    PSMPreferenceController         *preferenceController;  // singleton controller for pref window
    PSMPaymentController            *paymentController;     // singleton controller for payment and begging
    PSMRegistrationController       *registrationController; // for reg code entry
    IBOutlet    id                  sparkle;                // software update mechanism
    
    // windows and sessions
    NSMutableArray                  *_appWindowControllers;  // list of window controllers
    int                             _sessionID;
    BOOL                            _isSleeping;
    
    // about
    IBOutlet    NSWindow            *aboutPanel;            // gratuitous self-promotion
    IBOutlet    NSTextField         *aboutMessage;          // encouragement
    
    // log
    IBOutlet    NSWindow            *logWindow;
    NSMutableArray                  *_logArray;
    IBOutlet    NSArrayController   *logController;
    IBOutlet    NSArrayController   *filteredLog;
    NSMutableArray                  *_sessionIdenfiers;
    IBOutlet    NSArrayController   *sessionIDController;
    IBOutlet    NSArrayController   *statusIDController;
    IBOutlet    NSArrayController   *MIMEIDController;
    IBOutlet    NSPopUpButton       *sessionFilterButton;
    IBOutlet    NSPopUpButton       *statusFilterButton;
    IBOutlet    NSPopUpButton       *MIMEFilterButton;
    
    // plugins
    NSMutableArray                  *_pluginClasses;
    NSMenu                          *_pluginMenu;
    
    // crippling
    BOOL                            _isRegistered;
    int                             _singleLargeViewed;
    BOOL                            _canDropToFinder;
    
    // UI
    IBOutlet    NSMenu              *zoomMenu;
    IBOutlet    NSView              *openAccessoryView;
    IBOutlet    NSButton            *scrapeSubfolders;
}

// accessors
- (NSMutableArray *)pluginClasses;
- (void)setPluginClasses:(NSMutableArray *)classes;
- (NSMenu *)pluginMenu;
- (int)sessionID;
- (BOOL)isRegistered;
- (void)setIsRegistered:(BOOL)flag;
- (int)singleLargeViewed;
- (BOOL)canDropToFinder;
- (void)setCanDropToFinder:(BOOL)flag;
- (NSMutableArray *)appWindowControllers;
- (NSMutableArray *)sessionArray;
- (id)sparkle;
- (NSMutableArray *)logArray;
- (NSArrayController *)logController;
- (NSMutableArray *)sessionIdentifiers;
- (void)setSessionIdentifiers:(NSMutableArray *)array;
- (NSArrayController *)sessionIDController;
- (NSArrayController *)statusIDController;
- (NSArrayController *)MIMEIDController;
- (NSArrayController *)filteredLog;

// application delegate
- (void)applicationDidFinishLaunching:(NSNotification *)aNotification;
- (NSApplicationTerminateReply)applicationShouldTerminate:(NSApplication *)sender;
- (BOOL)applicationShouldTerminateAfterLastWindowClosed:(NSApplication *)sender;

// open files
- (BOOL)application:(NSApplication *)anApplication openFile:(NSString *)aFileName;
- (IBAction)openFiles:(id)sender;
- (void)finishOpenFiles:(NSOpenPanel *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInf;

// sleep/network
- (void)willSleep:(NSNotification *)aNotification;
- (void)didWakeFromSleep:(NSNotification *)aNotification;
- (void)networkStatusChanged:(int)flags;

// logging
- (IBAction)viewLog:(id)sender;
- (IBAction)logFilterAction:(id)sender;
- (IBAction)dumpLogToFile:(id)sender;

// app-wide functionality
- (void)loadPluginClasses;

// app-wide menu actions
- (IBAction)showPreferencePanel:(id)sender;
- (IBAction)purchaseOnline:(id)sender;
- (IBAction)enterRegistrationCode:(id)sender;
- (IBAction)newWindow:(id)sender;
- (IBAction)showAboutPanel:(id)sender;
- (IBAction)requestPlugin:(id)sender;

// other assistance
- (BOOL)validateMenuItem:(NSMenuItem*)anItem;
- (void)alertDidEnd:(NSAlert *)alert returnCode:(int)returnCode contextInfo:(void *)contextInfo;
- (void)paintAboutWindow;

// C functions
BOOL networkReachableWithoutAnythingSpecialHappening(void);
void NetworkStatusChanged(SCNetworkReachabilityRef target, SCNetworkConnectionFlags flags, void *info);

@end
