//
//  PSMAppController.m
//  Pandora
//
//  Created by John Pannell on 12/10/05.
//  Copyright 2005 Positive Spin Media. All rights reserved.
//

#import "PSMAppController.h"
#import "PSMPaymentController.h"
#import "PSMRegistrationController.h"
#import "PSMPreferenceController.h"
#import "PSMWindowController.h"
#import "PSMArrayHasSelectionValueTransformer.h"
#import "PandoraPlugin.h"

static float _NSDefaultScaleMenuFactors[] = {/* 0.0, */ 0.1, 0.25, 0.5, 0.75, 1.0, 1.28, 2.0, 4.0, 8.0, 16.0};

@implementation PSMAppController

+ (void)initialize
{
    // set my defaults up
    // create dictionary
    NSMutableDictionary *defaultValues = [NSMutableDictionary dictionary];
    
    // set values
    // reg info
    [defaultValues setObject:[NSNumber numberWithBool:NO] forKey:@"Pandora registration"];
    [defaultValues setObject:[NSString stringWithString:@""] forKey:@"Pandora code"];
    // MIME and size prefs
    [defaultValues setObject:[NSMutableArray arrayWithObjects:@"image/jpeg", @"image/gif", nil] forKey:@"MIME types"];
    [defaultValues setObject:[NSNumber numberWithBool:NO] forKey:@"Use minumum file size"];
    [defaultValues setObject:[NSNumber numberWithBool:NO] forKey:@"Use maximum file size"];
    [defaultValues setObject:[NSNumber numberWithInt:15] forKey:@"Minumum file size"];
    [defaultValues setObject:[NSNumber numberWithInt:1200] forKey:@"Maximum file size"];
    // storage options
    [defaultValues setObject:[NSNumber numberWithBool:NO] forKey:@"Use custom icons"];
    [defaultValues setObject:[NSNumber numberWithBool:NO] forKey:@"Enumerate file names"];
    [defaultValues setObject:[NSNumber numberWithBool:NO] forKey:@"Use custom image location"];
    [defaultValues setObject:[NSNumber numberWithBool:NO] forKey:@"Use custom audio location"];
    [defaultValues setObject:[NSNumber numberWithBool:NO] forKey:@"Use custom video location"];
    [defaultValues setObject:[NSNumber numberWithBool:NO] forKey:@"Use custom document location"];
    [defaultValues setObject:[NSSearchPathForDirectoriesInDomains(NSDesktopDirectory, NSUserDomainMask, YES) objectAtIndex:0] forKey:@"Default storage location"];
    [defaultValues setObject:[NSHomeDirectory() stringByAppendingString:@"/Pictures"] forKey:@"Default image location"];
    [defaultValues setObject:[NSHomeDirectory() stringByAppendingString:@"/Music"] forKey:@"Default audio location"];
    [defaultValues setObject:[NSHomeDirectory() stringByAppendingString:@"/Movies"] forKey:@"Default video location"];
    [defaultValues setObject:[NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0] forKey:@"Default document location"];
    // main view
    [defaultValues setObject:[NSNumber numberWithFloat:-1.0] forKey:@"Zoom scale"];
    // slideshow
    [defaultValues setObject:[NSNumber numberWithInt:0] forKey:@"Slideshow primary transition"];
    [defaultValues setObject:[NSNumber numberWithInt:0] forKey:@"Slideshow secondary transition"];
    [defaultValues setObject:[NSNumber numberWithFloat:30.0] forKey:@"Slideshow transition speed"];
    [defaultValues setObject:[NSNumber numberWithBool:NO] forKey:@"Slideshow full screen"];
    [defaultValues setObject:[NSNumber numberWithBool:NO] forKey:@"Slideshow randomize images"];
    [defaultValues setObject:[NSNumber numberWithBool:NO] forKey:@"Slideshow scale images"];
    [defaultValues setObject:[NSNumber numberWithBool:NO] forKey:@"Slideshow repeat show"];
    [defaultValues setObject:[NSNumber numberWithInt:4] forKey:@"Slideshow slide duration"];
    // babysitting
    [defaultValues setObject:[NSNumber numberWithBool:YES] forKey:@"Delete files warning"];
    // preview view
    [defaultValues setObject:[NSNumber numberWithInt:2] forKey:@"number of columns"];
    // network
    [defaultValues setObject:[NSString stringWithString:@"Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en-us) AppleWebKit/412 (KHTML, like Gecko) Pandora/2.0"] forKey:@"User agent"];
    [defaultValues setObject:[NSNumber numberWithBool:YES] forKey:@"SUCheckAtStartup"];
    [defaultValues setObject:[NSString stringWithString:@"PSMGoogleImageSearchPlugin"] forKey:@"Selected Plugin Name"];
    [defaultValues setObject:[NSNumber numberWithBool:NO] forKey:@"Delete files when finished"];
    [defaultValues setObject:[NSNumber numberWithBool:NO] forKey:@"Skip for width"];
    [defaultValues setObject:[NSNumber numberWithBool:NO] forKey:@"Skip for height"];
    [defaultValues setObject:[NSNumber numberWithInt:0] forKey:@"Skip width condition"];
    [defaultValues setObject:[NSNumber numberWithInt:0] forKey:@"Skip height condition"];
    [defaultValues setObject:[NSNumber numberWithInt:800] forKey:@"Skip width amount"];
    [defaultValues setObject:[NSNumber numberWithInt:600] forKey:@"Skip height amount"];
    [defaultValues setObject:[NSNumber numberWithBool:NO] forKey:@"Stop at limit"];
    [defaultValues setObject:[NSNumber numberWithInt:100] forKey:@"Stop limit amount"];
    
    
    // register the dictionary as defaults
    [[NSUserDefaults standardUserDefaults] registerDefaults:defaultValues];
    [[NSUserDefaults standardUserDefaults] synchronize];
    
    // write these prefs to persistent domain so spider can read them
    [[NSUserDefaults standardUserDefaults] setObject:[[NSUserDefaults standardUserDefaults] objectForKey:@"MIME types"] forKey:@"MIME types"];
    [[NSUserDefaults standardUserDefaults] setBool:[[NSUserDefaults standardUserDefaults] boolForKey:@"Use minumum file size"] forKey:@"Use minumum file size"];
    [[NSUserDefaults standardUserDefaults] setBool:[[NSUserDefaults standardUserDefaults] boolForKey:@"Use maximum file size"] forKey:@"Use maximum file size"];
    [[NSUserDefaults standardUserDefaults] setInteger:[[NSUserDefaults standardUserDefaults] integerForKey:@"Minumum file size"] forKey:@"Minumum file size"];
    [[NSUserDefaults standardUserDefaults] setInteger:[[NSUserDefaults standardUserDefaults] integerForKey:@"Maximum file size"] forKey:@"Maximum file size"];
    [[NSUserDefaults standardUserDefaults] setBool:[[NSUserDefaults standardUserDefaults] boolForKey:@"Use custom icons"] forKey:@"Use custom icons"];
    [[NSUserDefaults standardUserDefaults] setObject:[[NSUserDefaults standardUserDefaults] objectForKey:@"User agent"] forKey:@"User agent"];
    [[NSUserDefaults standardUserDefaults] setBool:[[NSUserDefaults standardUserDefaults] boolForKey:@"Use custom image location"] forKey:@"Use custom image location"];
    [[NSUserDefaults standardUserDefaults] setBool:[[NSUserDefaults standardUserDefaults] boolForKey:@"Use custom audio location"] forKey:@"Use custom audio location"];
    [[NSUserDefaults standardUserDefaults] setBool:[[NSUserDefaults standardUserDefaults] boolForKey:@"Use custom video location"] forKey:@"Use custom video location"];
    [[NSUserDefaults standardUserDefaults] setBool:[[NSUserDefaults standardUserDefaults] boolForKey:@"Use custom document location"] forKey:@"Use custom document location"];
    [[NSUserDefaults standardUserDefaults] setObject:[[NSUserDefaults standardUserDefaults] objectForKey:@"Default image location"] forKey:@"Default image location"];
    [[NSUserDefaults standardUserDefaults] setObject:[[NSUserDefaults standardUserDefaults] objectForKey:@"Default audio location"] forKey:@"Default audio location"];
    [[NSUserDefaults standardUserDefaults] setObject:[[NSUserDefaults standardUserDefaults] objectForKey:@"Default video location"] forKey:@"Default video location"];
    [[NSUserDefaults standardUserDefaults] setObject:[[NSUserDefaults standardUserDefaults] objectForKey:@"Default document location"] forKey:@"Default document location"];
    [[NSUserDefaults standardUserDefaults] setObject:[[NSUserDefaults standardUserDefaults] objectForKey:@"Default storage location"] forKey:@"Default storage location"];
    [[NSUserDefaults standardUserDefaults] setBool:[[NSUserDefaults standardUserDefaults] boolForKey:@"Delete files when finished"] forKey:@"Delete files when finished"];
    [[NSUserDefaults standardUserDefaults] setBool:[[NSUserDefaults standardUserDefaults] boolForKey:@"Skip for width"] forKey:@"Skip for width"];
    [[NSUserDefaults standardUserDefaults] setBool:[[NSUserDefaults standardUserDefaults] boolForKey:@"Skip for height"] forKey:@"Skip for height"];
    [[NSUserDefaults standardUserDefaults] setObject:[[NSUserDefaults standardUserDefaults] objectForKey:@"Skip width condition"] forKey:@"Skip width condition"];
    [[NSUserDefaults standardUserDefaults] setObject:[[NSUserDefaults standardUserDefaults] objectForKey:@"Skip height condition"] forKey:@"Skip height condition"];
    [[NSUserDefaults standardUserDefaults] setObject:[[NSUserDefaults standardUserDefaults] objectForKey:@"Skip width amount"] forKey:@"Skip width amount"];
    [[NSUserDefaults standardUserDefaults] setObject:[[NSUserDefaults standardUserDefaults] objectForKey:@"Skip height amount"] forKey:@"Skip height amount"];
    [[NSUserDefaults standardUserDefaults] setBool:[[NSUserDefaults standardUserDefaults] boolForKey:@"Stop at limit"] forKey:@"Stop at limit"];
    [[NSUserDefaults standardUserDefaults] setObject:[[NSUserDefaults standardUserDefaults] objectForKey:@"Stop limit amount"] forKey:@"Stop limit amount"];
    [[NSUserDefaults standardUserDefaults] setBool:[[NSUserDefaults standardUserDefaults] boolForKey:@"Enumerate file names"] forKey:@"Enumerate file names"];
    
    // Create and register value transformers
    NSValueTransformer *transformer = [[PSMArrayHasSelectionValueTransformer alloc] init];
    [NSValueTransformer setValueTransformer:transformer forName:@"ArrayHasSelectionValueTransformer"];
}

- (id)init
{
    if(self = [super init]){
        _appWindowControllers = [[NSMutableArray alloc] init];
        _pluginClasses = [[NSMutableArray alloc] init];
        _sessionID = 1;
        _singleLargeViewed = 0;
        _isRegistered = NO;
        _canDropToFinder = YES;
        _isSleeping = NO;
        _logArray = [[NSMutableArray alloc] init];
        
        // session IDs
        _sessionIdenfiers = [[NSMutableArray alloc] initWithObjects:[NSDictionary dictionaryWithObjectsAndKeys:@"All", @"name", @"-1", @"sessionID", nil], [NSDictionary dictionaryWithObjectsAndKeys:@"Pandora Status Console", @"name", @"0", @"sessionID", nil], nil];
    }
    return self;
}

- (void)awakeFromNib
{
    // populate the zoom menu
    int cnt;
    NSMenuItem *curItem;
    
    [[zoomMenu itemAtIndex:0] setRepresentedObject:[NSNumber numberWithFloat:-1.0]];
    if([[NSUserDefaults standardUserDefaults] floatForKey:@"Zoom scale"] == -1.0){
        [[zoomMenu itemAtIndex:0] setState:NSOnState];
    }
    for (cnt = 0; cnt < 10; cnt++) {
        curItem = [zoomMenu itemAtIndex:cnt + 2];
        if (_NSDefaultScaleMenuFactors[cnt] != 0.0) {
            [curItem setRepresentedObject:[NSNumber numberWithFloat:_NSDefaultScaleMenuFactors[cnt]]];
            if(_NSDefaultScaleMenuFactors[cnt] == [[NSUserDefaults standardUserDefaults] floatForKey:@"Zoom scale"]){
                [curItem setState:NSOnState];
            }
        }
    }
    
    // registration?
    NSArray *searchPaths = NSSearchPathForDirectoriesInDomains(NSApplicationSupportDirectory, NSLocalDomainMask | NSUserDomainMask, YES);
    NSEnumerator *e = [searchPaths objectEnumerator];
    NSString *specificPath;
    while(specificPath = [e nextObject]){
        NSString *machineLicenseFile = [specificPath stringByAppendingPathComponent:@"Pandora/.com.positivespinmedia.Pandora.license"];
        if([[NSFileManager defaultManager] fileExistsAtPath:machineLicenseFile]){
            NSDictionary *regDict = [NSDictionary dictionaryWithContentsOfFile:machineLicenseFile];
            [aboutMessage setStringValue:[NSString stringWithFormat:@"Registered to: %@ %@\n%@\nSupport? pandora@positivespinmedia.com", [regDict objectForKey:@"first name"], [regDict objectForKey:@"last name"], [regDict objectForKey:@"registration code"]]];
            _isRegistered = YES;
            _canDropToFinder = YES;
        }
    }
    
    if(_isRegistered)
        [[NSApp mainMenu] removeItem:[[NSApp mainMenu] itemWithTag:4]]; // purchase menu
}

#pragma mark -
#pragma mark Accessors

- (NSMutableArray *)pluginClasses
{
    return _pluginClasses;
}

- (void)setPluginClasses:(NSMutableArray *)classes
{
    [classes retain];
    [_pluginClasses release];
    _pluginClasses = classes;
}

- (NSMenu *)pluginMenu
{
    return _pluginMenu;
}

- (int)sessionID
{
    return _sessionID++;
}

- (BOOL)isRegistered
{
    return _isRegistered;
}

- (void)setIsRegistered:(BOOL)flag
{
    _isRegistered = flag;
}

- (int)singleLargeViewed
{
    return _singleLargeViewed++;
}

- (BOOL)canDropToFinder
{
    return _canDropToFinder;
}

- (void)setCanDropToFinder:(BOOL)flag
{
    _canDropToFinder = flag;
}

- (NSMutableArray *)appWindowControllers
{
    return _appWindowControllers;
}

- (NSMutableArray *)sessionArray
{
    // returns all sessions from all windows
    NSMutableArray *returnArray = [NSMutableArray array];
    NSEnumerator *e = [_appWindowControllers objectEnumerator];
    PSMWindowController *controller;
    while(controller = [e nextObject]){
        [returnArray addObjectsFromArray:[controller sessionArray]];
    }
    return returnArray;
}

- (id)sparkle
{
    return sparkle;
}

- (NSMutableArray *)logArray
{
    return _logArray;
}

- (NSArrayController *)logController;
{
    return logController;
}

- (NSMutableArray *)sessionIdentifiers
{
    return _sessionIdenfiers;
}

- (void)setSessionIdentifiers:(NSMutableArray *)array
{
    [array retain];
    [_sessionIdenfiers release];
    _sessionIdenfiers = array;
}

- (NSArrayController *)sessionIDController
{
    return sessionIDController;
}

- (NSArrayController *)statusIDController
{
    return statusIDController;
}

- (NSArrayController *)MIMEIDController
{
    return MIMEIDController;
}

- (NSArrayController *)filteredLog
{
    return filteredLog;
}

#pragma mark -
#pragma mark Application Delegate
- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
    // register for network notifications
    SCNetworkReachabilityRef target = SCNetworkReachabilityCreateWithName(NULL, "www.apple.com");
    SCNetworkReachabilityContext reachabilityContext = {
        .version         = 0,
        .info            = self,
        .retain          = CFRetain,
        .release         = CFRelease,
        .copyDescription = CFCopyDescription,
    };
    SCNetworkReachabilitySetCallback(target, NetworkStatusChanged, &reachabilityContext);
    SCNetworkReachabilityScheduleWithRunLoop(target, [[NSRunLoop currentRunLoop] getCFRunLoop], kCFRunLoopDefaultMode);
    
                                     
    // register for sleep notifications
    [[[NSWorkspace sharedWorkspace] notificationCenter] addObserver:self selector:@selector(willSleep:) name:NSWorkspaceWillSleepNotification object:[NSWorkspace sharedWorkspace]];
    [[[NSWorkspace sharedWorkspace] notificationCenter] addObserver:self selector:@selector(didWakeFromSleep:) name:NSWorkspaceDidWakeNotification object:[NSWorkspace sharedWorkspace]];
    
    // create application support folders if not already present
    NSFileManager *fm = [NSFileManager defaultManager];
    BOOL isDir;
    NSArray *supportFolders = NSSearchPathForDirectoriesInDomains(NSApplicationSupportDirectory, (NSLocalDomainMask | NSUserDomainMask),YES);
    NSEnumerator *e = [supportFolders objectEnumerator];
    NSString *path;
    while(path = [e nextObject]){
        if(!([fm fileExistsAtPath:[NSString stringWithFormat:@"%@/Pandora", path] isDirectory:&isDir] && isDir))
            [fm createDirectoryAtPath:[NSString stringWithFormat:@"%@/Pandora", path] attributes:nil];
        if(!([fm fileExistsAtPath:[NSString stringWithFormat:@"%@/Pandora/Plugins", path] isDirectory:&isDir] && isDir))
            [fm createDirectoryAtPath:[NSString stringWithFormat:@"%@/Pandora/Plugins", path] attributes:nil];
    }
    
    // load plugins
    [self loadPluginClasses];
    
    // build plugin selection menu
    _pluginMenu = [[NSMenu alloc] initWithTitle:@"Plugin menu"];
    NSMenuItem *nonItem = [[NSMenuItem alloc] initWithTitle:@"Never Seen" action:nil keyEquivalent:@""];
    [nonItem setState:NSOffState];
    NSMenuItem *spiderItem = [[NSMenuItem alloc] initWithTitle:@"Web Spiders" action:nil keyEquivalent:@""];
    [spiderItem setState:NSOffState];
    [spiderItem setImage:[NSImage imageNamed:@"spider"]];
    NSMenuItem *agentItem = [[NSMenuItem alloc] initWithTitle:@"Search Agents" action:nil keyEquivalent:@""];
    [agentItem setState:NSOffState];
    [agentItem setImage:[NSImage imageNamed:@"looking_glass"]];
    [_pluginMenu addItem:nonItem];
    [_pluginMenu addItem:spiderItem];
    [_pluginMenu addItem:agentItem];
    NSMenu *spiderMenu = [[NSMenu alloc] initWithTitle:@"Spiders"];
    NSMenu *agentMenu = [[NSMenu alloc] initWithTitle:@"Agents"];
    [spiderItem setSubmenu:spiderMenu];
    [agentItem setSubmenu:agentMenu];
    NSEnumerator *pe = [_pluginClasses objectEnumerator];
    Class<PandoraPlugin> specificClass;
    while(specificClass = [pe nextObject]){
        NSMenuItem *newItem = [[NSMenuItem alloc] initWithTitle:[specificClass name] action:@selector(selectPlugin:) keyEquivalent:@""];
        [newItem setTarget:self];
        [newItem setRepresentedObject:NSStringFromClass(specificClass)]; // represented object is STRING
        if([[specificClass category] isEqualToString:@"Spiders"])
            [spiderMenu addItem:newItem];
        if([[specificClass category] isEqualToString:@"Agents"])
            [agentMenu addItem:newItem];
    }
    
    // copy out needed resources if necessary
    NSString *mimePath = [NSString stringWithFormat:@"%@/Pandora/mime.plist", [NSSearchPathForDirectoriesInDomains(NSApplicationSupportDirectory, NSLocalDomainMask,YES) objectAtIndex:0]];
    if(![fm fileExistsAtPath:mimePath]){
        [fm copyPath:[[NSBundle mainBundle] pathForResource:@"mime" ofType:@"plist"] toPath:mimePath handler:nil];
    }
    NSString *mimePath2 = [NSString stringWithFormat:@"%@/Pandora/mime.plist", [NSSearchPathForDirectoriesInDomains(NSApplicationSupportDirectory, NSUserDomainMask,YES) objectAtIndex:0]];
    if(![fm fileExistsAtPath:mimePath2]){
        [fm copyPath:[[NSBundle mainBundle] pathForResource:@"mime" ofType:@"plist"] toPath:mimePath2 handler:nil];
    }
    
    // put up a window
    PSMWindowController *newWindow = [[PSMWindowController alloc] initWithWindowNibName:@"Window"];
    [_appWindowControllers addObject:newWindow];
    [newWindow showWindow:self];

}

- (NSApplicationTerminateReply)applicationShouldTerminate:(NSApplication *)sender
{
    if(_isRegistered){
        return NSTerminateNow;
    } else {
        // call to register
        if(!paymentController){
            paymentController = [[PSMPaymentController alloc] initWithWindowNibName:@"Payment"];
        }
        [paymentController window]; // weird - I need to call this to get the IBOutlets to load
        [paymentController beg:self];
        return NSTerminateLater;
    }
}

- (BOOL)applicationShouldTerminateAfterLastWindowClosed:(NSApplication *)sender
{
    return NO;
}

#pragma mark -
#pragma mark Open Files

- (BOOL)application:(NSApplication *)anApplication openFile:(NSString *)aFileName
{
    // start the preview process
    // recurse subdirectories by default
    if([[self appWindowControllers] count] < 1){
        // put up a window
        PSMWindowController *newWindow = [[PSMWindowController alloc] initWithWindowNibName:@"Window"];
        [_appWindowControllers addObject:newWindow];
        [newWindow showWindow:self];
    }
    
    [[[NSApp keyWindow] delegate] performFileScrapeInDirectory:aFileName scrapingSubdirectories:YES];
    return YES;
}

- (IBAction)openFiles:(id)sender
{
    if([[self appWindowControllers] count] < 1){
        // put up a window
        PSMWindowController *newWindow = [[PSMWindowController alloc] initWithWindowNibName:@"Window"];
        [_appWindowControllers addObject:newWindow];
        [newWindow showWindow:self];
    }
    
    // put up open dialog with accessory view (to do subfolders or not)
    NSOpenPanel *op = [NSOpenPanel openPanel];
    [op setCanChooseDirectories:YES];
    [op setCanChooseFiles:NO];
    [op setAllowsMultipleSelection:NO];
    [op setAccessoryView:openAccessoryView];
    
    [op beginSheetForDirectory:nil file:nil types:nil modalForWindow:[NSApp keyWindow] modalDelegate:self didEndSelector:@selector(finishOpenFiles:returnCode:contextInfo:) contextInfo:[NSApp keyWindow]];
}

- (void)finishOpenFiles:(NSOpenPanel *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInf
{
    if (returnCode == NSOKButton) {
        // start the preview process
        NSWindow *window = contextInf;
        NSString *chosenDirectory = [[sheet filenames] objectAtIndex:0];
        BOOL subDirs = [scrapeSubfolders state];
        [[window delegate] performFileScrapeInDirectory:chosenDirectory scrapingSubdirectories:subDirs];
    }
}


#pragma mark -
#pragma mark Sleep/Network

- (void)willSleep:(NSNotification *)aNotification
{
    _isSleeping = YES;
    [[self sessionArray] makeObjectsPerformSelector:@selector(pauseEngine)];
}

- (void)didWakeFromSleep:(NSNotification *)aNotification
{
    _isSleeping = NO;
    [[self sessionArray] makeObjectsPerformSelector:@selector(resumeEngine)];
}

- (void)networkStatusChanged:(int)flags
{
    /*
    NSLog(@"*** hostReachabilityChangedCallback got flags: %c%c%c%c%c%c%c \n", 
          (flags & kSCNetworkFlagsTransientConnection)  ? 't' : '-', 
          (flags & kSCNetworkFlagsReachable)            ? 'r' : '-', 
          (flags & kSCNetworkFlagsConnectionRequired)   ? 'c' : '-', 
          (flags & kSCNetworkFlagsConnectionAutomatic)  ? 'C' : '-', 
          (flags & kSCNetworkFlagsInterventionRequired) ? 'i' : '-', 
          (flags & kSCNetworkFlagsIsLocalAddress)       ? 'l' : '-', 
          (flags & kSCNetworkFlagsIsDirect)             ? 'd' : '-');
     */
    if(!_isSleeping){   // don't mess with this on wake from sleep!
        if((flags & kSCNetworkFlagsConnectionRequired) || (flags & kSCNetworkFlagsInterventionRequired)){
            // trouble - notify and pause
            [[self sessionArray] makeObjectsPerformSelector:@selector(pauseEngine)];
            NSAlert *networkAlert = [NSAlert alertWithMessageText:@"Network Issue" defaultButton:@"OK" alternateButton:nil otherButton:nil informativeTextWithFormat:@"Pandora can no longer connect to the Internet for some reason.  Please check your network settings and status.  Pandora will pause all processing for the time being, and automatically restart all processes when the network comes back online."];
            [networkAlert beginSheetModalForWindow:[NSApp keyWindow] modalDelegate:self didEndSelector:nil contextInfo:nil];
        } else {
            [[self sessionArray] makeObjectsPerformSelector:@selector(resumeEngine)];
        }
    }
}

#pragma mark -
#pragma mark Logging

- (IBAction)viewLog:(id)sender
{
    [logWindow center];
    [logWindow makeKeyAndOrderFront:self];
}

- (IBAction)logFilterAction:(id)sender
{
    // assemble format string for predicate
    NSString *formatString = [NSString stringWithString:@""];
    if([[[sessionFilterButton selectedItem] representedObject] intValue] != -1)
        formatString = [formatString stringByAppendingString:[NSString stringWithFormat:@"(sessionID == '%@') AND ", [[sessionFilterButton selectedItem] representedObject]]];
    
    if(![[MIMEFilterButton titleOfSelectedItem] isEqualToString:@"Any"])
         formatString = [formatString stringByAppendingString:[NSString stringWithFormat:@"(resourceMIME LIKE '%@') AND ", [MIMEFilterButton titleOfSelectedItem]]];
    
    if(![[statusFilterButton titleOfSelectedItem] isEqualToString:@"Any"])
        formatString = [formatString stringByAppendingString:[NSString stringWithFormat:@"(resourceStatus LIKE '%@') AND ", [statusFilterButton titleOfSelectedItem]]];
    
    // trim off last " AND "
    if([formatString length] > 5)
        formatString = [formatString substringToIndex:([formatString length] - 5)];
    
    if([formatString length] < 1){
        [filteredLog setFilterPredicate:nil];
    } else {
        [filteredLog setFilterPredicate:[NSPredicate predicateWithFormat:formatString]];
    }
}

- (IBAction)dumpLogToFile:(id)sender
{
    NSSavePanel *sp = [NSSavePanel savePanel];
    [sp beginSheetForDirectory:nil file:@"PandoraLog.xml" modalForWindow:[NSApp keyWindow] modalDelegate:self didEndSelector:@selector(saveLogDidEnd:returnCode:contextInfo:) contextInfo:nil];
}

- (void)saveLogDidEnd:(NSSavePanel *)sheet returnCode:(int)returnCode contextInfo:(void *)contentInfo
{
    if(returnCode == NSOKButton){
        if([[NSFileManager defaultManager] fileExistsAtPath:[sheet filename]])
            [[NSFileManager defaultManager] removeFileAtPath:[sheet filename] handler:nil];
        [_logArray writeToFile:[sheet filename] atomically:YES];
    }
}

#pragma mark -
#pragma mark App-wide Menu Actions
- (IBAction)showPreferencePanel:(id)sender
{
    if(!preferenceController){
        preferenceController = [[PSMPreferenceController alloc] initWithWindowNibName:@"Preferences"];
    }
    [preferenceController showWindow:self];
}

- (IBAction)purchaseOnline:(id)sender
{
    if(!paymentController){
        paymentController = [[PSMPaymentController alloc] initWithWindowNibName:@"Payment"];
    }
    [paymentController window]; // weird - I need to call this to get the IBOutlets to load
    [paymentController makePayment:self];
}

- (IBAction)enterRegistrationCode:(id)sender
{
    if(!registrationController){
        registrationController = [[PSMRegistrationController alloc] initWithWindowNibName:@"Registration"];
    }
    [registrationController window]; // weird - I need to call this to get the IBOutlets to load
    [registrationController runRegPanel:self];
}

- (IBAction)newWindow:(id)sender
{
    // put up a window
    PSMWindowController *newWindow = [[PSMWindowController alloc] initWithWindowNibName:@"Window"];
    [newWindow showWindow:self];
    [_appWindowControllers addObject:newWindow]; 
}

- (IBAction)showAboutPanel:(id)sender
{
    if([[NSUserDefaults standardUserDefaults] boolForKey:@"Pandora registration"]){
        [aboutMessage setStringValue:@"Thanks for registering!\nhttp://www.positivespinmedia.com"];
    }
    [self paintAboutWindow];
    [aboutPanel center];
    [aboutPanel makeKeyAndOrderFront:self];
}

- (void)requestPlugin:(id)sender
{
    [[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:@"http://www.positivespinmedia.com/shareware/Pandora/p2request_plugin.php"]];
}

#pragma mark -
#pragma mark Plugins

-(void)loadPluginClasses
{
    // start internal search
    NSFileManager *fm = [NSFileManager defaultManager];
    NSBundle *appBundle = [NSBundle mainBundle];
    NSString *pluginPath = [appBundle builtInPlugInsPath];
    NSArray *pluginList = [fm directoryContentsAtPath:pluginPath];
    NSEnumerator *e = [pluginList objectEnumerator];
    NSString *specificPlugin;
    while(specificPlugin = [e nextObject]){
        NSString *fullPath = [pluginPath stringByAppendingPathComponent:specificPlugin];
        // load
        NSBundle *bundle = [NSBundle bundleWithPath:fullPath];
        Class principalClass = [bundle principalClass];
        //NSLog(@"%@", bundle);
        // store
        [_pluginClasses addObject:principalClass];
    }
    
    // machine domain & user domain support folders
    NSArray *supportFolders = NSSearchPathForDirectoriesInDomains(NSApplicationSupportDirectory, (NSLocalDomainMask | NSUserDomainMask),YES);
    e = [supportFolders reverseObjectEnumerator];
    NSString *path;
    while(path = [e nextObject]){
        pluginPath = [NSString stringWithFormat:@"%@/Pandora/Plugins", path];
        pluginList = [fm directoryContentsAtPath:pluginPath];
        NSEnumerator *pe = [pluginList objectEnumerator];
        while(specificPlugin = [pe nextObject]){
            if([[specificPlugin pathExtension] isEqualToString:@"plugin"]){
                NSString *fullPath = [pluginPath stringByAppendingPathComponent:specificPlugin];
                // load
                NSBundle *bundle = [NSBundle bundleWithPath:fullPath];
                Class principalClass = [bundle principalClass];
                // store
                if([_pluginClasses containsObject:principalClass])
                    [_pluginClasses removeObject:principalClass];
                [_pluginClasses addObject:principalClass];
            }
        }
        
    }
}

- (void)selectPlugin:(id)sender
{
    [[NSUserDefaults standardUserDefaults] setObject:[sender representedObject] forKey:@"Selected Plugin Name"];
    NSString *prompt = [NSString stringWithFormat:@"%@ (%@)", [NSClassFromString([[NSUserDefaults standardUserDefaults] objectForKey:@"Selected Plugin Name"]) prompt], [NSClassFromString([[NSUserDefaults standardUserDefaults] objectForKey:@"Selected Plugin Name"]) name]];
    [_appWindowControllers makeObjectsPerformSelector:@selector(setPluginPlaceholder:) withObject:prompt];
}

#pragma mark -
#pragma mark Other Assistance

- (BOOL)validateMenuItem:(NSMenuItem*)anItem
{
    if ([anItem action] == @selector(openFiles:)){
        return YES;
    }
    if ([anItem action] == @selector(newWindow:)){
        return YES;
    }
    if ([anItem action] == @selector(showAboutPanel:)){
        return YES;
    }
    
    if ([anItem action] == @selector(requestPlugin:)){
        return YES;
    }
    
    if ([anItem action] == @selector(showPreferencePanel:)){
        return YES;
    }
    if ([anItem action] == @selector(viewLog:)){
        return YES;
    }
    if ([anItem action] == @selector(purchaseOnline:)){
        return YES;
    }
    if ([anItem action] == @selector(enterRegistrationCode:)){
        return YES;
    }
    if ([anItem action] == @selector(logFilterAction:)){
        return YES;
    }
    if ([anItem action] == @selector(selectPlugin:)){
        if([[anItem representedObject] isEqualToString:[[NSUserDefaults standardUserDefaults] objectForKey:@"Selected Plugin Name"]]){
            [anItem setState:NSOnState];
        } else {
            [anItem setState:NSOffState];
        }
        return YES;
    }
    return NO;
}

- (void)alertDidEnd:(NSAlert *)alert returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
    [NSApp stopModal];
}

- (void)paintAboutWindow
{
    NSImage *bg = [[NSImage alloc] initWithSize:[aboutPanel frame].size];
    
    [bg lockFocus];
    
    // Composite current background color into bg
    //[bgColor set];
    //NSRectFill(NSMakeRect(0, 0, [bg size].width, [bg size].height));
    
    // Composite top "pattern"
    NSImage *bgImage = [NSImage imageNamed:@"about_bg"];
    NSColor *bgColor = [NSColor colorWithPatternImage:bgImage];
    [bgColor set];
    [[NSGraphicsContext currentContext] setPatternPhase:NSMakePoint([bg size].width, [bg size].height)];
    NSRectFill(NSMakeRect(0, 0, [bg size].width, [bg size].height));
    
    [bg unlockFocus];
    
    [aboutPanel setBackgroundColor:[NSColor colorWithPatternImage:bg]];
    
    [bg release];
}

#pragma mark -
#pragma mark C Functions
BOOL networkReachableWithoutAnythingSpecialHappening(void)
{ 
    Boolean Success; 
    SCNetworkConnectionFlags ReachabilityStatus; 
    Success = SCNetworkCheckReachabilityByName("www.positivespinmedia.com", 
                                               &ReachabilityStatus); 
    return (Success && (ReachabilityStatus & 3)); 
}

void NetworkStatusChanged(SCNetworkReachabilityRef target, SCNetworkConnectionFlags flags, void *info)
{
    PSMAppController *self = info;
    [self networkStatusChanged:flags];
}

@end
