//
//  PSMMIMEAssistant.m
//  Pandora
//
//  Created by John Pannell on 5/2/06.
//  Copyright 2006 Positive Spin Media. All rights reserved.
//

#import "PSMMIMEAssistant.h"
#import "PSMPreferenceController.h"

@implementation PSMMIMEAssistant

#pragma mark -
#pragma mark Creation/Destruction

- (id)init
{
    if(self = [super init]){
        // get MIME information from file
        _mimeList = [[NSMutableArray alloc] initWithContentsOfFile:[NSString stringWithFormat:@"%@/Pandora/mime.plist", [NSSearchPathForDirectoriesInDomains(NSApplicationSupportDirectory, NSLocalDomainMask,YES) objectAtIndex:0]]];
        if(!_mimeList)
            _mimeList = [[NSMutableArray alloc] initWithContentsOfFile:[NSString stringWithFormat:@"%@/Pandora/mime.plist", [NSSearchPathForDirectoriesInDomains(NSApplicationSupportDirectory, NSUserDomainMask,YES) objectAtIndex:0]]];
    }
    return self;
}

- (void)awakeFromNib
{
    [outlineView setAutoresizesOutlineColumn:NO];
    
    // replace mime list storage labels with localized versions
    NSArray *specialDirectories = [prefController specialDirectories];
    NSEnumerator *e = [specialDirectories objectEnumerator];
    NSDictionary *listing;
    while(listing = [e nextObject]){
        NSEnumerator *m = [_mimeList objectEnumerator];
        NSMutableDictionary *item;
        while(item = [m nextObject]){
            if([[item objectForKey:@"description"] isEqualToString:[[listing objectForKey:@"path"] lastPathComponent]]){
                // got it - rewrite
                [item setObject:[listing objectForKey:@"displayName"] forKey:@"displayName"];
            }
        }
    }
}

#pragma mark -
#pragma mark Accessors

- (NSMutableArray *)mimeList
{
    return _mimeList;
}

- (NSSet *)defaultValues
{
    // return set for all selected MIME types
    NSMutableSet *returnSet = [NSMutableSet set];
    NSEnumerator *e = [_mimeList objectEnumerator];
    NSMutableDictionary *parent;
    while(parent = [e nextObject]){
        NSArray *childArray = [parent objectForKey:@"children"];
        NSEnumerator *ce = [childArray objectEnumerator];
        NSMutableDictionary *child;
        while(child = [ce nextObject]){
            if([[child objectForKey:@"selected"] intValue] == NSOnState){
                [returnSet addObject:[child objectForKey:@"MIME type"]];
            }
        }
    }
    return returnSet;
}

- (void)setDefaultValues:(NSSet *)values
{
    [values retain];
    // load state based on MIME types in set
    NSEnumerator *e = [_mimeList objectEnumerator];
    NSMutableDictionary *parent;
    while(parent = [e nextObject]){
        NSArray *childArray = [parent objectForKey:@"children"];
        NSEnumerator *ce = [childArray objectEnumerator];
        NSMutableDictionary *child;
        while(child = [ce nextObject]){
            if([values containsObject:[child objectForKey:@"MIME type"]]){
                [child setObject:[NSNumber numberWithInt:NSOnState] forKey:@"selected"];
            } else {
                [child setObject:[NSNumber numberWithInt:NSOffState] forKey:@"selected"];
            }
        }
        [self setStateForParent:[parent objectForKey:@"description"]];
    }
    [mimeController rearrangeObjects];
    [values release];
}

#pragma mark -
#pragma mark Outline View Datasource
// bogus implementations because binding is populating the view

- (id)outlineView:(NSOutlineView *)outlineView child:(int)index ofItem:(id)item
{ 
    return nil;
}

- (BOOL)outlineView:(NSOutlineView *)outlineView isItemExpandable:(id)item
{ 
    return NO;
}

- (int)outlineView:(NSOutlineView *)outlineView numberOfChildrenOfItem:(id)item
{
    return 0;
}

- (id)outlineView:(NSOutlineView *)outlineView objectValueForTableColumn:(NSTableColumn *)tableColumn byItem:(id)item
{
    return nil;
}

#pragma mark -
#pragma mark Outline View Delegate

- (void)outlineView:(NSOutlineView *)outlineView willDisplayCell:(id)cell forTableColumn:(NSTableColumn *)tableColumn item:(id)item
{
    // re-write the title of the checkbox
    if([[tableColumn identifier] isEqualToString:@"File Type"]){
        [cell setTitle:[[item observedObject] objectForKey:@"displayName"]];
        if([[item observedObject] objectForKey:@"children"]){
            [cell setTarget:self];
            [cell setAction:@selector(parentButtonClick:)];
        } else {
            [cell setTarget:self];
            [cell setAction:@selector(childButtonClick:)];
        }
    }
}

#pragma mark -
#pragma mark Button state management

- (void)childButtonClick:(id)sender
{
    // user clicks cannot have mixed state
    if([[sender selectedCell] state] == NSMixedState){
        [[sender selectedCell] performClick:sender];
        return;
    }
    
    // binding handles its state - fix parent state
    [self setStateForParent:[[[sender itemAtRow:[sender selectedRow]] observedObject] objectForKey:@"parent"]];
    
    // note the new pref
    if([[sender selectedCell] state] == NSOnState){
        [prefController addMIME:[[[sender itemAtRow:[sender selectedRow]] observedObject] objectForKey:@"MIME type"]];
    } else {
        [prefController removeMIME:[[[sender itemAtRow:[sender selectedRow]] observedObject] objectForKey:@"MIME type"]];
    }
}

- (void)parentButtonClick:(id)sender
{
    // user clicks cannot have mixed state
    int state = [[sender selectedCell] state];
    if(state == NSMixedState){
        [[sender selectedCell] performClick:sender];
        return;
    }
    
    // set all children to same state
    NSArray *childArray = [[[sender itemAtRow:[sender selectedRow]] observedObject] objectForKey:@"children"];
    NSEnumerator *e = [childArray objectEnumerator];
    NSMutableDictionary *child;
    while(child = [e nextObject]){
        [child setValue:[NSNumber numberWithInt:state] forKey:@"selected"];
        if(state == NSOnState){
            [prefController addMIME:[child objectForKey:@"MIME type"]];
        } else {
            [prefController removeMIME:[child objectForKey:@"MIME type"]];
        }
    }
    
    // refresh controller?
    [mimeController rearrangeObjects];
}

- (void)setStateForParent:(NSString *)parentDescription
{
    int state = NSOffState;
    
    // find parent and examine children
    NSEnumerator *e = [_mimeList objectEnumerator];
    NSMutableDictionary *parent;
    while(parent = [e nextObject]){
        if([[parent valueForKey:@"description"] isEqualToString:parentDescription])
            break;
    }
    
    NSArray *childArray = [parent objectForKey:@"children"];
    NSEnumerator *ce = [childArray objectEnumerator];
    NSMutableDictionary *child;
    int misses = 0;
    while(child = [ce nextObject]){
        if([[child objectForKey:@"selected"] intValue] == NSOnState){
            state = NSMixedState;
        } else {
            misses++;
        }
    }
    
    if(misses == 0)
        state = NSOnState;
    
    [parent setValue:[NSNumber numberWithInt:state] forKey:@"selected"];
    
    // refresh
    [mimeController rearrangeObjects];
    
}


@end
