//
//  PSMPreferenceController.h
//  Pandora
//
//  Created by John Pannell on 12/10/05.
//  Copyright 2005 Positive Spin Media. All rights reserved.
//

/*
Singleton object, controls the prefs window and is delegate for it (hooked up in nib). 
 
*/

#import <Cocoa/Cocoa.h>
@class PSMGLSlideShowView;
@class PSMMIMEAssistant;

@interface PSMPreferenceController : NSWindowController {
    IBOutlet PSMMIMEAssistant       *mimeAssistant;
    NSToolbar                       *toolbar;
    IBOutlet NSView                 *filesView;
    IBOutlet NSView                 *pluginsView;
    IBOutlet NSView                 *storageView;
    IBOutlet NSView                 *slideshowView;
    IBOutlet NSView                 *networkView;
    NSDictionary                    *_viewList;
    
    // storage
    NSMutableArray                  *_specialDirectories;
    
    IBOutlet NSPopUpButton          *defaultStorage;
    IBOutlet NSPopUpButton          *imageStorage;
    IBOutlet NSPopUpButton          *audioStorage;
    IBOutlet NSPopUpButton          *videoStorage;
    IBOutlet NSPopUpButton          *documentStorage;
    
    // plugins
    IBOutlet NSArrayController      *pluginController;
    
    // slideshow
    IBOutlet PSMGLSlideShowView     *glSlideshowView;
    
    // network
    NSMutableArray                  *_userAgentList;
    IBOutlet NSPopUpButton          *periodicUpdateFrequency;
    
    // first responders (for keyboard navigation)
    IBOutlet NSOutlineView          *fileTypesFirstResponder;
    IBOutlet NSTableView            *pluginsFirstResponder;
    IBOutlet NSButton               *storageFirstResponder;
    IBOutlet NSButton               *slideshowFirstResponder;
    IBOutlet NSButton               *networkFirstResponder;
    
    // outlets to textfields needed commitment for bindings
    IBOutlet NSTextField            *a;
    IBOutlet NSTextField            *b;
    IBOutlet NSTextField            *c;
    IBOutlet NSTextField            *d;
    IBOutlet NSTextField            *e;
    IBOutlet NSTextField            *f;
    
}

// accessors
- (NSMutableArray *)specialDirectories;
- (NSMutableArray *)userAgentList;
- (void)setUserAgentList:(NSMutableArray *)list;

// MIME type preference
- (void)addMIME:(NSString *)type;
- (void)removeMIME:(NSString *)type;

// plugin preferences
- (IBAction)configurePlugin:(id)sender;

// storage preferences
- (void)populateStoragePopUp:(NSPopUpButton *)button withDefault:(NSString *)defaultKey;
- (void)storageAction:(id)sender;

// slideshow preferences
- (IBAction)slideshowChange:(id)sender;

    // network preferences
- (IBAction)periodicUpdatePrefChanged:(id)sender;

// toolbar
- (void)selectPreferencePane:(id)sender;
- (NSToolbarItem *)toolbar:(NSToolbar *)toolbar itemForItemIdentifier:(NSString *)itemIdentifier willBeInsertedIntoToolbar:(BOOL)flag;
- (NSArray *)toolbarDefaultItemIdentifiers:(NSToolbar*)toolbar;
- (NSArray *)toolbarAllowedItemIdentifiers:(NSToolbar*)toolbar;
- (IBAction)customizeToolbar:(id)sender;
- (IBAction)toggleToolbar:(id)sender;
- (BOOL)validateToolbarItem:(NSToolbarItem *)theItem;

@end
