//
//  PSMPreferenceController.m
//  Pandora
//
//  Created by John Pannell on 12/10/05.
//  Copyright 2005 Positive Spin Media. All rights reserved.
//

#import "PSMPreferenceController.h"
#import "PSMMIMEAssistant.h"
#import "PSMAppController.h"
#import "PSMGLSlideShowView.h"
#import <Sparkle/SUUpdater.h>

@implementation PSMPreferenceController

#pragma mark -
#pragma mark Creation/Destruction

- (id)initWithWindowNibName:(NSString *)name
{
    if(self = [super initWithWindowNibName:name]){
        _specialDirectories = [[NSMutableArray alloc] init];
        _userAgentList = [[NSMutableArray alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"user_agents" ofType:@"plist"]];
        
        // desktop
        NSString *desktopPath = [NSSearchPathForDirectoriesInDomains(NSDesktopDirectory, NSUserDomainMask, YES) objectAtIndex:0];
        NSString *desktopDisplayName = [[NSFileManager defaultManager] displayNameAtPath:desktopPath];
        NSDictionary *desktopDict = [NSDictionary dictionaryWithObjectsAndKeys:desktopPath, @"path", desktopDisplayName, @"displayName", nil];
        [_specialDirectories addObject:desktopDict];
        
        // documents
        NSString *docPath = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];
        NSString *docDisplayName = [[NSFileManager defaultManager] displayNameAtPath:docPath];
        NSDictionary *docDict = [NSDictionary dictionaryWithObjectsAndKeys:docPath, @"path", docDisplayName, @"displayName", nil];
        [_specialDirectories addObject:docDict];
        
        // movies
        NSString *moviePath = [NSHomeDirectory() stringByAppendingString:@"/Movies"];
        NSString *movieDisplayName = [[NSFileManager defaultManager] displayNameAtPath:moviePath];
        NSDictionary *movieDict = [NSDictionary dictionaryWithObjectsAndKeys:moviePath, @"path", movieDisplayName, @"displayName", nil];
        [_specialDirectories addObject:movieDict];
        
        // music
        NSString *musicPath = [NSHomeDirectory() stringByAppendingString:@"/Music"];
        NSString *musicDisplayName = [[NSFileManager defaultManager] displayNameAtPath:musicPath];
        NSDictionary *musicDict = [NSDictionary dictionaryWithObjectsAndKeys:musicPath, @"path", musicDisplayName, @"displayName", nil];
        [_specialDirectories addObject:musicDict];
        
        // pictures
        NSString *picturePath = [NSHomeDirectory() stringByAppendingString:@"/Pictures"];
        NSString *pictureDisplayName = [[NSFileManager defaultManager] displayNameAtPath:picturePath];
        NSDictionary *pictureDict = [NSDictionary dictionaryWithObjectsAndKeys:picturePath, @"path", pictureDisplayName, @"displayName", nil];
        [_specialDirectories addObject:pictureDict];
    }
    
    return self;
}
        

- (void)awakeFromNib
{
    // populate MIME types from preferences
    [mimeAssistant setDefaultValues:[NSSet setWithArray:[[NSUserDefaults standardUserDefaults] objectForKey:@"MIME types"]]];

    // build view list
    _viewList = [[NSDictionary alloc] initWithObjectsAndKeys:filesView, @"FilesItem", pluginsView, @"PlugInsItem", storageView, @"StorageItem", slideshowView, @"SlideshowItem", networkView, @"NetworkItem", nil];
    [[self window] setAutorecalculatesKeyViewLoop:YES];
    
    // build default storage pop up menu
    [self populateStoragePopUp:defaultStorage withDefault:@"Default storage location"];
    [self populateStoragePopUp:imageStorage withDefault:@"Default image location"];
    [self populateStoragePopUp:audioStorage withDefault:@"Default audio location"];
    [self populateStoragePopUp:videoStorage withDefault:@"Default video location"];
    [self populateStoragePopUp:documentStorage withDefault:@"Default document location"];
    
    // update frequency menu
    [[periodicUpdateFrequency itemAtIndex:0] setRepresentedObject:[NSNumber numberWithInt:(60*60*24)]];
    [[periodicUpdateFrequency itemAtIndex:1] setRepresentedObject:[NSNumber numberWithInt:(60*60*24*7)]];
    [[periodicUpdateFrequency itemAtIndex:2] setRepresentedObject:[NSNumber numberWithInt:(60*60*24*7*30)]];
    NSEnumerator *en = [[periodicUpdateFrequency itemArray] objectEnumerator];
    NSMenuItem *item;
    while(item = [en nextObject]){
        if([[item representedObject] intValue] == [[[NSUserDefaults standardUserDefaults] objectForKey:@"SUScheduledCheckInterval"] intValue]){
            [periodicUpdateFrequency selectItem:item];
        }
    }
}

- (void)windowDidLoad
{
    [super windowDidLoad];
    
    // toolbar
    toolbar = [[NSToolbar alloc] initWithIdentifier:@"PandoraCPreferenceToolbar"];
    [toolbar setDelegate:self];
    [toolbar setAllowsUserCustomization:NO];
    [toolbar setAutosavesConfiguration:YES];
    [[self window] setToolbar:toolbar];
    [toolbar setSelectedItemIdentifier:@"FilesItem"];
    [[self window] setContentView:filesView];
    [[self window] setTitle:[[[toolbar items] objectAtIndex:0] label]];
}

#pragma mark -
#pragma mark Accessors

- (NSMutableArray *)specialDirectories
{
    return _specialDirectories;
}

- (NSMutableArray *)userAgentList
{
    return _userAgentList;
}

- (void)setUserAgentList:(NSMutableArray *)list
{
    [list retain];
    [_userAgentList release];
    _userAgentList = list;
}

#pragma mark -
#pragma mark MIME Preferences

- (void)addMIME:(NSString *)type
{
    if(![[[NSUserDefaults standardUserDefaults] objectForKey:@"MIME types"] containsObject:type]){
        NSMutableArray *temp = [[[NSUserDefaults standardUserDefaults] objectForKey:@"MIME types"] mutableCopy];
        [temp addObject:type];
        [[NSUserDefaults standardUserDefaults] setObject:temp forKey:@"MIME types"];
    }
}

- (void)removeMIME:(NSString *)type
{
    if([[[NSUserDefaults standardUserDefaults] objectForKey:@"MIME types"] containsObject:type]){
        NSMutableArray *temp = [[[NSUserDefaults standardUserDefaults] objectForKey:@"MIME types"] mutableCopy];
        [temp removeObject:type];
        [[NSUserDefaults standardUserDefaults] setObject:temp forKey:@"MIME types"];
    }
}

#pragma mark -
#pragma mark Plugin Preferences

- (IBAction)configurePlugin:(id)sender
{
    // instantiate selected plugin
    id plugin = [[[[pluginController selectedObjects] objectAtIndex:0] alloc] init];
    [NSApp beginSheet: [plugin window]
       modalForWindow: [NSApp mainWindow]
        modalDelegate: nil
       didEndSelector: nil
          contextInfo: nil];
    [NSApp runModalForWindow:[plugin window]];
    // Sheet is up here.
}

#pragma mark -
#pragma mark Storage Preferences

- (void)populateStoragePopUp:(NSPopUpButton *)button withDefault:(NSString *)defaultKey
{
    // build default storage pop up menu
    [button removeAllItems];
    [button setTarget:self];
    NSEnumerator *esd = [_specialDirectories objectEnumerator];
    NSDictionary *itemDict;
    BOOL defaultFound = NO;
    while(itemDict = [esd nextObject]){
        NSMenuItem *newItem = [[NSMenuItem alloc] initWithTitle:[itemDict objectForKey:@"displayName"] action:@selector(storageAction:) keyEquivalent:@""];
        NSDictionary *repObj = [NSDictionary dictionaryWithObjectsAndKeys:[itemDict objectForKey:@"path"], @"path", defaultKey, @"key", nil];
        [newItem setRepresentedObject:repObj];
        NSImage *icon = [[NSWorkspace sharedWorkspace] iconForFile:[itemDict objectForKey:@"path"]];
        [icon setScalesWhenResized:YES];
        [icon setSize:NSMakeSize(16.0, 16.0)];
        [newItem setImage:icon];
        [[button menu] addItem:newItem];
        if([[itemDict objectForKey:@"path"] isEqualToString:[[NSUserDefaults standardUserDefaults] objectForKey:defaultKey]]){
            [button selectItem:newItem];
            defaultFound = YES;
        }
        [newItem release];
    }
    
    if(!defaultFound){
        // add another item for the path stored in defaults
        NSMutableString *thisPath = [[[NSUserDefaults standardUserDefaults] objectForKey:defaultKey] mutableCopy];
        BOOL isDir;
        if (!([[NSFileManager defaultManager] fileExistsAtPath:thisPath isDirectory:&isDir] && isDir)){
            [thisPath setString:NSTemporaryDirectory()];
        }
        NSString *thisDisplayName = [[NSFileManager defaultManager] displayNameAtPath:thisPath];
        NSMenuItem *newItem = [[NSMenuItem alloc] initWithTitle:thisDisplayName action:@selector(storageAction:) keyEquivalent:@""];
        NSDictionary *repObj = [NSDictionary dictionaryWithObjectsAndKeys:thisPath, @"path", defaultKey, @"key", nil];
        [newItem setRepresentedObject:repObj];
        NSImage *icon = [[NSWorkspace sharedWorkspace] iconForFile:thisPath];
        [icon setScalesWhenResized:YES];
        [icon setSize:NSMakeSize(16.0, 16.0)];
        [newItem setImage:icon];
        [[button menu] addItem:newItem];
        [button selectItem:newItem];
    }
    
    [[button menu] addItem:[NSMenuItem separatorItem]];
    NSMenuItem *otherItem = [[NSMenuItem alloc] initWithTitle:@"Other..." action:@selector(storageAction:) keyEquivalent:@""];
    NSDictionary *repObj = [NSDictionary dictionaryWithObjectsAndKeys:@"", @"path", defaultKey, @"key", nil];
    [otherItem setRepresentedObject:repObj];
    [[button menu] addItem:otherItem];
}

- (void)storageAction:(id)sender
{
    if([[[sender representedObject] objectForKey:@"path"] rangeOfString:@"/"].length > 0){
        [[NSUserDefaults standardUserDefaults] setObject:[[sender representedObject] objectForKey:@"path"] forKey:[[sender representedObject] objectForKey:@"key"]];
        [self populateStoragePopUp:defaultStorage withDefault:@"Default storage location"];
        [self populateStoragePopUp:imageStorage withDefault:@"Default image location"];
        [self populateStoragePopUp:audioStorage withDefault:@"Default audio location"];
        [self populateStoragePopUp:videoStorage withDefault:@"Default video location"];
        [self populateStoragePopUp:documentStorage withDefault:@"Default document location"];
    } else {
        NSOpenPanel *op = [NSOpenPanel openPanel];
        [op setCanChooseFiles:NO];
        [op setCanChooseDirectories:YES];
        [op setAllowsMultipleSelection:NO];
        [op setCanCreateDirectories:YES];
        [op beginSheetForDirectory:nil file:nil types:nil modalForWindow:[self window] modalDelegate:self didEndSelector:@selector(setStorageDirectory:returnCode:contextInfo:) contextInfo:[[sender representedObject] retain]];
        // panel is up
    }
}

- (void)setStorageDirectory:(NSOpenPanel *)panel returnCode:(int)returnCode  contextInfo:(void  *)contextInfo
{
    [[NSUserDefaults standardUserDefaults] setObject:[[panel filenames] objectAtIndex:0] forKey:[(NSDictionary *)contextInfo objectForKey:@"key"]];
    [(NSDictionary *)contextInfo release];
    [self populateStoragePopUp:defaultStorage withDefault:@"Default storage location"];
    [self populateStoragePopUp:imageStorage withDefault:@"Default image location"];
    [self populateStoragePopUp:audioStorage withDefault:@"Default audio location"];
    [self populateStoragePopUp:videoStorage withDefault:@"Default video location"];
    [self populateStoragePopUp:documentStorage withDefault:@"Default document location"];
}

#pragma mark -
#pragma mark Slideshow

- (IBAction)slideshowChange:(id)sender
{
    [glSlideshowView triggerChange];
}

#pragma mark -
#pragma mark Network

- (void)periodicUpdatePrefChanged:(id)sender
{
    if([sender state] == NSOffState){
        [[NSUserDefaultsController sharedUserDefaultsController] setValue:nil forKeyPath:@"values.SUScheduledCheckInterval"];
        [[[NSApp delegate] sparkle] scheduleCheckWithInterval:0];
    } else {
        [[NSUserDefaultsController sharedUserDefaultsController] setValue:[[periodicUpdateFrequency selectedItem] representedObject] forKeyPath:@"values.SUScheduledCheckInterval"];
        [[[NSApp delegate] sparkle] scheduleCheckWithInterval:[[[periodicUpdateFrequency selectedItem] representedObject] intValue]];
    }
}

#pragma mark -
#pragma mark Toolbar

- (void)selectPreferencePane:(id)sender
{
    int bufferHeight = [[self window] frame].size.height - [[[self window] contentView] frame].size.height;
    NSView *targetView = [_viewList objectForKey:[sender itemIdentifier]];
    int targetHeight = bufferHeight + [targetView frame].size.height;
    NSView *tempView = [[NSView alloc] initWithFrame:[[[self window] contentView] frame]];
    [[self window] setContentView:tempView];
    [[self window] setFrame:NSMakeRect([[self window] frame].origin.x, [[self window] frame].origin.y + [[self window] frame].size.height - targetHeight, [[self window] frame].size.width, targetHeight) display:YES animate:YES];
    [[self window] setContentView:targetView];
    [[self window] setTitle:[sender label]];
    [tempView release];
    
    // set proper first responder
    if(targetView == filesView){
        [[self window] makeFirstResponder:fileTypesFirstResponder];
    }
    if(targetView == pluginsView){
        [[self window] makeFirstResponder:pluginsFirstResponder];
    }
    if(targetView == storageView){
        [[self window] makeFirstResponder:storageFirstResponder];
    }
    if(targetView == slideshowView){
        [[self window] makeFirstResponder:slideshowFirstResponder];
    }
    if(targetView == networkView){
        [[self window] makeFirstResponder:networkFirstResponder];
    }
    
    // make sure proper toolbar item chosen (keyboard navigation)
    [toolbar setSelectedItemIdentifier:[sender itemIdentifier]];
    
    // registration/cripple
    if(![[NSApp delegate] isRegistered] && [[sender itemIdentifier] isEqualToString:@"StorageItem"]){
        NSBeginAlertSheet(@"Please Register", nil, nil, nil, [NSApp keyWindow], nil, nil, nil, nil, @"This demo version of Pandora does not support storage options for collected files.  Please register today and unlock all of the power Pandora has to offer!");
    }
}

- (NSToolbarItem *)toolbar:(NSToolbar *)toolbar itemForItemIdentifier:(NSString *)itemIdentifier willBeInsertedIntoToolbar:(BOOL)flag {
    NSToolbarItem *item = [[NSToolbarItem alloc] initWithItemIdentifier:itemIdentifier];
    
    if([itemIdentifier isEqualToString:@"FilesItem"]){
        [item setPaletteLabel:@"File Types"]; 
        [item setLabel:@"File Types"]; 
        [item setToolTip:@"Specify file types you are interested in."];
        [item setImage:[NSImage imageNamed:@"32_out_filetype"]];
        [item setTarget:self]; 
        [item setAction:@selector(selectPreferencePane:)];
        
    } else if([itemIdentifier isEqualToString:@"PlugInsItem"]){
        [item setPaletteLabel:@"Plugins"]; 
        [item setLabel:@"Plugins"]; 
        [item setToolTip:@"View and configure Pandora plugins."];
        [item setImage:[NSImage imageNamed:@"32_out_plugin"]];
        [item setTarget:self]; 
        [item setAction:@selector(selectPreferencePane:)];
        
    } else if([itemIdentifier isEqualToString:@"StorageItem"]){
        [item setPaletteLabel:@"Storage"]; 
        [item setLabel:@"Storage"]; 
        [item setToolTip:@"Specify storage options for collected files."];
        [item setImage:[NSImage imageNamed:@"32_out_storage"]];
        [item setTarget:self]; 
        [item setAction:@selector(selectPreferencePane:)];
        
    } else if([itemIdentifier isEqualToString:@"SlideshowItem"]){
        [item setPaletteLabel:@"Slideshow"]; 
        [item setLabel:@"Slideshow"]; 
        [item setToolTip:@"Configure slideshow options."];
        [item setImage:[NSImage imageNamed:@"32_out_slideshow"]];
        [item setTarget:self]; 
        [item setAction:@selector(selectPreferencePane:)];
        
    } else if([itemIdentifier isEqualToString:@"NetworkItem"]){
        [item setPaletteLabel:@"Network"]; 
        [item setLabel:@"Network"]; 
        [item setToolTip:@"Configure network options."];
        [item setImage:[NSImage imageNamed:@"32_out_network"]];
        [item setTarget:self]; 
        [item setAction:@selector(selectPreferencePane:)];
        
    }
    
    return [item autorelease];
}

- (NSArray *)toolbarDefaultItemIdentifiers:(NSToolbar*)toolbar {
    return [NSArray arrayWithObjects: 
        @"FilesItem", 
        @"PlugInsItem",
        @"StorageItem",
        @"SlideshowItem",
        @"NetworkItem",
        nil];
}

- (NSArray *)toolbarAllowedItemIdentifiers:(NSToolbar*)toolbar {
    return [NSArray arrayWithObjects: 
        @"FilesItem", 
        @"PlugInsItem",
        @"StorageItem",
        @"SlideshowItem", 
        @"NetworkItem",
        nil];
}

- (NSArray *)toolbarSelectableItemIdentifiers:(NSToolbar*)toolbar {
    return [NSArray arrayWithObjects: 
        @"FilesItem", 
        @"PlugInsItem",
        @"StorageItem",
        @"SlideshowItem", 
        @"NetworkItem",
        nil];
}

- (IBAction)customizeToolbar:(id)sender {
    [toolbar runCustomizationPalette:sender];
}

- (IBAction)toggleToolbar:(id)sender {
    [toolbar setVisible:![toolbar isVisible]];
}

- (BOOL)validateToolbarItem:(NSToolbarItem *)theItem
{
    return YES;
}

- (BOOL)validateMenuItem:(NSMenuItem*)anItem
{
    if ([anItem action] == @selector(toggleToolbar:)){
        // set the title of the menu based on current state
        if([toolbar isVisible]){
            [anItem setTitle:@"Hide Toolbar"];
        } else {
            [anItem setTitle:@"Show Toolbar"];
        }
        return YES;
    }
    
    if ([anItem action] == @selector(customizeToolbar:)){
        return NO; // always bad
    }
    
    
    // return YES for anything else
    return YES;
}


- (void)windowWillClose:(NSNotification *)aNotification
{
    // end editing for various fields so values are picked up with bindings
    [[a cell] endEditing:[[self window] fieldEditor:NO forObject:a]];
    [[b cell] endEditing:[[self window] fieldEditor:NO forObject:b]];
    [[c cell] endEditing:[[self window] fieldEditor:NO forObject:c]];
    [[d cell] endEditing:[[self window] fieldEditor:NO forObject:d]];
    [[e cell] endEditing:[[self window] fieldEditor:NO forObject:e]];
    [[f cell] endEditing:[[self window] fieldEditor:NO forObject:f]];
    
    [[NSUserDefaults standardUserDefaults] synchronize];
}
@end
