//
//  PSMWindowController.h
//  Pandora
//
//  Created by John Pannell on 12/10/05.
//  Copyright 2005 Positive Spin Media. All rights reserved.
//

/*
This object is instantiated by the app controller in repsonse to the "New Window" menu item.  It is the window controller of the new window, and also its delegate (hooked up in nib).
 
*/

#import <Cocoa/Cocoa.h>
#import <Carbon/Carbon.h>
#import <WebKit/WebKit.h>
@class PSMGLSlideShowView;
@class PSMTabBarControl;
@class KFSplitView;
@class PSMFlippedImageView;
@class ScalingScrollView;
@class PSMWebBrowserButton;
@class PSMToolbarMenuButton;


@interface PSMWindowController : NSWindowController 
{
    IBOutlet    NSArrayController       *sessionController;     // storage for sessions/tabs
    NSMutableArray                      *_sessionArray;
    IBOutlet    NSSearchField           *URLField;              // text field in toolbar
    PSMToolbarMenuButton                *setupButton;
    IBOutlet    PSMTabBarControl        *tabBarControl;
    IBOutlet    KFSplitView             *mainSplitView;
    
    // media viewers
    NSMutableArray                      *_mediaViewers;
    
    // preview pane
    IBOutlet    NSTabView               *previewTabView;
    IBOutlet    NSImageView             *previewHeader;
    IBOutlet    NSTextField             *previewFooterTextField;
    
    // main pane
    IBOutlet    NSView                  *mainPane;
    
    // image viewer
    IBOutlet    NSView                  *imageViewer;
    IBOutlet    PSMFlippedImageView     *bigImageView;
    IBOutlet    ScalingScrollView       *bigScrollView;
    
    // unknown viewer
    IBOutlet    NSView                  *unknownView;
    IBOutlet    NSTextField             *unknownText;
    
    // mini web browser
    IBOutlet    NSView                  *miniWebBrowser;
    IBOutlet    WebView                 *theWebView;
    IBOutlet    PSMWebBrowserButton     *browserBackBtn;
    IBOutlet    PSMWebBrowserButton     *browserForwardBtn;
    IBOutlet    PSMWebBrowserButton     *browserStopReloadBtn;
    IBOutlet    NSTextField             *browserURLTextField;
    
    // status
    IBOutlet    NSTextField             *parentURLTextField;
    
    // delete warning
    IBOutlet NSPanel                    *clearWarningPanel;
    IBOutlet NSButton                   *clearDontRemindButton;
    
    NSToolbar                           *toolbar;
    PSMGLSlideShowView                  *slideShow;
}
// override
- (void)windowDidLoad;

// accessors
- (NSTextField *)URLField;
- (BOOL)isToolboxCollapsed;
- (NSMutableArray *)sessionArray;
- (NSTabView *)previewTabView;

// actions
- (IBAction)rootStringAction:(id)sender;
- (void)startSession:(id)sender returnCode:(int)returnCode contextInfo:(void *)contextInfo;
- (void)userClick:(id)sender;
- (IBAction)toggleScraping:(id)sender;
- (IBAction)pauseScraping:(id)sender;
- (IBAction)resumeScraping:(id)sender;
- (void)updatePreviewFooter;
- (IBAction)viewWebPage:(id)sender;
- (IBAction)showInFinder:(id)sender;
- (IBAction)openFileWithExternalEditor:(id)sender;
- (void)setDesktopPicture:(id)sender;
- (IBAction)addNewTab:(id)sender;
- (IBAction)closeTab:(id)sender;
- (IBAction)closeWindow:(id)sender;
- (IBAction)togglePreviewView:(id)sender;
- (IBAction)nextTab:(id)sender;
- (IBAction)previousTab:(id)sender;
- (void)performFileScrapeInDirectory:(NSString *)directory scrapingSubdirectories:(BOOL)value;
- (IBAction)showStartPage:(id)sender;

// window delegate methods
- (void)windowDidResize:(NSNotification *)aNotification;
- (void)windowWillClose:(NSNotification *)aNotification;

// tab view delegate
- (void)tabView:(NSTabView *)aTabView didSelectTabViewItem:(NSTabViewItem *)tabViewItem;
- (void)tabView:(NSTabView *)aTabView willCloseTabViewItem:(NSTabViewItem *)tabViewItem;

// split view delegate
- (IBAction)togglePreviewView:(id)sender;
- (void)splitViewDidCollapseSubview:(NSNotification *)notification;
- (void)splitViewDidExpandSubview:(NSNotification *)notification;
- (void)splitView:(id)sender didDoubleClickInDivider:(int)index;
- (void)splitView:(id)sender didFinishDragInDivider:(int)index;
- (float)splitView:(id)sender constrainMaxCoordinate:(float)proposedMax
       ofSubviewAt:(int)offset;
- (float)splitView:(id)sender constrainMinCoordinate:(float)proposedMin
       ofSubviewAt:(int)offset;
- (BOOL)splitView:(id)sender canCollapseSubview:(NSView *)subview;
- (void)splitView:(id)sender resizeSubviewsWithOldSize:(NSSize)oldSize;

// toolbar
- (NSToolbarItem *)toolbar:(NSToolbar *)toolbar itemForItemIdentifier:(NSString *)itemIdentifier willBeInsertedIntoToolbar:(BOOL)flag;
- (NSArray *)toolbarDefaultItemIdentifiers:(NSToolbar*)toolbar;
- (NSArray *)toolbarAllowedItemIdentifiers:(NSToolbar*)toolbar;
- (IBAction)customizeToolbar:(id)sender;
- (IBAction)toggleToolbar:(id)sender;
- (BOOL)validateToolbarItem:(NSToolbarItem *)theItem;

// plugin
- (IBAction)configurePlugin:(id)sender;
- (void)setPluginPlaceholder:(NSString *)value;

// clear menu item
- (BOOL)canClearSlides:(id)sender;
- (IBAction)finishClear:(id)sender;
- (IBAction)cancelClear:(id)sender;

// WebView delegate methods
- (void)webView:(WebView *)sender didStartProvisionalLoadForFrame:(WebFrame *)frame;
- (void)webView:(WebView *)sender didFinishLoadForFrame:(WebFrame *)frame;
- (void)webView:(WebView *)sender didFailProvisionalLoadWithError:(NSError *)error forFrame:(WebFrame *)frame;
- (void)webView:(WebView*)sender didCommitLoadForFrame:(WebFrame*)frame;
- (void)webView:(WebView*)webView mouseDidMoveOverElement:(NSDictionary*)info modifierFlags:(unsigned int)flags;
- (WebView *)webView:(WebView *)sender createWebViewWithRequest:(NSURLRequest *)request;

// funky webView stop button
- (void)browserStopLoading:(id)sender;

// slide show
- (void)toggleSlideShow:(id)sender;
- (void)startSlideShow:(id)sender;
- (void)stopSlideShow:(id)sender;

// utility
- (void)placeViewer:(NSView *)view;
- (void)sendFileAtPath:(NSString *)path withEventClass:(OSType)eventClass eventKind:(OSType)eventKind toAppWithSignature:(OSType)targetSignature activating:(BOOL)activateTarget;
OSErr SetDesktopPicture(NSString *picturePath,SInt32 pIndex);

// special coloration
- (void)paintWindow;

@end
