//
//  PSMWindowController.m
//  Pandora
//
//  Created by John Pannell on 12/10/05.
//  Copyright 2005 Positive Spin Media. All rights reserved.
//

#import "PSMAppController.h"
#import "PSMWindowController.h"
#import "PSMPandoraSession.h"
#import "PSMTabBarControl.h"
#import "KFSplitView.h"
#import "ThumbnailMatrix.h"
#import "PSMSplitGrabButton.h"
#import "ScalingScrollView.h"
#import "PSMFlippedImageView.h"
#import "PSMWebBrowserButton.h"
#import "FileController.h"
#import "PSMGLSlideShowView.h"
#import "SlideshowWindow.h"
#import "PandoraPlugin.h"
#import "PSMToolbarMenuButton.h"
#import "PSMToolbarMenuButtonCell.h"
#import "PSMSearchFieldCell.h"

#define kPSMBottomWindowMargin 27

@implementation PSMWindowController
#pragma mark -
#pragma mark Override

- (id)initWithWindowNibName:(NSString *)name
{
    if(self = [super initWithWindowNibName:name]){
        _sessionArray = [[NSMutableArray alloc] init];
        _mediaViewers = [[NSMutableArray alloc] init];
        slideShow = nil;
        setupButton = [[PSMToolbarMenuButton alloc] initWithFrame:NSMakeRect(0,0,32,32)];
        [setupButton setIconImage:[NSImage imageNamed:@"32_out_configure"]];
        [setupButton setPullsDown:YES];
    }
    return self;
}

- (void)windowDidLoad
{
    [super windowDidLoad];
    
    // configure tab bar control
    [tabBarControl setHideForSingleTab:NO];
    [tabBarControl setSizeCellsToFit:YES];
    [tabBarControl setCanCloseOnlyTab:YES];
    
    // proper search field appearance
    if([[NSClassFromString([[NSUserDefaults standardUserDefaults] objectForKey:@"Selected Plugin Name"]) category] isEqualToString:@"Spiders"]){
        [[[URLField cell] searchButtonCell] setImage:[NSImage imageNamed:@"spider"]];
        [[[URLField cell] searchButtonCell] setAlternateImage:[NSImage imageNamed:@"spider"]];
    } else {
        [[[URLField cell] searchButtonCell] setImage:[NSImage imageNamed:@"looking_glass"]];
        [[[URLField cell] searchButtonCell] setAlternateImage:[NSImage imageNamed:@"looking_glass"]];
    }
    
    // web browser buttons
    [browserBackBtn setEnabledImage:[NSImage imageNamed:@"browser_back"]];
    [browserBackBtn setDisabledImage:[NSImage imageNamed:@"browser_back_dim"]];
    [browserForwardBtn setEnabledImage:[NSImage imageNamed:@"browser_forward"]];
    [browserForwardBtn setDisabledImage:[NSImage imageNamed:@"browser_forward_dim"]];
    [browserBackBtn setEnabled:NO];
    [browserForwardBtn setEnabled:NO];
    [[browserBackBtn cell] setImageDimsWhenDisabled:NO];
    [[browserForwardBtn cell] setImageDimsWhenDisabled:NO];
    
    // preview header sizing
    NSRect previewFrame = [[previewHeader superview] frame];
    [previewHeader setFrame:NSMakeRect(previewFrame.origin.x, previewFrame.size.height - 18.0, previewFrame.size.width, 18.0)];
    
    // preview header title
    NSTextField *headerTitle = [[[NSTextField alloc] initWithFrame:NSMakeRect(previewFrame.origin.x, previewFrame.size.height - 18.0, previewFrame.size.width, 16.0)] autorelease];
    [headerTitle setStringValue:@"Preview"];
    [headerTitle setAlignment:NSCenterTextAlignment];
    [headerTitle setAutoresizingMask:(NSViewWidthSizable | NSViewMinYMargin)];
    [headerTitle setFont:[NSFont systemFontOfSize:11.0]];
    [headerTitle setBordered:NO];
    [headerTitle setBezeled:NO];
    [headerTitle setEditable:NO];
    [[headerTitle cell] setDrawsBackground:NO];
    [[previewHeader superview] addSubview:headerTitle positioned:NSWindowAbove relativeTo:previewHeader];
    
    // split grabber button
    PSMSplitGrabButton *splitGrabberButton = [[[PSMSplitGrabButton alloc] initWithFrame:NSMakeRect(previewFrame.size.width - 12.0, previewFrame.size.height - 18.0, 12.0, 18.0)] autorelease];
    [splitGrabberButton setImage:[NSImage imageNamed:@"split_grab.png"]];
    [splitGrabberButton setAutoresizingMask:(NSViewMinXMargin | NSViewMinYMargin)];
    [splitGrabberButton setButtonType:NSMomentaryPushInButton];
    [splitGrabberButton setBordered:NO];
    [[splitGrabberButton cell] setHighlightsBy:NSNoCellMask];
    [splitGrabberButton setSplitView:mainSplitView];
    [[previewHeader superview] addSubview:splitGrabberButton positioned:NSWindowAbove relativeTo:headerTitle];
    
    // preview tab view
    [previewTabView setFrame:NSMakeRect(previewFrame.origin.x, previewFrame.origin.y + kPSMBottomWindowMargin, previewFrame.size.width, previewFrame.size.height - 18.0 - kPSMBottomWindowMargin)];
    [previewTabView removeTabViewItem:[previewTabView tabViewItemAtIndex:0]];
    
    // media viewers
    [_mediaViewers addObject:miniWebBrowser];
    [_mediaViewers addObject:imageViewer];
    [_mediaViewers addObject:unknownView];
    
    // main view - start with web browser
    [self placeViewer:miniWebBrowser];
    [[theWebView mainFrame] loadRequest:[NSURLRequest requestWithURL:[NSURL fileURLWithPath:[[NSBundle mainBundle] pathForResource:@"p2start" ofType:@"html" inDirectory:@"StartPage"]]]];
    
    // configure the big view
    [bigScrollView setDrawsBackground:YES];
    [bigScrollView setBackgroundColor:[NSColor whiteColor]];
    
    // toolbar
    toolbar = [[NSToolbar alloc] initWithIdentifier:@"PandoraDToolbar"];
    [toolbar setDelegate:self];
    [toolbar setAllowsUserCustomization:YES];
    [toolbar setAutosavesConfiguration:YES];
    [toolbar setShowsBaselineSeparator:NO];
    [[self window] setToolbar:toolbar];
    
    // plugin menu
    [setupButton setMenu:[[NSApp delegate] pluginMenu]];
    [self setPluginPlaceholder:[NSString stringWithFormat:@"%@ (%@)", [NSClassFromString([[NSUserDefaults standardUserDefaults] objectForKey:@"Selected Plugin Name"]) prompt], [NSClassFromString([[NSUserDefaults standardUserDefaults] objectForKey:@"Selected Plugin Name"]) name]]];
    
    // positioning and size of parent URL Field - New Leopard Bug?
    // it was getting abruptly cut off and not as wide as was present in the nib file
    float parentWidth = [mainPane bounds].size.width;
    NSRect fieldRect = [parentURLTextField frame];
    fieldRect.size.width = parentWidth - 15.0;
    [parentURLTextField setFrame:fieldRect];
    
    // first responder
    [[self window] makeFirstResponder:setupButton];
    
    // window title
    [[self window] setTitle:[NSString stringWithFormat:@"Pandora (%@)", [NSClassFromString([[NSUserDefaults standardUserDefaults] objectForKey:@"Selected Plugin Name"]) name]]];
    
    // configure autosave of window position and size
    [[self window] setFrameAutosaveName:@"PandoraWindow"];
    
    // set window position using defaults
    [[self window] setFrameUsingName:@"PandoraWindow"];
    
    // color
    [self paintWindow];
}

#pragma mark -
#pragma mark Accessors

- (NSTextField *)URLField
{ 
    return URLField;
}

- (BOOL)isToolboxCollapsed
{
    return [mainSplitView isSubviewCollapsed:[[mainSplitView subviews] objectAtIndex:0]];
}

- (NSMutableArray *)sessionArray
{
    return _sessionArray;
}

- (NSTabView *)previewTabView
{
    return previewTabView;
}

#pragma mark -
#pragma mark Actions

- (IBAction)rootStringAction:(id)sender
{
    // perform verification
    if([[URLField stringValue] isEqualToString:@""])
        return;
    
    // verify that entered text is formatted correctly
    if([[NSClassFromString([[NSUserDefaults standardUserDefaults] objectForKey:@"Selected Plugin Name"]) category] isEqualToString:@"Spiders"] && ([[URLField stringValue] rangeOfString:@"."].length == 0)){
        // url without dots? I don't think so
        NSBeginAlertSheet(@"Questionable URL", @"Continue", @"Cancel", nil, [NSApp keyWindow], self, @selector(startSession:returnCode:contextInfo:), nil, nil, @"Your selected web spider plugin (%@) requires a URL as entry, but the entry you've typed doesn't appear to be a URL.  Do you wish to continue with your entry?", [NSClassFromString([[NSUserDefaults standardUserDefaults] objectForKey:@"Selected Plugin Name"]) name]);
        return;
    }
    
    if(![[NSClassFromString([[NSUserDefaults standardUserDefaults] objectForKey:@"Selected Plugin Name"]) category] isEqualToString:@"Spiders"] && (([[URLField stringValue] rangeOfString:@"."].length != 0) && ([[URLField stringValue] rangeOfString:@" "].length == 0))){
        // keyword with dots and no spaces? Questionable...
        NSBeginAlertSheet(@"Questionable Keyword", @"Continue", @"Cancel", nil, [NSApp keyWindow], self, @selector(startSession:returnCode:contextInfo:), nil, nil, @"Your selected search agent plugin (%@) requires a keyword as entry, but the entry you've typed looks like it might be a URL.  Do you wish to continue with your entry?", [NSClassFromString([[NSUserDefaults standardUserDefaults] objectForKey:@"Selected Plugin Name"]) name]);
        return;
    }
    
    // everything is fine ->
    [self startSession:sender returnCode:NSAlertDefaultReturn contextInfo:NULL];
}

- (void)startSession:(id)sender returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
    if(returnCode != NSAlertDefaultReturn)
        return;
    
    int newSessionID = [[NSApp delegate] sessionID];
    PSMPandoraSession *newSession = [[PSMPandoraSession alloc] initWithSessionID:newSessionID];
    [sessionController addObject:newSession];
    
    // add new tab item
    NSTabViewItem *aTabViewItem;
    if([[[previewTabView selectedTabViewItem] label] isEqualToString:@"Untitled"]){
        aTabViewItem = [previewTabView selectedTabViewItem];
        [aTabViewItem setIdentifier:[newSession objectController]];
        [aTabViewItem setLabel:[URLField stringValue]];
    } else {
        aTabViewItem = [[NSTabViewItem alloc] initWithIdentifier:[newSession objectController]];
        [aTabViewItem setLabel:[URLField stringValue]];
    }
    
    // alloc/init tab view item and add to tab view
    NSScrollView *aScrollView = [[NSScrollView alloc] initWithFrame:[previewTabView frame]];
    [aScrollView setHasVerticalScroller:YES];
    [aScrollView setAutoresizesSubviews:YES];
    [aScrollView setDrawsBackground:YES];
    [aScrollView setBackgroundColor:[NSColor whiteColor]];
    [aScrollView setAutoresizingMask:(NSViewWidthSizable | NSViewHeightSizable)];
    [[[aScrollView subviews] objectAtIndex:0] setPostsBoundsChangedNotifications:YES];
    ThumbnailMatrix *aThumbnailMatrix = [[ThumbnailMatrix alloc] initWithFrame:[[[aScrollView subviews] objectAtIndex:0] frame]];
    [aThumbnailMatrix setAutoresizingMask:(NSViewWidthSizable | NSViewHeightSizable)];
    [aThumbnailMatrix setEnabled:YES];
    [aThumbnailMatrix setTarget:self];
    [aThumbnailMatrix setAction:@selector(userClick:)];
    int cellWidth = (int)([aScrollView contentSize].width / [ThumbnailMatrix correctNumberOfColumns:[aScrollView contentSize].width]);
    [aThumbnailMatrix setCellSize:NSMakeSize(cellWidth,cellWidth)];
    [aThumbnailMatrix addRow];
    [aThumbnailMatrix sizeToCells];
    [aThumbnailMatrix setNeedsDisplay];
    [aScrollView setDocumentView:aThumbnailMatrix];
    [aTabViewItem setView:aScrollView];
    
    // start
    NSString *pluginString = [NSString stringWithFormat:@"%@.plugin", [NSClassFromString([[NSUserDefaults standardUserDefaults] objectForKey:@"Selected Plugin Name"]) name]];
    [newSession startEngineWithRootString:[URLField stringValue] andPlugin:pluginString];
    // field icon
    if([[NSClassFromString([[NSUserDefaults standardUserDefaults] objectForKey:@"Selected Plugin Name"]) category] isEqualToString:@"Spiders"]){
        [newSession setIcon:[NSImage imageNamed:@"spider"]];
    } else {
        [newSession setIcon:[NSImage imageNamed:@"looking_glass"]];
    }
    
    if(![[previewTabView tabViewItems] containsObject:aTabViewItem])
        [previewTabView addTabViewItem:aTabViewItem];
    
    // pick it
    [previewTabView selectTabViewItem:aTabViewItem];
    
    // make the thumbnail matrix view the responder now
    [aThumbnailMatrix setNextKeyView:URLField];
    [[self window] makeFirstResponder:aThumbnailMatrix];
    [aThumbnailMatrix adjustSize];
    
    // hook up view to model
    [newSession setThumbnailMatrix:aThumbnailMatrix];
    [aThumbnailMatrix setEngine:newSession];
    
    // log
    [[[NSApp delegate] sessionIDController] addObject:[NSDictionary dictionaryWithObjectsAndKeys:[NSString stringWithFormat:@"\"%@\" : %@", [URLField stringValue], [NSClassFromString([[NSUserDefaults standardUserDefaults] objectForKey:@"Selected Plugin Name"]) name]], @"name", [NSString stringWithFormat:@"%d", newSessionID], @"sessionID", nil]];
    // update to take up potential new status (and keyboard shortcuts)
    [[[[NSApp mainMenu] itemWithTag:1] submenu] update];
}

- (void)userClick:(id)sender
{
    if(![[sender selectedCell] representedObject])
        return;

    if((![[NSBitmapImageRep imageUnfilteredFileTypes] containsObject:[[[[sender selectedCell] representedObject] storagePath] pathExtension]]) && (![[NSBitmapImageRep imageUnfilteredFileTypes] containsObject:[[[[sender selectedCell] representedObject] MIMEType] lastPathComponent]])){
        // not a bitmap - look into alternate viewers
        // put all pdf, audio, and video into webview
        NSString *mime = [[[sender selectedCell] representedObject] MIMEType];
        NSRange slashChar = [mime rangeOfString:@"/"];
        if(([[mime lastPathComponent] isEqualToString:@"pdf"]) || ([[mime substringToIndex:slashChar.location] isEqualToString:@"video"]) || ([[mime substringToIndex:slashChar.location] isEqualToString:@"audio"])){
            if(![[mainPane subviews] containsObject:miniWebBrowser]){
                [self placeViewer:miniWebBrowser];
            }
            [[theWebView mainFrame] loadRequest:[NSURLRequest requestWithURL:[NSURL fileURLWithPath:[[[sender selectedCell] representedObject] storagePath]]]];
            return;
        } else {
            // can't read this
            if(![[mainPane subviews] containsObject:unknownView]){
                [self placeViewer:unknownView];
            }
            NSString *theApplication;
            NSString *theType;
            [[NSWorkspace sharedWorkspace] getInfoForFile:[[[sender selectedCell] representedObject] storagePath] application:&theApplication type:&theType];
            NSString *warningString = [NSString stringWithFormat:@"Pandora cannot view the file \"%@\"\nWould you like to open this file with %@?", [[[[sender selectedCell] representedObject] storagePath] lastPathComponent], [theApplication lastPathComponent]];
            [unknownText setStringValue:warningString];
            return;
        }
        
    }
    // when a user clicks on the matrix, this is called
    // get the image and display it in a large view
    NSFileManager *fm = [NSFileManager defaultManager];
    float imageWidth, imageHeight;
    NSImage *showImage;
    NSBitmapImageRep *bitmapImageRep;
    
    // if needed, show the image viewer
    if(![[mainPane subviews] containsObject:imageViewer]){
        [self placeViewer:imageViewer];
    }
    
    // only perform the following if the selected cell's file exists
    NSString *filePath = [[[sender selectedCell] representedObject] storagePath];
    if([fm fileExistsAtPath:filePath]){
        // create large image from file on disk
        bitmapImageRep = [[NSBitmapImageRep alloc] 
    initWithData:[NSData dataWithContentsOfFile:filePath]];
        showImage = [[NSImage alloc] initWithSize:[bitmapImageRep size]];
        [showImage addRepresentation:bitmapImageRep]; 
        [showImage setScalesWhenResized:YES]; 
        [showImage setSize:NSMakeSize([bitmapImageRep pixelsWide], [bitmapImageRep pixelsHigh])];
        
        // must size view to the image
        imageWidth = [showImage size].width;
        imageHeight = [showImage size].height;
        [bigImageView setFrame:NSMakeRect(0, 0, imageWidth, imageHeight)];
        [bigImageView setImage:showImage];
        
        // set info
        unsigned long long diskSpace = [[[[[[previewTabView selectedTabViewItem] view] documentView] selectedCell] representedObject] actualContentLength];
        NSString *size;
        if((diskSpace/1024) < 1000){
            // measure in K
            int thisSize = floor(diskSpace/1024);
            size = [NSString stringWithFormat:@"%d K",thisSize];
        } else {
            // measure in M
            float thisSize = diskSpace/(1024.0*1024.0);
            size = [NSString stringWithFormat:@"%0.2f M",thisSize];
        }
        [[bigScrollView infoTextField] setStringValue:[NSString stringWithFormat:@"%d x %d pixels, %@", (int)imageWidth, (int)imageHeight, size]];
        
        // and scale
        [bigImageView scaleFrameBy:[bigScrollView scaleFactor]];
        
        // cripple for non-registered users
        if(![[NSApp delegate] isRegistered]){
            if([[NSApp delegate] singleLargeViewed] < 50){
                // set the parent URL
                [parentURLTextField setStringValue:[[[[[[previewTabView selectedTabViewItem] view] documentView] selectedCell] representedObject] sourceURL]];
            } else {
                // they've viewed more than 50 big images
                // set the start page view in the web view
                [self placeViewer:miniWebBrowser];
                [[theWebView mainFrame] loadRequest:[NSURLRequest requestWithURL:[NSURL URLWithString:@"http://www.positivespinmedia.com/shareware/Pandora/startpage.html"]]];
                [browserURLTextField setStringValue:@"http://www.positivespinmedia.com/shareware/Pandora/startpage.html"];
                // alert sheet
                NSBeginAlertSheet(@"Please Register", nil, nil, nil, [NSApp keyWindow], nil, nil, nil, nil, @"Until it is registered, Pandora will only show fifty full sized views per session.  Please support shareware and register today!");
            }
        } else {
            // registered app
            
            // set the parent URL
            [parentURLTextField setStringValue:[[[[[[previewTabView selectedTabViewItem] view] documentView] selectedCell] representedObject] sourceURL]];
            
        }
        [bitmapImageRep release];
        [showImage release];
    }
}

- (IBAction)toggleScraping:(id)sender
{
    NSMenuItem *stopItem = [[[[NSApp mainMenu] itemWithTag:1] submenu] itemWithTag:11];
    if([[[[[previewTabView selectedTabViewItem] view] documentView] engine] isProcessing]){
        [self pauseScraping:self];
        [stopItem setTitle:@"Resume Loading"];
        [stopItem setKeyEquivalent:@"r"];
        [stopItem setKeyEquivalentModifierMask:NSCommandKeyMask];
    } else {
        [self resumeScraping:self];
        [stopItem setTitle:@"Stop Loading"];
        [stopItem setKeyEquivalent:@"."];
        [stopItem setKeyEquivalentModifierMask:NSCommandKeyMask];
    }
    
    // file menu must update to take up keyboard shortcuts for start/stop item
    [[[[NSApp mainMenu] itemWithTag:1] submenu] update];
}

- (IBAction)pauseScraping:(id)sender
{
    // identify current tab view index and stop corresponding session
    [(PSMPandoraSession *)[[[previewTabView selectedTabViewItem] identifier] content] pause:sender];
}

- (IBAction)dumpSnapshot:(id)sender
{
    [(PSMPandoraSession *)[[[previewTabView selectedTabViewItem] identifier] content] dumpSnapshot];
}

- (IBAction)resumeScraping:(id)sender
{
    [(PSMPandoraSession *)[[[previewTabView selectedTabViewItem] identifier] content] resume:sender];
}

- (void)updatePreviewFooter
{
    if([[[previewTabView selectedTabViewItem] view] respondsToSelector:@selector(documentView)]){
        [previewFooterTextField setStringValue:[[[[previewTabView selectedTabViewItem] view] documentView] imageCountString]];
    } else {
        // empty tab
        [previewFooterTextField setStringValue:@""];
    }
}

- (IBAction)viewWebPage:(id)sender
{
    [self placeViewer:miniWebBrowser];
    [[theWebView mainFrame] loadRequest:[NSURLRequest requestWithURL:[NSURL URLWithString:[[[[[[previewTabView selectedTabViewItem] view] documentView] selectedCell] representedObject] parentWebPage]]]];
    [browserURLTextField setStringValue:[parentURLTextField stringValue]];
    [parentURLTextField setStringValue:@""];
}

- (IBAction)showInFinder:(id)sender
{
    // user wants to see file in finder
    // buggy way:
    //[[NSWorkspace sharedWorkspace] selectFile:[[[[[[previewTabView selectedTabViewItem] view] documentView] selectedCell] representedObject] storagePath] inFileViewerRootedAtPath:@""];
    [self sendFileAtPath:[[[[[[previewTabView selectedTabViewItem] view] documentView] selectedCell] representedObject] storagePath] withEventClass:kAEMiscStandards eventKind:kAEMakeObjectsVisible toAppWithSignature:'MACS' activating:YES];
}

- (IBAction)openFileWithExternalEditor:(id)sender
{
    NSString *theApplication;
    NSString *theType;
    [[NSWorkspace sharedWorkspace] getInfoForFile:[[[[[[previewTabView selectedTabViewItem] view] documentView] selectedCell] representedObject] storagePath] application:&theApplication type:&theType];
    [[NSWorkspace sharedWorkspace] openFile:[[[[[[previewTabView selectedTabViewItem] view] documentView] selectedCell] representedObject] storagePath] withApplication:theApplication];
}

- (void)setDesktopPicture:(id)sender
{
    // get the currently selected image path
    SetDesktopPicture([[[[[[previewTabView selectedTabViewItem] view] documentView] selectedCell] representedObject] storagePath],0);
}

- (IBAction)addNewTab:(id)sender
{
    NSTabViewItem *newItem = [[[NSTabViewItem alloc] initWithIdentifier:nil] autorelease];
    [newItem setLabel:@"Untitled"];
    [previewTabView addTabViewItem:newItem];
    [previewTabView selectTabViewItem:newItem]; // this is optional, but expected behavior
}

- (IBAction)closeTab:(id)sender
{
    if([[[[previewTabView selectedTabViewItem] identifier] content] isProcessing])
        [[[[previewTabView selectedTabViewItem] identifier] content] stopEngine];
    [_sessionArray removeObject:[[[previewTabView selectedTabViewItem] identifier] content]];
    [previewTabView removeTabViewItem:[previewTabView selectedTabViewItem]];
}

- (IBAction)closeWindow:(id)sender
{
    [[self window] performClose:sender];
}

- (IBAction)togglePreviewView:(id)sender
{
    // expand/collapse the preview view
    id viewInQuestion = [[mainSplitView subviews] objectAtIndex:0];
    BOOL currentState = [mainSplitView isSubviewCollapsed:viewInQuestion];
    [mainSplitView setSubview:viewInQuestion isCollapsed:!currentState];
    [mainSplitView resizeSubviewsWithOldSize:[mainSplitView frame].size];
    [mainSplitView setNeedsDisplay:YES];
}

- (IBAction)nextTab:(id)sender
{
    [previewTabView selectTabViewItemAtIndex:([previewTabView indexOfTabViewItem:[previewTabView selectedTabViewItem]] + 1)];
    // update to take up potential new status (and keyboard shortcuts)
    [[[[NSApp mainMenu] itemWithTag:1] submenu] update];
}

- (IBAction)previousTab:(id)sender
{
    [previewTabView selectTabViewItemAtIndex:([previewTabView indexOfTabViewItem:[previewTabView selectedTabViewItem]] - 1)];
    // update to take up potential new status (and keyboard shortcuts)
    [[[[NSApp mainMenu] itemWithTag:1] submenu] update];
}

- (IBAction)zoomScale:(id)sender
{
    // user selected a zoom from the view menu
    [bigScrollView setScaleFactor:[[sender representedObject] floatValue] adjustPopup:YES];
    [[[sender menu] itemArray] makeObjectsPerformSelector:@selector(setState:) withObject:NSOffState];
    [sender setState:NSOnState];
    [[NSUserDefaults standardUserDefaults] setFloat:[[sender representedObject] floatValue] forKey:@"Zoom scale"];
}

- (void)performFileScrapeInDirectory:(NSString *)directory scrapingSubdirectories:(BOOL)value
{
    // create session controller and add to array
    FileController *newSession = [[FileController alloc] initWithString:directory scrapeSubdirectories:value];
    [sessionController addObject:newSession];
    
    // add new tab item
    NSTabViewItem *aTabViewItem;
    if([[[previewTabView selectedTabViewItem] label] isEqualToString:@"Untitled"]){
        aTabViewItem = [previewTabView selectedTabViewItem];
        [aTabViewItem setIdentifier:[newSession objectController]];
        [aTabViewItem setLabel:directory];
    } else {
        aTabViewItem = [[NSTabViewItem alloc] initWithIdentifier:[newSession objectController]];
        [aTabViewItem setLabel:directory];
    }
    
    // alloc/init tab view item and add to tab view
    NSScrollView *aScrollView = [[NSScrollView alloc] initWithFrame:[previewTabView frame]];
    [aScrollView setHasVerticalScroller:YES];
    [aScrollView setAutoresizesSubviews:YES];
    [aScrollView setDrawsBackground:YES];
    [aScrollView setBackgroundColor:[NSColor whiteColor]];
    [aScrollView setAutoresizingMask:(NSViewWidthSizable | NSViewHeightSizable)];
    [[[aScrollView subviews] objectAtIndex:0] setPostsBoundsChangedNotifications:YES];
    ThumbnailMatrix *aThumbnailMatrix = [[ThumbnailMatrix alloc] initWithFrame:[[[aScrollView subviews] objectAtIndex:0] frame]];
    [aThumbnailMatrix setAutoresizingMask:(NSViewWidthSizable | NSViewHeightSizable)];
    [aThumbnailMatrix setEnabled:YES];
    [aThumbnailMatrix setTarget:self];
    [aThumbnailMatrix setAction:@selector(userClick:)];
    int cellWidth = (int)([aScrollView contentSize].width / [ThumbnailMatrix correctNumberOfColumns:[aScrollView contentSize].width]);
    [aThumbnailMatrix setCellSize:NSMakeSize(cellWidth,cellWidth)];
    [aThumbnailMatrix addRow];
    [aThumbnailMatrix sizeToCells];
    [aThumbnailMatrix setNeedsDisplay];
    [aScrollView setDocumentView:aThumbnailMatrix];
    [aTabViewItem setView:aScrollView];
    if(![[previewTabView tabViewItems] containsObject:aTabViewItem])
        [previewTabView addTabViewItem:aTabViewItem];
    [previewTabView selectTabViewItem:aTabViewItem];
    
    // make the thumbnail matrix view the responder now
    [aThumbnailMatrix setNextKeyView:URLField];
    [[self window] makeFirstResponder:aThumbnailMatrix];
    [aThumbnailMatrix adjustSize];
    
    // hook up view to model
    [newSession setThumbnailMatrix:aThumbnailMatrix];
    [aThumbnailMatrix setEngine:newSession];
    
    // and go!
    [newSession startScript];
}

- (IBAction)showStartPage:(id)sender
{
    [self placeViewer:miniWebBrowser];
    [[theWebView mainFrame] loadRequest:[NSURLRequest requestWithURL:[NSURL fileURLWithPath:[[NSBundle mainBundle] pathForResource:@"p2start" ofType:@"html" inDirectory:@"StartPage"]]]];
}

#pragma mark -
#pragma mark Window Delegate

- (void)windowDidBecomeMain:(NSNotification *)notification
{
    [self paintWindow];
}

- (void)windowDidResignMain:(NSNotification *)notification
{
    [self paintWindow];
}

- (void)windowDidResize:(NSNotification *)aNotification
{
    [self paintWindow];
}

- (void)windowWillClose:(NSNotification *)aNotification
{
    // kill all of my spiders
    NSEnumerator *e = [_sessionArray objectEnumerator];
    PSMPandoraSession *session;
    while(session = [e nextObject]){
        [session stopEngine];
    }
    
    [[[NSApp delegate] appWindowControllers] removeObject:self];
}

#pragma mark -
#pragma mark TabView delegate

- (void)tabView:(NSTabView *)aTabView didSelectTabViewItem:(NSTabViewItem *)tabViewItem
{
    // select image
    if([[tabViewItem view] respondsToSelector:@selector(documentView)])
        [self userClick:[[tabViewItem view] documentView]];
    
    // title window and url field
    PSMPandoraSession *session = [[tabViewItem identifier] content];
    if([session rootString]){
        [[self window] setTitle:[NSString stringWithFormat:@"\"%@\" (%@)", [session rootString], [[session pluginName] stringByDeletingPathExtension]]];
        [[[URLField cell] searchButtonCell] setImage:[session icon]];
        [[[URLField cell] searchButtonCell] setAlternateImage:[session icon]];
        [URLField setStringValue:[session rootString]];
    } else {
        [[self window] setTitle:[NSString stringWithFormat:@"Untitled (%@)", [NSClassFromString([[NSUserDefaults standardUserDefaults] objectForKey:@"Selected Plugin Name"]) name]]];
        [URLField setStringValue:@""];
        if([[NSClassFromString([[NSUserDefaults standardUserDefaults] objectForKey:@"Selected Plugin Name"]) category] isEqualToString:@"Spiders"]){
            [[[URLField cell] searchButtonCell] setImage:[NSImage imageNamed:@"spider"]];
            [[[URLField cell] searchButtonCell] setAlternateImage:[NSImage imageNamed:@"spider"]];
        } else {
            [[[URLField cell] searchButtonCell] setImage:[NSImage imageNamed:@"looking_glass"]];
            [[[URLField cell] searchButtonCell] setAlternateImage:[NSImage imageNamed:@"looking_glass"]];
        }
    }
    
    // fix footer
    [self updatePreviewFooter];
    
}

- (void)tabView:(NSTabView *)aTabView willCloseTabViewItem:(NSTabViewItem *)tabViewItem
{
    [[[tabViewItem identifier] content] stopEngine];
    
    // find and remove this session from session IDs
    NSEnumerator *e = [[[[NSApp delegate] sessionIDController] arrangedObjects] objectEnumerator];
    NSDictionary *dict;
    while(dict = [e nextObject]){
        if([[dict objectForKey:@"sessionID"] intValue] == [[[tabViewItem identifier] content] sessionID])
            [[[NSApp delegate] sessionIDController] removeObject:dict];
    }
    
    [_sessionArray removeObject:[[tabViewItem identifier] content]];
    // update to take up potential new status (and keyboard shortcuts)
    [[[[NSApp mainMenu] itemWithTag:1] submenu] update];
}

#pragma mark -
#pragma mark Split View Delegate

- (void)splitViewDidCollapseSubview:(NSNotification *)notification
{
    [mainSplitView resizeSubviewsWithOldSize:[mainSplitView frame].size];
    [bigScrollView viewDidEndLiveResize];
}

- (void)splitViewDidExpandSubview:(NSNotification *)notification
{  
    [mainSplitView resizeSubviewsWithOldSize:[mainSplitView frame].size];
    [bigScrollView viewDidEndLiveResize];
}

- (void)splitView:(id)sender didDoubleClickInDivider:(int)index
{
    [self togglePreviewView:nil];
}

- (void)splitView:(id)sender didFinishDragInDivider:(int)index
{
    [bigScrollView viewDidEndLiveResize];
}

- (float)splitView:(id)sender constrainMaxCoordinate:(float)proposedMax
       ofSubviewAt:(int)offset
{
    return proposedMax - 90;
}

- (float)splitView:(id)sender constrainMinCoordinate:(float)proposedMin
       ofSubviewAt:(int)offset
{
    return proposedMin + 108;
}

- (BOOL)splitView:(id)sender canCollapseSubview:(NSView *)subview
{
    return YES;
}

- (void)splitView:(id)sender resizeSubviewsWithOldSize:(NSSize)oldSize
{
    // It's our responsibility to set the frame rectangles of
    // all subviews.  This means setting origin and size of
    // uncollapsed subviews, and setting origin of collapsed subviews
    // to the offscreen position, KFOffScreenPoint.
    int numSubviews, numDividers;
    float splitViewWidth, splitViewHeight;
    float dividerThickness, availableSpace;
    float minimumFirstSubviewWidth;
    id subviews;
    
    // setup
    subviews = [sender subviews];
    numSubviews = [subviews count];
    numDividers = numSubviews - 1;
    splitViewWidth = [sender frame].size.width;
    splitViewHeight = [sender frame].size.height;
    dividerThickness = [sender dividerThickness];
    
    minimumFirstSubviewWidth = 108;
    
    // my desire - the left subview does not resize, unless dragged, expanded, or collapsed.
    if (![sender isSubviewCollapsed:[subviews objectAtIndex:0]]){
        // preview view is showing
        availableSpace = splitViewWidth - [[subviews objectAtIndex:0] frame].size.width - dividerThickness;
        [[subviews objectAtIndex:1] setFrame:NSMakeRect([[subviews objectAtIndex:0] frame].size.width + dividerThickness, 0, availableSpace, splitViewHeight)];
        [[subviews objectAtIndex:0] setFrame:NSMakeRect(0, 0, [[subviews objectAtIndex:0] frame].size.width, splitViewHeight)];
    } else {
        // preview view is hidden
        availableSpace = splitViewWidth - dividerThickness;
        [[subviews objectAtIndex:1] setFrame:NSMakeRect(dividerThickness, 0, availableSpace, splitViewHeight)];
        [[subviews objectAtIndex:0] setFrameOrigin:KFOffScreenPoint];
    }
}


#pragma mark -
#pragma mark Toolbar

- (NSToolbarItem *)toolbar:(NSToolbar *)toolbar itemForItemIdentifier:(NSString *)itemIdentifier willBeInsertedIntoToolbar:(BOOL)flag {
    NSToolbarItem *item = [[NSToolbarItem alloc] initWithItemIdentifier:itemIdentifier];
    
    if([itemIdentifier isEqualToString:@"OpenItem"]){
        [item setPaletteLabel:@"Open"]; 
        [item setLabel:@"Open"]; 
        [item setToolTip:@"Collect images from contents of folder(s)"];
        [item setImage:[NSImage imageNamed:@"32_out_open"]];
        [item setTarget:[NSApp delegate]]; 
        [item setAction:@selector(openFiles:)];
        
    } else if([itemIdentifier isEqualToString:@"URLSearchItem"]){
        [item setPaletteLabel:@"URL or Keywords"]; 
        [item setLabel:@"URL or Keywords"]; 
        [item setView:[URLField retain]];
        [item setMinSize:NSMakeSize(100, [URLField frame].size.height)];
        [item setMaxSize:NSMakeSize(400, [URLField frame].size.height)];
        
    } else if([itemIdentifier isEqualToString:@"ConfigureItem"]){
        [item setPaletteLabel:@"Setup"]; 
        [item setLabel:@"Setup"]; 
        [item setToolTip:@"Select and configure Agent or Spider plugin"];
        [item setView:setupButton];
        [item setMinSize:NSMakeSize(32,32)];
        [item setMaxSize:NSMakeSize(32,32)];
        [setupButton setTarget:self]; 
        [setupButton setAction:@selector(configurePlugin:)];
        
    } else if([itemIdentifier isEqualToString:@"ShowInFinderItem"]){
        [item setPaletteLabel:@"Finder"]; 
        [item setLabel:@"Finder"]; 
        [item setToolTip:@"Display location of file in Finder"];
        [item setImage:[NSImage imageNamed:@"32_out_finder"]];
        [item setTarget:self]; 
        [item setAction:@selector(showInFinder:)];
        
    } else if([itemIdentifier isEqualToString:@"ViewWebPageItem"]){
        [item setPaletteLabel:@"Web Page"]; 
        [item setLabel:@"Web Page"]; 
        [item setToolTip:@"Display web page where file was found"];
        [item setImage:[NSImage imageNamed:@"32_out_web"]];
        [item setTarget:self]; 
        [item setAction:@selector(viewWebPage:)];
        
    } else if([itemIdentifier isEqualToString:@"SetDesktopItem"]){
        [item setPaletteLabel:@"Desktop"]; 
        [item setLabel:@"Desktop"]; 
        [item setToolTip:@"Set Desktop to selected image"];
        [item setImage:[NSImage imageNamed:@"32_out_desktop"]];
        [item setTarget:self]; 
        [item setAction:@selector(setDesktopPicture:)]; 
        
    } else if([itemIdentifier isEqualToString:@"TrashItem"]){
        [item setPaletteLabel:@"Trash"]; 
        [item setLabel:@"Trash"]; 
        [item setToolTip:@"Move selected files to Trash"];
        [item setImage:[NSImage imageNamed:@"32_out_trash"]];
        [item setTarget:nil]; 
        [item setAction:@selector(clear:)];
        
    } else if([itemIdentifier isEqualToString:@"SlideshowItem"]){
        [item setPaletteLabel:@"Slideshow"]; 
        [item setLabel:@"Slideshow"]; 
        [item setToolTip:@"Start slideshow of previewed images"];
        [item setImage:[NSImage imageNamed:@"32_out_slideshow_play"]];
        [item setTarget:self]; 
        [item setAction:@selector(toggleSlideShow:)];
        
    } else if([itemIdentifier isEqualToString:@"LogItem"]){
        [item setPaletteLabel:@"Log"]; 
        [item setLabel:@"Log"]; 
        [item setToolTip:@"View Log"];
        [item setImage:[NSImage imageNamed:@"32_out_log"]];
        [item setTarget:[NSApp delegate]]; 
        [item setAction:@selector(viewLog:)];
        
    } else if([itemIdentifier isEqualToString:@"PrintItem"]){
        [item setPaletteLabel:@"Print"]; 
        [item setLabel:@"Print"]; 
        [item setToolTip:@"Print currently viewed image"];
        [item setImage:[NSImage imageNamed:@"32_out_print"]];
        [item setTarget:self]; 
        [item setAction:@selector(printDocument:)];
        
    } else if([itemIdentifier isEqualToString:@"PrefsItem"]){
        [item setPaletteLabel:@"Preferences"]; 
        [item setLabel:@"Preferences"]; 
        [item setToolTip:@"Show Preferences"];
        [item setImage:[NSImage imageNamed:@"32_out_preferences"]];
        [item setTarget:[NSApp delegate]]; 
        [item setAction:@selector(showPreferencePanel:)];
        
    } else if([itemIdentifier isEqualToString:@"StopItem"]){
        [item setPaletteLabel:@"Start"]; 
        [item setLabel:@"Start"]; 
        [item setToolTip:@"Begin the loading process"];
        [item setImage:[NSImage imageNamed:@"32_out_play"]];
        [item setTarget:self]; 
        [item setAction:@selector(rootStringAction:)];
    }
    
    return [item autorelease];
}

- (NSArray *)toolbarDefaultItemIdentifiers:(NSToolbar*)toolbar {
    return [NSArray arrayWithObjects: 
        @"ConfigureItem",
        @"URLSearchItem",
        @"StopItem",
        NSToolbarSeparatorItemIdentifier,
        @"ShowInFinderItem",
        @"ViewWebPageItem",
        @"SetDesktopItem",
        @"TrashItem",
        NSToolbarFlexibleSpaceItemIdentifier,
        @"SlideshowItem", 
        @"LogItem",
        nil];
}

- (NSArray *)toolbarAllowedItemIdentifiers:(NSToolbar*)toolbar {
    return [NSArray arrayWithObjects: 
        @"OpenItem", 
        @"URLSearchItem",
        @"ConfigureItem",
        @"StopItem",
        @"SlideshowItem", 
        @"LogItem",
        @"ShowInFinderItem",
        @"ViewWebPageItem",
        @"SetDesktopItem",
        @"TrashItem",
        @"PrintItem",
        @"PrefsItem",
        NSToolbarSeparatorItemIdentifier,
        NSToolbarSpaceItemIdentifier,
        NSToolbarFlexibleSpaceItemIdentifier,
        NSToolbarCustomizeToolbarItemIdentifier,
        nil];
}

- (IBAction)customizeToolbar:(id)sender {
    [toolbar runCustomizationPalette:sender];
}

- (IBAction)toggleToolbar:(id)sender {
    [toolbar setVisible:![toolbar isVisible]];
}

- (BOOL)validateToolbarItem:(NSToolbarItem *)theItem
{
    NSString *itemIdentifier = [theItem itemIdentifier];
    
    if([itemIdentifier isEqualToString:@"OpenItem"]){
        return YES;     // always available
        
    } else if([itemIdentifier isEqualToString:@"URLSearchItem"]){
        return YES;     // always available
        
    } else if([itemIdentifier isEqualToString:@"ConfigureItem"]){
        return YES;     // always available
        
    } else if([itemIdentifier isEqualToString:@"ShowInFinderItem"]){
        // must be registered and have selected a file
        if([_sessionArray count] <= 0)
            return NO;
        
        if([[[previewTabView selectedTabViewItem] label] isEqualToString:@"Untitled"])
            return NO;
        
        if([[NSApp delegate] isRegistered] && ([[[[[previewTabView selectedTabViewItem] view] documentView] selectedCell] representedObject] != nil)){
            return YES;
        }
        
        return NO;
        
    } else if([itemIdentifier isEqualToString:@"ViewWebPageItem"]){
        // must select valid object from a web scrape, not a file scrape
        // must have a selected item with a URL and not a file URL
        if([_sessionArray count] == 0){
            return NO;
        }
        
        if([[[previewTabView selectedTabViewItem] label] isEqualToString:@"Untitled"])
            return NO;
        
        if([[[[[[previewTabView selectedTabViewItem] view] documentView] selectedCell] representedObject] parentWebPage] != nil){
            return YES;
        } else {
            return NO;
        }
        
        
    } else if([itemIdentifier isEqualToString:@"SetDesktopItem"]){
        // must have something selected
        if([[[previewTabView selectedTabViewItem] label] isEqualToString:@"Untitled"])
            return NO;
        
        if([_sessionArray count] > 0){
            if([[NSBitmapImageRep imageUnfilteredFileTypes] containsObject:[[[[[[[previewTabView selectedTabViewItem] view] documentView] selectedCell] representedObject] storagePath] pathExtension]]){
                return YES;
            } 
        }
        return NO;
        
    } else if([itemIdentifier isEqualToString:@"SlideshowItem"]){
        // set state based on slideshow toggle
        if(slideShow != nil){
            [theItem setImage:[NSImage imageNamed:@"32_out_slideshow_stop"]];
        } else {
            [theItem setImage:[NSImage imageNamed:@"32_out_slideshow_play"]];
        }
        
        if([[[previewTabView selectedTabViewItem] label] isEqualToString:@"Untitled"])
            return NO;
        
        // must be more than zero slides
        if([[[[[previewTabView selectedTabViewItem] view] documentView] slideShowPaths] count] > 0){
            return YES;
        }
        return NO;
        
    } else if([itemIdentifier isEqualToString:@"LogItem"]){
        return YES;
        
    } else if([itemIdentifier isEqualToString:@"PrintItem"]){
        // something must be present in the main view
        return YES;
        
    } else if([itemIdentifier isEqualToString:@"PrefsItem"]){
        return YES;
        
    } else if([itemIdentifier isEqualToString:@"StopItem"]){
        
        if([previewTabView numberOfTabViewItems] > 0){
            if([[[previewTabView selectedTabViewItem] label] isEqualToString:@"Untitled"]){
                [theItem setLabel:@"Start"];
                [theItem setToolTip:@"Start Loading"];
                [theItem setImage:[NSImage imageNamed:@"32_out_play"]];
                [theItem setAction:@selector(rootStringAction:)];
                return YES;
            }
            if([[[[[previewTabView selectedTabViewItem] view] documentView] engine] isDone]){
                // nothing left to get, no resume.
                [theItem setLabel:@"Stop"];
                [theItem setToolTip:@"Stop Loading"];
                [theItem setImage:[NSImage imageNamed:@"32_out_stop"]];
                [theItem setAction:@selector(pauseScraping:)];
                return NO;
            }
            if([[[[[previewTabView selectedTabViewItem] view] documentView] engine] isProcessing]){
                [theItem setLabel:@"Pause"];
                [theItem setToolTip:@"Pause Loading"];
                [theItem setImage:[NSImage imageNamed:@"32_out_pause"]];
                [theItem setAction:@selector(pauseScraping:)];
                return YES;
            } else {
                [theItem setLabel:@"Resume"];
                [theItem setToolTip:@"Resume Loading"];
                [theItem setImage:[NSImage imageNamed:@"32_out_play"]];
                [theItem setAction:@selector(resumeScraping:)];
                return YES;
            }
        } else {
            [theItem setLabel:@"Start"];
            [theItem setToolTip:@"Start Loading"];
            [theItem setImage:[NSImage imageNamed:@"32_out_play"]];
            [theItem setAction:@selector(rootStringAction:)];
            return YES;
        }
    }
    
    return NO;
}

#pragma mark -
#pragma mark Plugin Preferences

- (IBAction)configurePlugin:(id)sender
{
    // instantiate selected plugin
    id plugin = [[NSClassFromString([[NSUserDefaults standardUserDefaults] objectForKey:@"Selected Plugin Name"]) alloc] init];
    [NSApp beginSheet: [plugin window]
       modalForWindow: [NSApp mainWindow]
        modalDelegate: nil
       didEndSelector: nil
          contextInfo: nil];
    [NSApp runModalForWindow:[plugin window]];
    // Sheet is up here.
}

- (void)setPluginPlaceholder:(NSString *)value
{
    [[URLField cell] setPlaceholderString:value];
    // change field icon if there is not existing icon in place
    if(![[[[previewTabView selectedTabViewItem] identifier] content] respondsToSelector:@selector(icon)]){
        if([[NSClassFromString([[NSUserDefaults standardUserDefaults] objectForKey:@"Selected Plugin Name"]) category] isEqualToString:@"Spiders"]){
            [[[URLField cell] searchButtonCell] setImage:[NSImage imageNamed:@"spider"]];
            [[[URLField cell] searchButtonCell] setAlternateImage:[NSImage imageNamed:@"spider"]];
        } else {
            [[[URLField cell] searchButtonCell] setImage:[NSImage imageNamed:@"looking_glass"]];
            [[[URLField cell] searchButtonCell] setAlternateImage:[NSImage imageNamed:@"looking_glass"]];
        }
    }
}

#pragma mark -
#pragma mark Clear Menu Item

- (BOOL)canClearSlides:(id)sender
{
    if([[NSUserDefaults standardUserDefaults] boolForKey:@"Delete files warning"]){
        [NSApp beginSheet: clearWarningPanel
           modalForWindow: [self window]
            modalDelegate: nil
           didEndSelector: nil
              contextInfo: nil];
        [NSApp runModalForWindow: clearWarningPanel];
        // Sheet is up here.
        [NSApp endSheet:clearWarningPanel];
        [clearWarningPanel orderOut:self];
        return NO;
    } else {
        return YES;
    }
}

- (IBAction)finishClear:(id)sender
{
    [NSApp stopModal];
    // note the reminder state
    if([clearDontRemindButton state] == NSOnState){
        [[NSUserDefaults standardUserDefaults] setBool:NO forKey:@"Delete files warning"];
    }
    
    // nuke the slide(s) and the file(s) associated with it(them)
    [[[[previewTabView selectedTabViewItem] view] documentView] deleteSelectedCellsWithTrashing];
}

- (IBAction)cancelClear:(id)sender
{
    [NSApp stopModal];
    // note the reminder state
    if([clearDontRemindButton state] == NSOnState){
        [[NSUserDefaults standardUserDefaults] setBool:NO forKey:@"Delete files warning"];
    }
}

#pragma mark -
#pragma mark WebView Delegate

- (void)webView:(WebView *)sender didStartProvisionalLoadForFrame:(WebFrame *)frame
{
    [browserStopReloadBtn setImage:[NSImage imageNamed:@"browser_stop"]];
    [browserStopReloadBtn setAlternateImage:[NSImage imageNamed:@"browser_stop_click"]];
    [browserStopReloadBtn setTarget:self];
    [browserStopReloadBtn setAction:@selector(browserStopLoading:)];
    
    [parentURLTextField setStringValue:[NSString stringWithFormat:@"Connecting to \"%@\"", [[[[frame provisionalDataSource] request] URL] host]]];
    
}

- (void)webView:(WebView*)sender didCommitLoadForFrame:(WebFrame*)frame
{
    if (frame == [sender mainFrame]){
        NSString *url = [[[[frame dataSource] request] URL] absoluteString];
        [browserURLTextField setStringValue:url];
    }
    [parentURLTextField setStringValue:[NSString stringWithFormat:@"Loading \"%@\"", [[[[frame dataSource] request] URL] absoluteString]]];
}

- (void)webView:(WebView *)sender didFinishLoadForFrame:(WebFrame *)frame
{
    [browserStopReloadBtn setImage:[NSImage imageNamed:@"browser_reload"]];
    [browserStopReloadBtn setAlternateImage:[NSImage imageNamed:@"browser_reload_click"]];
    [browserStopReloadBtn setTarget:theWebView];
    [browserStopReloadBtn setAction:@selector(reload:)];
    [parentURLTextField setStringValue:@""];
}

- (void)webView:(WebView *)sender didFailProvisionalLoadWithError:(NSError *)error forFrame:(WebFrame *)frame
{
    [browserStopReloadBtn setImage:[NSImage imageNamed:@"browser_reload"]];
    [browserStopReloadBtn setAlternateImage:[NSImage imageNamed:@"browser_reload_click"]];
    [browserStopReloadBtn setTarget:theWebView];
    [browserStopReloadBtn setAction:@selector(reload:)];
    [parentURLTextField setStringValue:@""];
}

- (void)webView:(WebView*)webView mouseDidMoveOverElement:(NSDictionary*)info modifierFlags:(unsigned int)flags
{
    NSString*   status = nil;
    // Get URL string
    NSString*   URLString;
    URLString = [[info objectForKey:WebElementLinkURLKey] _web_userVisibleString];
    
    if (URLString) {
        // Get scheme
        NSURL*      URL;
        NSString*   scheme;
        URL = [NSURL _web_URLWithUserTypedString:URLString];
        scheme = [URL scheme];
        
        // For mailto:
        if ([scheme isEqualToString:@"mailto"]) {
            status = @"Send e-mail to \"%@\"", nil;
            goto finalize;
        }
        
        // Get target
        WebFrame*   target;
        target = [info objectForKey:WebElementLinkTargetFrameKey];
        
        // Target is in this frame
        if (target) {
            // Go to URL
            status = @"Go to \"%@\"", nil;
            goto finalize;
        } else {
            // Open page in new window - not doing this in this app
            status = @"Go to \"%@\"", nil;
            goto finalize;
        }
    }
    
finalize:
        if (status) {
            [parentURLTextField setStringValue:[NSString stringWithFormat:status, URLString]];
        } else {
            [parentURLTextField setStringValue:@""];
        }
}

- (WebView *)webView:(WebView *)sender createWebViewWithRequest:(NSURLRequest *)request
{
    // I'm not popping up any stinking new windows
    [[sender mainFrame] loadRequest:request];
    return sender;
}

// funky webView stop button
- (void)browserStopLoading:(id)sender
{
    [theWebView stopLoading:sender];
    // fix button back
    [browserStopReloadBtn setImage:[NSImage imageNamed:@"browser_reload"]];
    [browserStopReloadBtn setAlternateImage:[NSImage imageNamed:@"browser_reload_click"]];
    [browserStopReloadBtn setTarget:theWebView];
    [browserStopReloadBtn setAction:@selector(reload:)];
    [parentURLTextField setStringValue:@""];
}

#pragma mark -
#pragma mark Saving Files

- (void)saveFileToDisk:(id)sender
{
    NSSavePanel *theSavePanel;
    
    // get a save panel object
    theSavePanel = [NSSavePanel savePanel];
    
    // save the selected file(s) to disk
    [theSavePanel beginSheetForDirectory:nil file:[[[[[[[previewTabView selectedTabViewItem] view] documentView] selectedCell] representedObject] storagePath] lastPathComponent] modalForWindow:[self window] modalDelegate:self didEndSelector:@selector(performSave:returnCode:contextInfo:) contextInfo:nil];
    
}

- (void)performSave:(NSSavePanel *)sheet returnCode:(int)returnCode contextInfo:(void  *)contextInfo
{
    // called after user fills out the save panel
    if(returnCode == NSOKButton){
        // copy file from current location to new location - already challenged if file overwrite
        if([[NSFileManager defaultManager] fileExistsAtPath:[sheet filename]])
            [[NSFileManager defaultManager] removeFileAtPath:[sheet filename] handler:nil];
       
        if(![[NSFileManager defaultManager] copyPath:[[[[[[previewTabView selectedTabViewItem] view] documentView] selectedCell] representedObject] storagePath] toPath:[sheet filename] handler:nil])
            NSBeep();
    }
}

#pragma mark -
#pragma mark Printing Files

- (IBAction)printDocument:(id)sender
{
    // find placed viewer
    NSEnumerator *e = [_mediaViewers objectEnumerator];
    NSView *v, *printView;
    while(v = [e nextObject]){
        if([v superview] != nil)
            printView = v;
    }
    
    // figure what to print
    NSPrintOperation *op;
    if(printView == miniWebBrowser)
        op = [NSPrintOperation printOperationWithView:theWebView];
    
    if(printView == imageViewer)
        op = [NSPrintOperation printOperationWithView:bigImageView];
    
    [op setShowPanels:YES];
    [op runOperationModalForWindow:[self window] delegate:self didRunSelector:@selector(printOperationDidRun:success:contextInfo:) contextInfo:NULL];
}

- (void)printOperationDidRun:(NSPrintOperation *)printOperation success:(BOOL)success contextInfo:(void *)info
{
    if (success) {
        // Can save updated NSPrintInfo, but only if you have
        // a specific reason for doing so
        // [self setPrintInfo: [printOperation printInfo]];
    }
}


#pragma mark -
#pragma mark Slide Show

- (void)toggleSlideShow:(id)sender
{
    if(slideShow != nil){
        [self stopSlideShow:sender];
    } else {
        [self startSlideShow:sender];
    }
}

- (void)startSlideShow:(id)sender
{
    if([[NSUserDefaults standardUserDefaults] boolForKey:@"Slideshow full screen"]){
        // going full screen
        int windowLevel;
        NSRect screenRect = [[NSScreen mainScreen] frame];
        
        slideShow = [[PSMGLSlideShowView alloc] initWithFrame:screenRect];
        [slideShow setImageList:[[[[previewTabView selectedTabViewItem] view] documentView] slideShowPaths]];
        [slideShow setAutoresizingMask:(NSViewWidthSizable | NSViewHeightSizable)];
        
        // Capture the main display
        if (CGDisplayCapture( kCGDirectMainDisplay ) != kCGErrorSuccess) {
            NSLog( @"Couldn't capture the main display!" );
            // Note: you'll probably want to display a proper error dialog here
        }
        
        // Get the shielding window level
        windowLevel = CGShieldingWindowLevel();
        
        // here's the window
        SlideshowWindow *fullScreenWindow = [[SlideshowWindow alloc] initWithContentRect:screenRect
                                                                styleMask:NSBorderlessWindowMask
                                                                  backing:NSBackingStoreBuffered
                                                                    defer:NO
                                                                   screen:[NSScreen mainScreen]];
        [fullScreenWindow setLevel:windowLevel];
        [fullScreenWindow setContentView:slideShow];
        [fullScreenWindow makeKeyAndOrderFront:nil];
        
    } else {
        [tabBarControl setHidden:YES];
        NSRect newSplit = [mainSplitView frame];
        [mainSplitView setFrame:NSMakeRect(newSplit.origin.x, newSplit.origin.y, newSplit.size.width, newSplit.size.height + 22)];
        [mainSplitView setSubview:[[mainSplitView subviews] objectAtIndex:0] isCollapsed:YES];
        
        // remove the existing view
        NSRect theFrame = [bigScrollView frame];
        
        // allocate a slideshow view
        slideShow = [[PSMGLSlideShowView alloc] initWithFrame:theFrame];
        [slideShow setImageList:[[[[previewTabView selectedTabViewItem] view] documentView] slideShowPaths]];
        [slideShow setAutoresizingMask:(NSViewWidthSizable | NSViewHeightSizable)];
        
        [bigScrollView retain];
        [[bigScrollView superview] replaceSubview:bigScrollView with:slideShow];
        
        // clear the status area, hide the buttons
        [parentURLTextField setStringValue:@""];
        
        // put away previews
        [mainSplitView setSubview:[[mainSplitView subviews] objectAtIndex:0] isCollapsed:YES];
    }
}

- (void)stopSlideShow:(id)sender
{
    if([[NSUserDefaults standardUserDefaults] boolForKey:@"Slideshow full screen"]){
        // we were full screen
        [slideShow killTimers];
        [slideShow release];
        slideShow = nil;
        NSWindow *fullScreenWindow = [NSApp keyWindow];
        [fullScreenWindow orderOut:self];
        [fullScreenWindow release];
        
        // Release the display(s)
        if (CGDisplayRelease( kCGDirectMainDisplay ) != kCGErrorSuccess) {
            NSLog( @"Couldn't release the display(s)!" );
            // Note: if you display an error dialog here, make sure you set
            // its window level to the same one as the shield window level,
            // or the user won't see anything.
        }
    } else {
        // bring back preview view
        [tabBarControl setHidden:NO];
        NSRect newSplit = [mainSplitView frame];
        [mainSplitView setFrame:NSMakeRect(newSplit.origin.x, newSplit.origin.y, newSplit.size.width, newSplit.size.height - 22)];
        [mainSplitView setSubview:[[mainSplitView subviews] objectAtIndex:0] isCollapsed:NO];
        
        // remove the slide show view
        NSRect theFrame = [slideShow frame];
        [bigScrollView setFrame:theFrame];
        [[slideShow superview] replaceSubview:slideShow with:bigScrollView];
        [slideShow killTimers];
        [slideShow release];
        slideShow = nil;
        
        // click myself for URL text
        [self userClick:[[[previewTabView selectedTabViewItem] view] documentView]];
    }
}

#pragma mark -
#pragma mark Menu Validation

- (BOOL)validateMenuItem:(NSMenuItem*)anItem
{
    if ([anItem action] == @selector(toggleSlideShow:)){
        // set the title of the menu based on current state
        if(slideShow != nil){
            [anItem setTitle:@"Stop Slideshow"];
        } else {
            [anItem setTitle:@"Start Slideshow"];
        }
        
        if([[[previewTabView selectedTabViewItem] label] isEqualToString:@"Untitled"])
            return NO;
        
        // see if there are any tabs allotted and images present
        // must be more than zero slides
        if([[[[[previewTabView selectedTabViewItem] view] documentView] slideShowPaths] count] > 0){
            return YES;
        }
        return NO;
    }
    if ([anItem action] == @selector(performMiniaturize:)){
        //NSLog(@"called");
        return YES;
    }
    
    if ([anItem action] == @selector(performZoom:)){
        return YES;
    }
    
    if ([anItem action] == @selector(closeWindow:)){
        if([_sessionArray count] > 0){
            [anItem setKeyEquivalent:@"w"];
            [anItem setKeyEquivalentModifierMask:(NSCommandKeyMask | NSShiftKeyMask)];
        } else {
            [anItem setKeyEquivalent:@"w"];
            [anItem setKeyEquivalentModifierMask:NSCommandKeyMask];
        }
        
        return YES;
    }
    
    if([anItem action] == @selector(zoomScale:)){
        if([[anItem representedObject] floatValue] == [bigScrollView scaleFactor]){
            [anItem setState:NSOnState];
        } else {
            [anItem setState:NSOffState];
        }
        
        if([imageViewer superview] != nil){
            return YES;
        }
        
        return NO;
    }
    
    if ([anItem action] == @selector(closeTab:)){
        if([_sessionArray count] < 1){
            [anItem setKeyEquivalent:@""];
            [anItem setKeyEquivalentModifierMask:NSCommandKeyMask];
            return NO;
        } else {
            [anItem setKeyEquivalent:@"w"];
            [anItem setKeyEquivalentModifierMask:NSCommandKeyMask];
        }
        
        return YES;
    }
    
    if ([anItem action] == @selector(nextTab:)){
        // see if there is another tab
        if([previewTabView indexOfTabViewItem:[previewTabView selectedTabViewItem]] < [previewTabView numberOfTabViewItems] - 1){
            return YES;
        }
        return NO;
    }
    
    if ([anItem action] == @selector(previousTab:)){
        // see if there is previous tab
        if([previewTabView numberOfTabViewItems] > 0){
            if([previewTabView indexOfTabViewItem:[previewTabView selectedTabViewItem]] > 0){
                return YES;
            }
        }
        return NO;
    }
    
    if ([anItem action] == @selector(openFiles:)){
        return YES; // always good
    }
    
    if ([anItem action] == @selector(addNewTab:)){
        return YES; // always good
    }
    
    if ([anItem action] == @selector(showInFinder:)){
        // must be registered and select valid object
        if([[[previewTabView selectedTabViewItem] label] isEqualToString:@"Untitled"])
            return NO;
        
        if([_sessionArray count] > 0){
            if([[NSApp delegate] isRegistered] && ([[[[[previewTabView selectedTabViewItem] view] documentView] selectedCell] representedObject] != nil)){
                return YES;
            }
        }
        return NO;
    }
    
    if ([anItem action] == @selector(viewWebPage:)){
        // must select valid object from a web scrape, not a file scrape
        // must have a selected item with a URL and not a file URL
        if([_sessionArray count] == 0){
            return NO;
        }
        
        if([[[previewTabView selectedTabViewItem] label] isEqualToString:@"Untitled"])
            return NO;
        
        if([[[[[[previewTabView selectedTabViewItem] view] documentView] selectedCell] representedObject] parentWebPage] != nil){
            return YES;
        } else {
            return NO;
        }
    }
    
    if ([anItem action] == @selector(toggleScraping:)){
        // current tab must have active session
        if([[[previewTabView selectedTabViewItem] label] isEqualToString:@"Untitled"])
            return NO;
        
        if([previewTabView numberOfTabViewItems] > 0){
            if([[[[[previewTabView selectedTabViewItem] view] documentView] engine] isDone]){
                [anItem setTitle:@"Stop Loading"];
                [anItem setKeyEquivalent:@"."];
                [anItem setKeyEquivalentModifierMask:NSCommandKeyMask];
                return NO;
            }
        }
        
        if([_sessionArray count] > 0){
            if([[[[[previewTabView selectedTabViewItem] view] documentView] engine] isProcessing]){
                [anItem setTitle:@"Stop Loading"];
                [anItem setKeyEquivalent:@"."];
                [anItem setKeyEquivalentModifierMask:NSCommandKeyMask];
            } else {
                [anItem setTitle:@"Resume Loading"];
                [anItem setKeyEquivalent:@"r"];
                [anItem setKeyEquivalentModifierMask:NSCommandKeyMask];
            }
            return YES;
        }
        return NO;
    }
    
    if ([anItem action] == @selector(saveFileToDisk:)){
        // must select valid object
        if([[[previewTabView selectedTabViewItem] label] isEqualToString:@"Untitled"])
            return NO;
        
        if([_sessionArray count] > 0){
            if([[[[[[previewTabView selectedTabViewItem] view] documentView] selectedCell] representedObject] storagePath] != nil){
                return YES;
            }
        }
        return NO;
    }
    
    if ([anItem action] == @selector(setDesktopPicture:)){
        // must select valid object
        if([[[previewTabView selectedTabViewItem] label] isEqualToString:@"Untitled"])
            return NO;
        
        if([_sessionArray count] > 0){
            if([[NSBitmapImageRep imageUnfilteredFileTypes] containsObject:[[[[[[[previewTabView selectedTabViewItem] view] documentView] selectedCell] representedObject] storagePath] pathExtension]]){
                return YES;
            }
        }
        return NO;
    }
    
    if ([anItem action] == @selector(printDocument:)){
        // something must be present in the main view
        return YES;
    }
    
    if ([anItem action] == @selector(toggleToolbar:)){
        // set the title of the menu based on current state
        if([toolbar isVisible]){
            [anItem setTitle:@"Hide Toolbar"];
        } else {
            [anItem setTitle:@"Show Toolbar"];
        }
        return YES;
    }
    
    if ([anItem action] == @selector(customizeToolbar:)){
        return YES; // always good
    }
    
    if ([anItem action] == @selector(togglePreviewView:)){
        // set the title of the menu based on current state
        if(![mainSplitView isSubviewCollapsed:[previewTabView superview]]){
            [anItem setTitle:@"Hide Preview"];
        } else {
            [anItem setTitle:@"Show Preview"];
        }
        return YES;
    }
    
    if ([anItem action] == @selector(toggleLog:)){
        return YES;
    }
    
    
    // return YES for anything else
    return YES;
}

#pragma mark -
#pragma mark Utility

- (void)placeViewer:(NSView *)view
{
    // remove any previously placed viewer
    NSEnumerator *e = [_mediaViewers objectEnumerator];
    NSView *v;
    while(v = [e nextObject]){
        if([v superview] != nil)
            [v removeFromSuperview];
    }
    
    // place new viewer
    NSRect properRect = [mainPane bounds];
    properRect.size.height -= (kPSMBottomWindowMargin - 1);
    properRect.origin.y += (kPSMBottomWindowMargin -1);
    [view setFrame:properRect];
    [mainPane addSubview:view];
}

- (void)sendFileAtPath:(NSString *)path withEventClass:(OSType)eventClass eventKind:(OSType)eventKind toAppWithSignature:(OSType)targetSignature activating:(BOOL)activateTarget
{
    AppleEvent event, reply; 
    OSErr err; 
    FSRef fileRef; 
    AliasHandle fileAlias; 
    
    if (activateTarget == YES)
    {
        err = AEBuildAppleEvent('misc', kAEActivate, typeApplSignature, &targetSignature, sizeof(targetSignature), kAutoGenerateReturnID, kAnyTransactionID, &event, NULL, ""); 
        if (err == noErr)
        {
            err = AESend(&event, &reply, kAEWaitReply, kAENormalPriority, kAEDefaultTimeout, NULL, NULL);
            if (err == noErr)
            {
                AEDisposeDesc(&reply);
            }
            AEDisposeDesc(&event);	
        }
    }
    
    // Create an FSRef to our file system representation
    err = FSPathMakeRef((const UInt8 *)[path fileSystemRepresentation], &fileRef, NULL); 
    if (err == noErr)
    {
        err = FSNewAliasMinimal(&fileRef, &fileAlias);
        if (err == noErr)
        {
            err = AEBuildAppleEvent(eventClass, eventKind, typeApplSignature, &targetSignature, sizeof(targetSignature), kAutoGenerateReturnID, kAnyTransactionID, &event, NULL, "'----':alis(@@)", fileAlias); 
            if (err == noErr)
            {
                err = AESend(&event, &reply, kAEWaitReply, kAENormalPriority, kAEDefaultTimeout, NULL, NULL); 
                if (err == noErr)
                {
                    AEDisposeDesc(&reply); 
                }				
                AEDisposeDesc(&event);
            }	
            
            DisposeHandle((Handle)fileAlias);
        }
    }
    return; 
}

OSErr SetDesktopPicture(NSString *picturePath,SInt32 pIndex)
{
    AEDesc tAEDesc = {typeNull, nil};  //  always init AEDescs
    OSErr    anErr = noErr;
    AliasHandle    aliasHandle=nil;
    FSRef    pictRef;
    OSStatus    status;
    
    // Someday pIndex will hopefully determine on which monitor to set the desktop picture.
    // This doesn't work in Mac OS X right now, so we don't do anything with the parameter.
#pragma unused (pIndex)
    
    // Let's make an FSRef from the NSString picture path that was passed in
    status = FSPathMakeRef((UInt8 *)[picturePath fileSystemRepresentation],&pictRef,NULL);
    
    // Now we create an alias to the picture from that FSRef
    if (status==noErr)
        anErr = FSNewAlias( nil, &pictRef, &aliasHandle);
    
    if ( noErr == anErr  &&  aliasHandle == nil )
        anErr = paramErr;
    
    // Now we create an AEDesc containing the alias to the picture
    if ( noErr == anErr )
    {
        char  handleState = HGetState( (Handle) aliasHandle );
        HLock( (Handle) aliasHandle);
        anErr = AECreateDesc( typeAlias, *aliasHandle, GetHandleSize((Handle) aliasHandle), &tAEDesc);
        HSetState( (Handle) aliasHandle, handleState );
        DisposeHandle( (Handle)aliasHandle );
    }
    if (noErr == anErr)
    {
        // Now we need to build the actual Apple Event that we're going to send the Finder
        AppleEvent tAppleEvent;
        OSType sig = 'MACS'; // The app signature for the Finder
        AEBuildError tAEBuildError;
        anErr = AEBuildAppleEvent(kAECoreSuite,kAESetData,typeApplSignature,&sig,sizeof(OSType),
                                  kAutoGenerateReturnID,kAnyTransactionID,&tAppleEvent,&tAEBuildError,
                                  "'----':'obj '{want:type(prop),form:prop,seld:type('dpic'),from:'null'()},data:(@)",&tAEDesc);
        
        // Finally we can go ahead and send the Apple Event using AESend                          
        if (noErr == anErr)
        {
            AppleEvent    theReply = {typeNull, nil};
            anErr = AESend(&tAppleEvent,&theReply,kAENoReply,kAENormalPriority,kNoTimeOut,nil,nil);
            AEDisposeDesc(&tAppleEvent);    // Always dispose ASAP
        }
    }
    return anErr;
}

#pragma mark -
#pragma mark Special Window Appearance

- (void)paintWindow
{
    NSColor *bgColor = [[self window] backgroundColor];
    NSImage *bg = [[NSImage alloc] initWithSize:[[self window] frame].size];
    
    [bg lockFocus];
    
    // Composite current background color into bg
    [bgColor set];
    NSRectFill(NSMakeRect(0, 0, [bg size].width, [bg size].height));
    
    NSImage *topMiddleImage;
    NSImage *bottomMiddleImage;
    
    if([[self window] isMainWindow]){
        topMiddleImage = [NSImage imageNamed:@"top_middle2main"];
        bottomMiddleImage = [NSImage imageNamed:@"bottom_middle2main"];
    } else {
        topMiddleImage = [NSImage imageNamed:@"top_middle2bg"];
        bottomMiddleImage = [NSImage imageNamed:@"bottom_middle2bg"];
    }
        
    
    // Composite top "pattern"
    NSColor *topMiddle = [NSColor colorWithPatternImage:topMiddleImage];
    [topMiddle set];
    [[NSGraphicsContext currentContext] setPatternPhase:NSMakePoint([bg size].width, [bg size].height)];
    NSRectFill(NSMakeRect(0, [bg size].height - [topMiddleImage size].height, [bg size].width, [topMiddleImage size].height));
    
    // Composite bottom-middle "pattern"
    NSColor *bottomMiddle = [NSColor colorWithPatternImage:bottomMiddleImage];
    [bottomMiddle set];
    [[NSGraphicsContext currentContext] setPatternPhase:NSMakePoint(0, [bottomMiddleImage size].height)];
    NSRectFill(NSMakeRect(0, 0, [bg size].width, [bottomMiddleImage size].height));
    
    [bg unlockFocus];
    
    [[self window] setBackgroundColor:[NSColor colorWithPatternImage:bg]];
    
    [bg release];
}
@end