//
//  PasteController.h
//  NetScrape
//
//  Created by John Pannell on 10/7/05.
//  Copyright 2005 Positive Spin Media. All rights reserved.
//
/*

PasteController objects are responsible for taking a list of file paths, creating PSMWebFile objects, and injecting them into their assigned ThumbnailMatrix.  They must process files sequentially for performance reasons.  They may inject files at a specified index in the matrix, or add them to the end of the matrix.
 
 */

#import <Cocoa/Cocoa.h>
#import "ThumbnailMatrix.h"
#import "PSMWebFile.h"


@interface PasteController : NSObject {
    NSMutableArray              *_fileList;         // full paths for each image
    ThumbnailMatrix             *_thumbnailMatrix;  // matrix to inject objects into
    int                         _injectionIndex;    // location to insert pasted objects
}

// designated initializer
- (id)initWithFiles:(NSMutableArray *)fileList matrix:(ThumbnailMatrix *)matrix injectionIndex:(int)injectionIndex;

// accessors
- (NSMutableArray *)fileList;
- (void)setFileList:(NSMutableArray *)fileList;
- (ThumbnailMatrix *)thumbnailMatrix;
- (void)setThumbnailMatrix:(ThumbnailMatrix *)matrix;
- (int)injectionIndex;
- (void)setInjectionIndex:(int)value;
- (NSString *)myString;

// process
- (void)startPaste;
- (void)loadFile;
- (void)addScrapedFile:(PSMWebFile *)object;
- (NSMutableArray *)fileList;

@end
