//
//  PasteController.m
//  NetScrape
//
//  Created by John Pannell on 10/7/05.
//  Copyright 2005 Positive Spin Media. All rights reserved.
//

#import "PasteController.h"
#import "PSMWebFile.h"
#import "PSMPandoraSession.h"


@implementation PasteController

// designated initializer
- (id)initWithFiles:(NSMutableArray *)fileList matrix:(ThumbnailMatrix *)matrix injectionIndex:(int)injectionIndex
{
    if(!(self = [super init])){
        return nil;
    }
    [fileList retain];
    [matrix retain];
    _fileList = fileList;
    _thumbnailMatrix = matrix;
    _injectionIndex = injectionIndex;
    return self;
}

- (void)dealloc
{
    [_fileList release];
    [_thumbnailMatrix release];
    [super dealloc];
}

#pragma mark ---- accessors ----

- (NSMutableArray *)fileList
{
    return _fileList;
}

- (void)setFileList:(NSMutableArray *)fileList
{
    [fileList retain];
    [_fileList release];
    _fileList = fileList;
}

- (ThumbnailMatrix *)thumbnailMatrix
{
    return _thumbnailMatrix;
}

- (void)setThumbnailMatrix:(ThumbnailMatrix *)matrix
{
    [matrix retain];
    [_thumbnailMatrix release];
    _thumbnailMatrix = matrix;
}

- (int)injectionIndex
{
    return _injectionIndex;
}

- (void)setInjectionIndex:(int)value
{
    _injectionIndex = value;
}
- (NSString *)myString
{
    return nil;
}

#pragma mark ---- process ----

- (void)startPaste
{
    // user has pasted list of files into view
    [self loadFile];
}

- (void)loadFile
{
    // create scrapedFile object - start at end for injection
    if([[NSFileManager defaultManager] fileExistsAtPath:[[self fileList] lastObject]]){
        PSMWebFile *scrapedFile = [[PSMWebFile alloc] initWithSourceURL:[[self fileList] lastObject]];
        [scrapedFile setStoragePath:[[self fileList] lastObject]];
        [scrapedFile setParentWebPage:nil];
        [NSThread detachNewThreadSelector:@selector(loadFileData:) toTarget:scrapedFile withObject:self];
        [[self fileList] removeLastObject];
    } else {
        [[self fileList] removeLastObject];
        [self addScrapedFile:nil];
    }
}

- (void)addScrapedFile:(PSMWebFile *)object
{
    // ding! my file is ready!
    [object retain];
    if([self thumbnailMatrix] && (object != nil)){
        [[[[self thumbnailMatrix] engine] scrapedFiles] insertObject:object atIndex:[self injectionIndex]]; // add to model
        int currentCount = (int)[[[self thumbnailMatrix] engine] objectCount];
        currentCount++;
        [[[[self thumbnailMatrix] engine] objectController] setValue:[NSNumber numberWithInt:currentCount] forKeyPath:@"selection.objectCount"]; // increment object count
        [[self thumbnailMatrix] injectPastedFile:object atIndex:[self injectionIndex]];
    }
    [object release];
    
    // if there are still more files, load another
    if([[self fileList] count] > 0){
        [self loadFile];
    } else {
        // we are done
        [self release];
    }
}

@end
