//
//  PSMPandoraSession.h
//  Pandora
//
//  Created by John Pannell on 5/25/06.
//  Copyright 2006 Positive Spin Media. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import "PandoraSession.h"
#import "PandoraWebSpider.h"
@class ThumbnailMatrix;

@interface PSMPandoraSession : NSObject <PandoraSession> {
    
    // spider connections
    int                         _sessionID;
    NSString                    *_broadcastName;
    id<PandoraWebSpider>        _engine;                // proxy for spider running in separate process
    NSFileHandle                *theOutput;
    NSString                    *_rootString;
    NSString                    *_pluginName;
    
    // bound values (for display in tabs)
    NSObjectController          *_objectController;     // for bindings
    BOOL                        _isProcessing;
    BOOL                        _userPaused;            // different than network/sleep paused
    BOOL                        _isDone;                // spider has finished - nothing left
    NSImage                     *_icon;
    int                         _objectCount;
    
    // the model objects
    NSMutableArray              *_scrapedFiles;
    
    // the view object
    ThumbnailMatrix             *_thumbnailMatrix;
}

// creation/destruction
- (id)initWithSessionID:(int)value;

// accessors
- (NSString *)rootString;
- (void)setRootString:(NSString *)string;
- (NSString *)pluginName;
- (void)setPluginName:(NSString *)name;
- (BOOL)isProcessing;
- (void)setIsProcessing:(BOOL)value;
- (BOOL)isDone;
- (void)setIsDone:(BOOL)flag;
- (int)objectCount;
- (void)setObjectCount:(int)value;
- (NSImage *)icon;
- (void)setIcon:(NSImage *)icon;
- (NSMutableArray *)scrapedFiles;
- (void)setScrapedFiles:(NSMutableArray *)array;
- (NSObjectController *)objectController;
- (ThumbnailMatrix *)thumbnailMatrix;
- (void)setThumbnailMatrix:(ThumbnailMatrix *)matrix;
- (int)sessionID;

// functionality
- (void)pause:(id)sender;
- (void)resume:(id)sender;
- (void)startEngineWithRootString:(NSString *)root andPlugin:(NSString *)name;
- (void)engineStarted:(NSNotification *)aNotification;
- (void)pauseEngine;
- (void)dumpSnapshot;
- (void)resumeEngine;
- (void)stopEngine;

// from the spider
- (oneway void)addWebFile:(in bycopy PSMWebFile *)file;

// logging
- (oneway void)logConsoleMessageWithURL:(in bycopy NSString *)url status:(in bycopy NSString *)status andComment:(in bycopy NSString *)comment;
- (oneway void)logNewResourceID:(in bycopy int)resourceID withURL:(in bycopy NSString *)url;
- (oneway void)logMIMEType:(in bycopy NSString *)type forResourceID:(in bycopy int)resourceID;
- (oneway void)logStatus:(in bycopy NSString*)status forResourceID:(in bycopy int)resourceID;
- (oneway void)logProgress:(in bycopy NSString *)progress forResourceID:(in bycopy int)resourceID;
- (oneway void)logComment:(in bycopy NSString *)comment forResourceID:(in bycopy int)resourceID;
- (oneway void)logIdentifyTime:(in bycopy NSString *)comment forResourceID:(in bycopy int)resourceID;
- (oneway void)logParseTime:(in bycopy NSString *)comment forResourceID:(in bycopy int)resourceID;
- (oneway void)logLoadTime:(in bycopy NSString *)comment forResourceID:(in bycopy int)resourceID;
- (oneway void)logThumbnailTime:(in bycopy NSString *)comment forResourceID:(in bycopy int)resourceID;

@end
