//
//  PSMPandoraSession.m
//  Pandora
//
//  Created by John Pannell on 5/25/06.
//  Copyright 2006 Positive Spin Media. All rights reserved.
//

#import "PSMPandoraSession.h"
#import "ThumbnailMatrix.h"
#import "PandoraWebSpider.h"
#import "PSMAppController.h"


@implementation PSMPandoraSession

#pragma mark -
#pragma mark Creation/destruction

- (id)initWithSessionID:(int)value
{
    if(self = [super init]){
        _sessionID = value;
        _broadcastName = [[NSString alloc] initWithFormat:@"com.positivespinmedia.Pandora.%d", value];
        _objectController = [[NSObjectController alloc] initWithContent:self];
        _scrapedFiles = [[NSMutableArray alloc] init];
        _userPaused = NO;
        _isDone = NO;
    }
    return self;
}

#pragma mark -
#pragma mark Accessors

- (NSString *)rootString
{
    return _rootString;
}

- (void)setRootString:(NSString *)string
{
    [string retain];
    [_rootString release];
    _rootString = string;
}

- (NSString *)pluginName
{
    return _pluginName;
}

- (void)setPluginName:(NSString *)name
{
    [name retain];
    [_pluginName release];
    _pluginName = name;
}

- (BOOL)isProcessing
{
    return _isProcessing;
}

- (void)setIsProcessing:(BOOL)value
{
    _isProcessing = value;
}

- (BOOL)isDone
{
    return _isDone;
}

- (void)setIsDone:(BOOL)flag
{
    _isDone = flag;
    [_objectController setValue:[NSNumber numberWithBool:NO] forKeyPath:@"selection.isProcessing"];
}

- (int)objectCount
{
    return _objectCount;
}

- (void)setObjectCount:(int)value
{
    _objectCount = value;
}

- (NSImage *)icon
{
    return _icon;
}

- (void)setIcon:(NSImage *)icon
{
    [icon retain];
    [_icon release];
    _icon = icon;
}

- (NSMutableArray *)scrapedFiles
{
    return _scrapedFiles;
}

- (void)setScrapedFiles:(NSMutableArray *)array
{
    [array retain];
    [_scrapedFiles release];
    _scrapedFiles = array;
}

- (NSObjectController *)objectController
{
    return _objectController;
}

- (ThumbnailMatrix *)thumbnailMatrix
{
    return _thumbnailMatrix;
}

- (void)setThumbnailMatrix:(ThumbnailMatrix *)matrix
{
    [matrix retain];
    [_thumbnailMatrix release];
    _thumbnailMatrix = matrix;
}

- (int)sessionID
{
    return _sessionID;
}


#pragma mark -
#pragma mark Functionality

- (void)pause:(id)sender
{
    [self pauseEngine];
    _userPaused = YES;
}

- (void)resume:(id)sender
{
    _userPaused = NO;
    [self resumeEngine];
}

- (void)startEngineWithRootString:(NSString *)root andPlugin:(NSString *)name
{
    [self setRootString:root];
    [self setPluginName:name];
    [_objectController setValue:[NSNumber numberWithBool:YES] forKeyPath:@"selection.isProcessing"];
    NSArray *theArguments = [NSArray arrayWithObjects:root, name, _broadcastName, nil];
    NSTask *theTask = [[NSTask alloc] init];
    NSPipe *thePipe = [NSPipe pipe];
    
    [theTask setLaunchPath:[NSBundle pathForResource:@"PandoraSpider" ofType:nil inDirectory:[[[NSBundle mainBundle] bundlePath] stringByAppendingString:@"/Contents/Resources/PandoraSpider.app/Contents/MacOS/"]]];
    [theTask setCurrentDirectoryPath:[[[NSBundle mainBundle] bundlePath] stringByAppendingString:@"/Contents/Resources/"]];
    [theTask setArguments:theArguments];
    [theTask setStandardOutput:thePipe];
    [theTask launch];
    
    //NSLog(@"task started");
    
    // get notified when engine is done launching
    theOutput = [[thePipe fileHandleForReading] retain];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(engineStarted:) name:NSFileHandleReadCompletionNotification object:theOutput];
    [theOutput readInBackgroundAndNotify];
}

- (void)engineStarted:(NSNotification *)aNotification
{
    // is the subtask running yet?
    //NSLog(@"task pipe says: %@", [NSString stringWithCString:[[[aNotification userInfo] objectForKey:NSFileHandleNotificationDataItem] bytes]]);
    if([[NSString stringWithCString:[[[aNotification userInfo] objectForKey:NSFileHandleNotificationDataItem] bytes]] rangeOfString:@"running"].length > 0){
        // connect to vended spider object
        _engine = [[NSConnection rootProxyForConnectionWithRegisteredName:_broadcastName host:nil] retain];
        //NSLog(@"task engine: %@", _engine);
        [(NSDistantObject *)_engine setProtocolForProxy:@protocol(PandoraWebSpider)];
        [_engine setSessionObject:self];
        [self logConsoleMessageWithURL:[NSString stringWithFormat:@"Session responding at %@", _broadcastName] status:@"Connected" andComment:[[NSDate date] description]];
        [_engine startProcessing];
        [_objectController setValue:[NSNumber numberWithInt:0] forKeyPath:@"selection.objectCount"];
    } else {
        // reset
        [theOutput readInBackgroundAndNotify];
    }
}

- (void)pauseEngine
{
    if(_isProcessing && !_userPaused){
        [_engine pauseProcessing];
        [_objectController setValue:[NSNumber numberWithBool:NO] forKeyPath:@"selection.isProcessing"];
    }
}

- (void)dumpSnapshot
{
    [_engine dumpSnapshot];
}

- (void)resumeEngine
{
    if(!_isProcessing && !_userPaused){
        [_engine resumeProcessing];
        [_objectController setValue:[NSNumber numberWithBool:YES] forKeyPath:@"selection.isProcessing"];
    }
}

- (void)stopEngine
{
    [_engine stopProcessing];
    [_engine release];
    // note that we are no longer running a task
    [_objectController setValue:[NSNumber numberWithBool:NO] forKeyPath:@"selection.isProcessing"];
}

#pragma mark -
#pragma mark From the spider

- (oneway void)addWebFile:(in bycopy PSMWebFile *)file
{
    // keep it around
    [_scrapedFiles addObject:file];
    [_objectController setValue:[NSNumber numberWithInt:[self objectCount]+1] forKeyPath:@"selection.objectCount"];
    
    // add the image to the view
    if(_thumbnailMatrix){
        [_thumbnailMatrix addScrapedFile:file];
    }
}

#pragma mark -
#pragma mark Logging

- (oneway void)logConsoleMessageWithURL:(in bycopy NSString *)url status:(in bycopy NSString *)status andComment:(in bycopy NSString *)comment
{
    NSMutableDictionary *logEntry = [NSMutableDictionary dictionaryWithObjectsAndKeys:@"0", @"sessionID", @"0", @"resourceID", url, @"resourceURL", @"", @"resourceMIME", status, @"resourceStatus", @"", @"resourceProgress", comment, @"resourceComment", nil];
    [[[NSApp delegate] logController] addObject:logEntry];
    [[[NSApp delegate] filteredLog] fetch:self];
}

- (oneway void)logNewResourceID:(in bycopy int)resourceID withURL:(in bycopy NSString *)url
{
    NSMutableDictionary *logEntry = [NSMutableDictionary dictionaryWithObjectsAndKeys:[NSString stringWithFormat:@"%d", _sessionID], @"sessionID", [NSString stringWithFormat:@"%d", resourceID], @"resourceID", url, @"resourceURL", @"", @"resourceMIME", @"Found", @"resourceStatus", @"", @"resourceProgress", @"", @"resourceComment", @"", @"resourceIdentifyTime", @"", @"resourceLoadTime", @"", @"resourceParseTime", @"", @"resourceThumbnailTime", nil];
    [[[NSApp delegate] logController] addObject:logEntry];
    [[[NSApp delegate] filteredLog] fetch:self];
    
    // update filter criteria
    if(![[[[NSApp delegate] statusIDController] arrangedObjects] containsObject:@"Found"])
        [[[NSApp delegate] statusIDController] addObject:@"Found"];
}

- (oneway void)logMIMEType:(in bycopy NSString *)type forResourceID:(in bycopy int)resourceID
{
    // update the appropriate resource
    NSPredicate *filterPredicate = [NSPredicate predicateWithFormat:@"(sessionID == %@) AND (resourceID == %@)", [NSString stringWithFormat:@"%d", _sessionID], [NSString stringWithFormat:@"%d", resourceID]];
    NSArray *tempArray = [[[[NSApp delegate] logController] arrangedObjects] filteredArrayUsingPredicate:filterPredicate];
    if([tempArray count] > 0){
        NSMutableDictionary *tempEntry = [tempArray objectAtIndex:0];
        [tempEntry setValue:type forKey:@"resourceMIME"];
        [[[NSApp delegate] filteredLog] fetch:self];
    }
    
    // update filter criteria
    if(![[[[NSApp delegate] MIMEIDController] arrangedObjects] containsObject:type])
        [[[NSApp delegate] MIMEIDController] addObject:type];
}

- (oneway void)logStatus:(in bycopy NSString*)status forResourceID:(in bycopy int)resourceID
{
    // update the appropriate resource
    NSPredicate *filterPredicate = [NSPredicate predicateWithFormat:@"(sessionID == %@) AND (resourceID == %@)", [NSString stringWithFormat:@"%d", _sessionID], [NSString stringWithFormat:@"%d", resourceID]];
    NSArray *tempArray = [[[[NSApp delegate] logController] arrangedObjects] filteredArrayUsingPredicate:filterPredicate];
    if([tempArray count] > 0){
        NSMutableDictionary *tempEntry = [tempArray objectAtIndex:0];
        [tempEntry setValue:status forKey:@"resourceStatus"];
        [[[NSApp delegate] filteredLog] fetch:self];
    }
    
    // update filter criteria
    if(![[[[NSApp delegate] statusIDController] arrangedObjects] containsObject:status])
        [[[NSApp delegate] statusIDController] addObject:status];
}

- (oneway void)logProgress:(in bycopy NSString *)progress forResourceID:(in bycopy int)resourceID
{
    NSPredicate *filterPredicate = [NSPredicate predicateWithFormat:@"(sessionID == %@) AND (resourceID == %@)", [NSString stringWithFormat:@"%d", _sessionID], [NSString stringWithFormat:@"%d", resourceID]];
    NSArray *tempArray = [[[[NSApp delegate] logController] arrangedObjects] filteredArrayUsingPredicate:filterPredicate];
    if([tempArray count] > 0){
        NSMutableDictionary *tempEntry = [tempArray objectAtIndex:0];
        [tempEntry setValue:progress forKey:@"resourceProgress"];
        [[[NSApp delegate] filteredLog] fetch:self];
    }
}

- (oneway void)logComment:(in bycopy NSString *)comment forResourceID:(in bycopy int)resourceID
{
    NSPredicate *filterPredicate = [NSPredicate predicateWithFormat:@"(sessionID == %@) AND (resourceID == %@)", [NSString stringWithFormat:@"%d", _sessionID], [NSString stringWithFormat:@"%d", resourceID]];
    NSArray *tempArray = [[[[NSApp delegate] logController] arrangedObjects] filteredArrayUsingPredicate:filterPredicate];
    if([tempArray count] > 0){
        NSMutableDictionary *tempEntry = [tempArray objectAtIndex:0];
        [tempEntry setValue:comment forKey:@"resourceComment"];
        [[[NSApp delegate] filteredLog] fetch:self];
    }
}

- (oneway void)logIdentifyTime:(in bycopy NSString *)comment forResourceID:(in bycopy int)resourceID
{
    NSPredicate *filterPredicate = [NSPredicate predicateWithFormat:@"(sessionID == %@) AND (resourceID == %@)", [NSString stringWithFormat:@"%d", _sessionID], [NSString stringWithFormat:@"%d", resourceID]];
    NSArray *tempArray = [[[[NSApp delegate] logController] arrangedObjects] filteredArrayUsingPredicate:filterPredicate];
    if([tempArray count] > 0){
        NSMutableDictionary *tempEntry = [tempArray objectAtIndex:0];
        [tempEntry setValue:comment forKey:@"resourceIdentifyTime"];
        [[[NSApp delegate] filteredLog] fetch:self];
    }
}

- (oneway void)logParseTime:(in bycopy NSString *)comment forResourceID:(in bycopy int)resourceID
{
    NSPredicate *filterPredicate = [NSPredicate predicateWithFormat:@"(sessionID == %@) AND (resourceID == %@)", [NSString stringWithFormat:@"%d", _sessionID], [NSString stringWithFormat:@"%d", resourceID]];
    NSArray *tempArray = [[[[NSApp delegate] logController] arrangedObjects] filteredArrayUsingPredicate:filterPredicate];
    if([tempArray count] > 0){
        NSMutableDictionary *tempEntry = [tempArray objectAtIndex:0];
        [tempEntry setValue:comment forKey:@"resourceParseTime"];
        [[[NSApp delegate] filteredLog] fetch:self];
    }
}

- (oneway void)logLoadTime:(in bycopy NSString *)comment forResourceID:(in bycopy int)resourceID
{
    NSPredicate *filterPredicate = [NSPredicate predicateWithFormat:@"(sessionID == %@) AND (resourceID == %@)", [NSString stringWithFormat:@"%d", _sessionID], [NSString stringWithFormat:@"%d", resourceID]];
    NSArray *tempArray = [[[[NSApp delegate] logController] arrangedObjects] filteredArrayUsingPredicate:filterPredicate];
    if([tempArray count] > 0){
        NSMutableDictionary *tempEntry = [tempArray objectAtIndex:0];
        [tempEntry setValue:comment forKey:@"resourceLoadTime"];
        [[[NSApp delegate] filteredLog] fetch:self];
    }
}

- (oneway void)logThumbnailTime:(in bycopy NSString *)comment forResourceID:(in bycopy int)resourceID
{
    NSPredicate *filterPredicate = [NSPredicate predicateWithFormat:@"(sessionID == %@) AND (resourceID == %@)", [NSString stringWithFormat:@"%d", _sessionID], [NSString stringWithFormat:@"%d", resourceID]];
    NSArray *tempArray = [[[[NSApp delegate] logController] arrangedObjects] filteredArrayUsingPredicate:filterPredicate];
    if([tempArray count] > 0){
        NSMutableDictionary *tempEntry = [tempArray objectAtIndex:0];
        [tempEntry setValue:comment forKey:@"resourceThumbnailTime"];
        [[[NSApp delegate] filteredLog] fetch:self];
    }
}

@end
