//
//  PSMScrapedFile.h
//  NetScrape
//
//  Created by John Pannell on 9/20/05.
//  Copyright 2005 Positive Spin Media. All rights reserved.
//
/*
    This object holds all pertinent data for an object (image, video, pdf, etc.)
    retreived from the web by NetScrape.  The application creates these as it processes.
*/

#import <Cocoa/Cocoa.h>


@interface PSMScrapedFile : NSObject {
    NSURL               *sourceURL;         // the URL of the file when found
    NSString            *storagePath;       // the path to the file in the filesystem
    NSString            *parentURL;         // the URL of the web page this was found on
    unsigned long long  fileSize;           // the size of the file in bytes
    NSImage             *thumbnail;         // the thumbnail representing this image
    id                  controller;         // the engine object that created me
    BOOL                completed;          // have I completed building myself?
    NSMutableData       *myData;            // the actual data for the retreived file
    int                 pixelsHigh;
    int                 pixelsWide;
}

// designated initializer
-(id)initWithSourceURL:(NSURL *)url;

// accessors
-(NSURL *)sourceURL;
-(void)setSourceURL:(NSURL *)url;
-(NSString *)storagePath;
-(void)setStoragePath:(NSString *)path;
-(NSString *)parentURL;
-(void)setParentURL:(NSString *)url;
-(unsigned long long)fileSize;
-(void)setFileSize:(unsigned long long)numBytes;
-(NSImage *)thumbnail;
-(void)setThumbnail:(NSImage *)image;
-(id)controller;
-(void)setController:(id)anObject;
-(int)pixelsHigh;
-(void)setPixelsHigh:(int)value;
-(int)pixelsWide;
-(void)setPixelsWide:(int)value;

// loading
-(void)loadWebData;
-(void)loadFileData;
-(void)makeThumbnail:(NSMutableData *)thumbData;

// attaching
-(void)addImage:(NSString *)filePath;
-(void)corruptImage:(NSString *)filePath;

// NSURLConnection delegate methods
- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response;
- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data;
- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error;
- (void)connectionDidFinishLoading:(NSURLConnection *)connection;
-(NSCachedURLResponse *)connection:(NSURLConnection *)connection willCacheResponse:(NSCachedURLResponse *)cachedResponse;

// C Functions
CGImageRef PSMCreateThumbnailFromImageSource(CFURLRef url);

@end
