//
//  PSMScrapedFile.m
//  NetScrape
//
//  Created by John Pannell on 9/20/05.
//  Copyright 2005 Positive Spin Media. All rights reserved.
//

#import "PSMScrapedFile.h"


@implementation PSMScrapedFile

-(id)init;
{
    return [self initWithSourceURL:[[NSURL alloc] initWithString:@""]];
}

-(id)initWithSourceURL:(NSURL *)url
{
    if (![super init])
        return nil;
    
    // init my own variables
    completed = NO;
    [self setSourceURL:url];
    return self;
}

// accessors
-(NSURL *)sourceURL
{
    return sourceURL;
}

-(void)setSourceURL:(NSURL *)url
{
    [url retain];
    [sourceURL release];
    sourceURL = url;
}

-(NSString *)storagePath
{
    return storagePath;
}

-(void)setStoragePath:(NSString *)path
{
    [path retain];
    [storagePath release];
    storagePath = path;
}

-(NSString *)parentURL
{
    return parentURL;
}

-(void)setParentURL:(NSString *)url
{
    [url retain];
    [parentURL release];
    parentURL = url;
}

-(unsigned long long)fileSize
{
    return fileSize;
}

-(void)setFileSize:(unsigned long long)numBytes
{
    fileSize = numBytes;
}

-(NSImage *)thumbnail
{
    return thumbnail;
}

-(void)setThumbnail:(NSImage *)image
{
    [image retain];
    [thumbnail release];
    thumbnail = image;
}

-(id)controller
{
    return controller;
}

-(void)setController:(id)anObject
{
    [anObject retain];
    [controller release];
    controller = anObject;
}

-(int)pixelsHigh
{
    return pixelsHigh;
}

-(void)setPixelsHigh:(int)value
{
    pixelsHigh = value;
}

-(int)pixelsWide
{
    return pixelsWide;
}

-(void)setPixelsWide:(int)value
{
    pixelsWide = value;
}

// loading
-(void)loadWebData
{
    myData = [[NSMutableData alloc] init];
    NSURLConnection *theConnection;
    NSURLRequest *theRequest = [NSURLRequest requestWithURL:[self sourceURL] cachePolicy:NSURLRequestUseProtocolCachePolicy timeoutInterval:20.0];
    theConnection = [[NSURLConnection alloc] initWithRequest:theRequest delegate:self];
    // I will now hear back about things through the NSURLConnection delegate methods
}

-(void)loadFileData
{
    // retrieve data from disk given URL
    myData = [[NSMutableData alloc] initWithContentsOfURL:[self sourceURL]];
    [self setFileSize:[myData length]];
    if([myData length] > 0){
        // time to make my thumbnail - separate thread for performance
        SInt32 MacVersion;
        if (Gestalt(gestaltSystemVersion, &MacVersion) == noErr){
            if (MacVersion >= 0x1040){
                // this call is Tiger only
                [NSThread detachNewThreadSelector:@selector(makeThumbnailTiger:) toTarget:self withObject:[self storagePath]];
            } else {
                [NSThread detachNewThreadSelector:@selector(makeThumbnail:) toTarget:self withObject:myData];
            }
        } else {
            [NSThread detachNewThreadSelector:@selector(makeThumbnail:) toTarget:self withObject:myData];
        }
    } else {
        [controller addScrapedFile:nil];
        [self corruptImage:[self storagePath]];
    }
}

- (void)makeThumbnailTiger:(NSString *)pathToFile
{
    NSAutoreleasePool *thePool = [[NSAutoreleasePool alloc] init];
    
    NSBitmapImageRep *sourceBitmapImageRep = [[NSBitmapImageRep alloc] initWithData:myData];
    pixelsHigh = [sourceBitmapImageRep pixelsHigh];
    pixelsWide = [sourceBitmapImageRep pixelsWide];
    
    CFURLRef imageLocation = CFURLCreateWithFileSystemPath(kCFAllocatorDefault, (CFStringRef)pathToFile, kCFURLPOSIXPathStyle, false);
    CGImageRef thumbRef = PSMCreateThumbnailFromImageSource(imageLocation);
    CFRelease(imageLocation);
    
    if(thumbRef != NULL){
        //IconFamily* iconFamily;
        NSImage *slideImage;
        float scaleFactor;
        if(pixelsHigh > pixelsWide){
            slideImage = [NSImage imageNamed:@"vert_slide2"];
            scaleFactor = 114.0/pixelsHigh;
        } else {
            slideImage = [NSImage imageNamed:@"horiz_slide2"];
            scaleFactor = 114.0/pixelsWide;
        }
        [slideImage setCachedSeparately:YES];
        NSRect rect = NSMakeRect(67.0-scaleFactor*pixelsWide/2.0,67.0-scaleFactor*pixelsHigh/2.0,scaleFactor*pixelsWide,scaleFactor*pixelsHigh);
        NSImage *thumbImage = [[NSImage alloc] initWithSize:NSMakeSize(134.0, 134.0)];
        [thumbImage setCachedSeparately:YES];
        [thumbImage lockFocus];
        [[NSGraphicsContext currentContext] setImageInterpolation:NSImageInterpolationHigh];
        CGContextDrawImage([[NSGraphicsContext currentContext] graphicsPort], *(CGRect*)&rect, thumbRef);
        [slideImage compositeToPoint:NSMakePoint(3.0,3.0) fromRect:NSMakeRect(0.0, 0.0, 128.0, 128.0) operation:NSCompositeSourceOver];
        [thumbImage unlockFocus];
        
        thumbnail = [[NSImage alloc] initWithData:[thumbImage TIFFRepresentation]];
        
        //iconFamily = [IconFamily iconFamilyWithThumbnailsOfImage:thumbnail];
        //[iconFamily setAsCustomIconForFile:pathToFile];
        
        // write spotlight comment
        /*
        if([controller isKindOfClass:[SessionController class]]){
            NSAppleEventDescriptor *firstParameter = [NSAppleEventDescriptor descriptorWithString:pathToFile];
            NSAppleEventDescriptor *secondParameter = [NSAppleEventDescriptor descriptorWithString:[NSString stringWithFormat:@"%@, %@, %@", [controller myString], sourceURL, parentURL]];
            // create and populate the list of parameters
            // note that the array starts at index 1
            NSAppleEventDescriptor *parameters = [NSAppleEventDescriptor listDescriptor];
            [parameters insertDescriptor:firstParameter atIndex:1];
            [parameters insertDescriptor:secondParameter atIndex:2];
            // create the AppleEvent target
            ProcessSerialNumber psn = { 0, kCurrentProcess };
            NSAppleEventDescriptor *target = [NSAppleEventDescriptor descriptorWithDescriptorType:typeProcessSerialNumber bytes:&psn length:sizeof(ProcessSerialNumber)];
            // create an NSAppleEventDescriptor with the method name
            // note that the name must be lowercase (even if it is uppercase in AppleScript)
            NSAppleEventDescriptor *handler = [NSAppleEventDescriptor descriptorWithString:[@"setSpotlightComment" lowercaseString]];
            // last but not least, create the event for an AppleScript subroutine
            // set the method name and the list of parameters
            NSAppleEventDescriptor *event = [NSAppleEventDescriptor appleEventWithEventClass:kASAppleScriptSuite eventID:kASSubroutineEvent targetDescriptor:target returnID:kAutoGenerateReturnID transactionID:kAnyTransactionID];
            [event setParamDescriptor:handler forKeyword:keyASSubroutineName];
            [event setParamDescriptor:parameters forKeyword:keyDirectObject];
            // at last, call the event in AppleScript
            [[NSApp delegate] performSelectorOnMainThread:@selector(runCommentScriptWithEvent:) withObject:event waitUntilDone:NO];
        }
        */
        [thumbImage release];
        [self performSelectorOnMainThread:@selector(addImage:) withObject:storagePath waitUntilDone:NO];
    } else {
        // no thumb
        [self performSelectorOnMainThread:@selector(corruptImage:) withObject:storagePath waitUntilDone:YES];
    }
    
    // bye bye pool
    [thePool release];
    
    // kill thread (necessary?)
    [NSThread exit];    
}

-(void)makeThumbnail:(NSMutableData *)thumbData
{
    // thread safe
    NSAutoreleasePool *thePool = [[NSAutoreleasePool alloc] init];
    NSImage *theImage;
    NSBitmapImageRep *bitmapImageRep = [[NSBitmapImageRep alloc] 
initWithData:thumbData];
    pixelsHigh = [bitmapImageRep pixelsHigh];
    pixelsWide = [bitmapImageRep pixelsWide];
    theImage = [[NSImage alloc] initWithSize:[bitmapImageRep size]];
    [theImage addRepresentation:bitmapImageRep];
    [theImage setCachedSeparately:YES];

     
    // check to see if the image is a good one - not corrupt or partial download
    if([theImage isValid]){
        float scale, heightFactor, widthFactor;
        //IconFamily* iconFamily;
        NSImage *thumbImage;
        
        [theImage setScalesWhenResized:YES]; 
        [theImage setSize:NSMakeSize([bitmapImageRep pixelsWide], [bitmapImageRep pixelsHigh])];
        
        // create custom icon for the image if desired
        /*
        if([[NSUserDefaults standardUserDefaults] boolForKey:PSMcheckCustomIcon]){
            iconFamily = [IconFamily iconFamilyWithThumbnailsOfImage:theImage];
            [iconFamily setAsCustomIconForFile:storagePath];
        }
         */
        
        // take image and resize to 80 x 80 limit, then crop to 2x3 or 3x2 aspect ratio
        NSAffineTransform *at = [NSAffineTransform transform];
        heightFactor = 80.0/[theImage size].height;
        widthFactor = 80.0/[theImage size].width;
        if(heightFactor > widthFactor){
            // wide
            scale = widthFactor;
            [at scaleBy:scale];
            [theImage setSize:[at transformSize:[theImage size]]];
            thumbImage = [[NSImage imageNamed:@"horiz_slide"] copy];
            [thumbImage setCachedSeparately:YES];
            // crop rect
            int startY = (int)(([theImage size].height - 53)/2);
            // compose
            [thumbImage lockFocus];
            [[NSGraphicsContext currentContext] setImageInterpolation:NSImageInterpolationLow];
            [theImage compositeToPoint:NSMakePoint(8,21) fromRect:NSMakeRect(0,startY,80,53) operation:NSCompositeCopy];
            [thumbImage unlockFocus];
        } else {
            // tall
            scale = heightFactor;
            [at scaleBy:scale];
            [theImage setSize:[at transformSize:[theImage size]]];
            thumbImage = [[NSImage imageNamed:@"vert_slide"] copy];
            [thumbImage setCachedSeparately:YES];
            // crop rect
            int startX = (int)(([theImage size].width - 53)/2);
            // compose
            [thumbImage lockFocus];
            [[NSGraphicsContext currentContext] setImageInterpolation:NSImageInterpolationLow];
            [theImage compositeToPoint:NSMakePoint(22,8) fromRect:NSMakeRect(startX,0,53,80) operation:NSCompositeCopy];
            [thumbImage unlockFocus];
        }
        
        thumbnail = [[NSImage alloc] initWithData:[thumbImage TIFFRepresentation]];
        [thumbImage release];
        // send to main thread for inclusion in matrix
        [self performSelectorOnMainThread:@selector(addImage:) withObject:storagePath waitUntilDone:NO];
        
    } else {
        // corrupt image - notify myself to remove it from list
        [self performSelectorOnMainThread:@selector(corruptImage:) withObject:storagePath waitUntilDone:YES];
    }
    [bitmapImageRep release];
    [theImage release];
    
    // bye bye pool
    [thePool release];
    
    // kill thread (necessary?)
    [NSThread exit];
}

-(void)addImage:(NSString *)filePath
{
    // all steps are complete - tell controller to attach to thumbnail matrix
    completed = YES;
    [controller addScrapedFile:self];
}

-(void)corruptImage:(NSString *)filePath
{
    // the image to be stored at this location was corrupt - remove it
    [myData release];
    [self release];
}

// NSURLConnection delegate methods

- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response
{
    // this method is called when the server has determined that it
    // has enough information to create the NSURLResponse
    // it can be called multiple times, for example in the case of a 
    // redirect, so each time we reset the data.
    [myData setLength:0];
}

- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data
{
    // append the new data to the proper container
    [myData appendData:data];
}

- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error
{
    // release the connection, and the data object
    [myData release];
    myData = nil;
    [connection release];
    [self release]; // dead file object, no resource on web - get rid of it
}

- (void)connectionDidFinishLoading:(NSURLConnection *)connection
{
    // success - we were able to get all data from the web for this file
    // write data to disk
    if(![myData writeToFile:storagePath atomically:YES]){
        // couldn't write to disk
        [myData release];
        [self release];
        return;
    }
    [self setFileSize:[myData length]];
    
    // time to make my thumbnail - separate thread for performance
    SInt32 MacVersion;
    if (Gestalt(gestaltSystemVersion, &MacVersion) == noErr){
        if (MacVersion >= 0x1040){
            // this call is Tiger only
            [NSThread detachNewThreadSelector:@selector(makeThumbnailTiger:) toTarget:self withObject:[self storagePath]];
        } else {
            [NSThread detachNewThreadSelector:@selector(makeThumbnail:) toTarget:self withObject:myData];
        }
    } else {
        [NSThread detachNewThreadSelector:@selector(makeThumbnail:) toTarget:self withObject:myData];
    }
    
    // release the connection
    [connection release];
}

-(NSCachedURLResponse *)connection:(NSURLConnection *)connection willCacheResponse:(NSCachedURLResponse *)cachedResponse
{
    // don't cache anything I fetch
    return nil;
}

// housekeeping

-(NSString *)description
{
    NSString *result;
    result = [[NSString alloc] initWithFormat:@"file from %@ stored at %@", [sourceURL absoluteString], storagePath];
    return result;
}

-(void)dealloc
{
    [sourceURL release];
    [storagePath release];
    [parentURL release];
    [thumbnail release];
    // [myData release]; this has already been released prior to this point
    [super dealloc];
}

#pragma mark -
#pragma mark C Functions

CGImageRef PSMCreateThumbnailFromImageSource(CFURLRef url)
{
    // from page 230 of Programming With Quartz
    CGImageRef thumb = NULL;
    CGImageSourceRef imageSource = NULL;
    int maxThumbSize = 126;
    CFNumberRef maxThumbSizeRef;
    CFStringRef keys[3];
    CFTypeRef values[3];
    CFDictionaryRef options = NULL;
    
    imageSource = CGImageSourceCreateWithURL(url, NULL);
    if(imageSource == NULL)
        return NULL;
    
    maxThumbSizeRef = CFNumberCreate(NULL, kCFNumberIntType, &maxThumbSize);
    
    // options for thumb creation
    keys[0] = kCGImageSourceCreateThumbnailFromImageAlways;
    values[0] = (CFTypeRef)kCFBooleanTrue;
    
    keys[1] = kCGImageSourceThumbnailMaxPixelSize;
    values[1] = (CFTypeRef)maxThumbSizeRef;
    
    keys[2] = kCGImageSourceCreateThumbnailWithTransform;
    values[2] = (CFTypeRef)kCFBooleanTrue;
    
    options = CFDictionaryCreate(NULL, (const void **)keys, (const void **)values, 3, &kCFTypeDictionaryKeyCallBacks, &kCFTypeDictionaryValueCallBacks);
    CFRelease(maxThumbSizeRef);
    
    // make thumb
    thumb = CGImageSourceCreateThumbnailAtIndex(imageSource, 0, options);
    
    CFRelease(options);
    CFRelease(imageSource);
    
    if(thumb == NULL)
        return NULL;
    
    return thumb;
}

@end
