/*
 *  GLString.h

 *  Created November 3 2004.
 *  Copyright (c) 2004 Apple Computer Inc. All rights reserved.

 *
 */

#import <Cocoa/Cocoa.h>
#import <OpenGL/gl.h>
#import <OpenGL/glext.h>
#import <OpenGL/OpenGL.h>
#import <OpenGL/CGLContext.h>

@interface GLString : NSObject 
{
	CGLContextObj cgl_ctx; // current context at time of texture creation
	GLuint texID;			// Texture ID
	NSSize texSize;			// Texture size
	
	NSMutableAttributedString *str;
	NSBitmapImageRep	*bmpRep;
	
	// Color variables
	NSColor *textColor, *boxColor, *borderColor;
	NSFont *theFont;
	BOOL staticFrame; // default in NO
	NSSize marginSize, frameSize;
	
	NSMutableDictionary *attribs;
}

// Accessors
-(GLuint)texID;
-(NSSize)texSize;
-(NSColor*)textColor;
-(NSColor*)boxColor;
-(NSColor*)borderColor;
-(BOOL)staticFrame;
-(NSSize)frameSize;
-(NSSize)marginSize;

// 'Set' functions
-(void)setMargins:(NSSize)size;
-(void)useStaticFrame:(NSSize)size;
-(void)useDynamicFrame;
-(void)setTextColor:(NSColor*)color;
-(void)setBoxColor:(NSColor*)color;
-(void)setBorderColor:(NSColor*)color;

// Initialization and string adjustment
-(id)initWithString:(NSMutableString*)newString;
-(id)initWithAttributedString:(NSAttributedString*)newString;
-(id)initWithString:(NSMutableString*)newString textColor:(NSColor*)txColor boxColor:(NSColor*)bxColor borderColor:(NSColor*)brColor;
-(void)setString:(NSMutableString*)newString;
-(void)initTextAttributes;

// Texture generation and rendering
-(void)genTexture;
-(void)drawWithBounds:(NSRect)bounds;
-(void)drawAtPoint:(NSPoint)point;

// Clean up
-(void)dealloc;
-(void)deleteTexture;

@end
