//
//  PSMActionImageCell.m
//  NetScrape
//
//  Created by John Pannell on Tue Oct 01 2002.
//

#import "PSMActionImageCell.h"
#import "PSMBezierPathCategories.h"
#import "ThumbnailMatrix.h"


@implementation PSMActionImageCell
- (id)target
{
    return target;
}

- (void)setTarget:(id)anObject
{
    [anObject retain];
    [target release];
    target = anObject;
}

- (SEL)action
{
    return action;
}

- (void)setAction:(SEL)aSelector
{
    action = aSelector;
}

- (void)drawWithFrame:(NSRect)cellFrame inView:(NSView *)controlView
{
    if([(ThumbnailMatrix *)controlView drawForDrop]){
        // someone has dragged something over the view
        // if this cell is in the mouse's row to the right of the mouse,
        // scoot it over to the right
        NSPoint mouseLoc = [controlView convertPoint:[(ThumbnailMatrix *)controlView dropMouseLocation] fromView:[[controlView window] contentView]];
        if((cellFrame.origin.y < mouseLoc.y) && ((cellFrame.origin.y + cellFrame.size.height) > mouseLoc.y)){
            // in row
            if(cellFrame.origin.x > (mouseLoc.x - cellFrame.size.width)){
                cellFrame.origin.x = cellFrame.origin.x + (cellFrame.size.width/2);
            }
        }
    }
    [self drawInteriorWithFrame:cellFrame inView:controlView];
    
    if ([self isHighlighted]) {
        NSRect outerFrame = NSMakeRect(cellFrame.origin.x+2.5, cellFrame.origin.y+2.5, cellFrame.size.width-5.0, cellFrame.size.height-5.0);
        NSBezierPath *path = [NSBezierPath bezierPathWithRoundedRect:outerFrame cornerRadius:3.0];
        float h, s, b, a;
        if([[controlView window] firstResponder] == controlView){
            // get highlighted cell color
            NSColor* controlColor = [[NSColor alternateSelectedControlColor] colorUsingColorSpaceName:NSCalibratedRGBColorSpace];
            // pump it up for surrounding edge
            [controlColor getHue:&h saturation:&s brightness:&b alpha:&a];
            controlColor = [NSColor colorWithCalibratedHue:h saturation:1.0 brightness:0.5 alpha:1.0];
            [controlColor set];
            // draw
            [path setLineWidth:3.0];
            [path stroke];
            controlColor = [[NSColor selectedControlColor] colorUsingColorSpaceName:NSCalibratedRGBColorSpace];
            [controlColor getHue:&h saturation:&s brightness:&b alpha:&a];
            controlColor = [NSColor colorWithCalibratedHue:h saturation:s brightness:b alpha:0.2];
            [controlColor set];
            [path fill];
        } else {
            // draw things greyed out
            NSColor* controlColor = [[NSColor grayColor] colorUsingColorSpaceName:NSCalibratedRGBColorSpace];
            // pump it up for surrounding edge
            [controlColor getHue:&h saturation:&s brightness:&b alpha:&a];
            controlColor = [NSColor colorWithCalibratedHue:h saturation:s brightness:0.5 alpha:1.0];
            [controlColor set];
            // draw
            [path setLineWidth:3.0];
            [path stroke];
            controlColor = [[NSColor lightGrayColor] colorUsingColorSpaceName:NSCalibratedRGBColorSpace];
            [controlColor getHue:&h saturation:&s brightness:&b alpha:&a];
            controlColor = [NSColor colorWithCalibratedHue:h saturation:s brightness:b alpha:0.2];
            [controlColor set];
            [path fill];
        }
    }
}

- (void)dealloc
{
    [self setAction:nil];
    [self setTarget:nil];
    [super dealloc];
}

@end
