//
//  PSMGLImage.h
//  NetScrape
//
//  Created by John Pannell on Sat Jan 01 2005.
//  Copyright (c) 2005 Positive Spin Media. All rights reserved.
//
//  Heavily leveraged from Apple's GLImage example code

#import <Cocoa/Cocoa.h>
#import <OpenGL/OpenGL.h>
#include <OpenGL/gl.h>
#include <OpenGL/glu.h>
#include <OpenGL/glext.h>

@class GLString;

typedef struct
{
    float x, y, z;
} Vector3;

@interface PSMGLImage : NSObject
{
    NSBitmapImageRep        *bmpRep;                // the bitmap image itself
    Vector3                 *glQuad;                // the gl rectangle the image is drawn on
    Vector3                 *backgroundQuad;        // the gl rectangle that the background mask is on
    int                     viewWidth, viewHeight;  // pixels for view size
    GLfloat                 glViewWidth, glViewHeight;  // gl coords for view (max 100)
    int                     imgWidth, imgHeight;    // pixels for the image size
    float                   viewRatio, imgRatio;    // aspect ratio for view and image
    GLuint                  txID;                   // GL texture ID
    GLenum                  pixfmt, bpp, datatype;  // other needed GL parameters for texturing
    GLString                *imageInfo;             // image information for output
    BOOL                    ready;                  // successfully inited?
    float                   zOffset;                // how far out from origin the quad is
    GLfloat                 myAlpha;                // opacity of the image (0.0 - 1.0)
    NSPoint                 ul, ur, ll, lr;         // positioning of the texture on quad
}
// Accessors
-(NSBitmapImageRep*)bmpImage;
-(GLString*)imageInfo;
-(float)zOffset;
-(int)txID;

// Initialization
-(void)initGL;
-(void)setFrame:(NSRect)frame;

// Data loading
-(void)loadTextureFromFile:(NSString*)filePath;
-(void)setTxID:(int)newTxID;

// Information generation
-(void)genQuad;
-(void)getTextureInfo;
-(void)genImageInfo:(NSString*)imgName;

// Rendering function
-(void)drawTexture;

// Cleanup functions
-(void)discardData;

// transition
-(void)setAlpha:(float)newAlpha;

// cube/flip placement methods
-(void)placeFront;
-(void)placeLeft;
-(void)placeRight;
-(void)placeBottom;
-(void)placeTop;
-(void)placeFlip;

@end
