//
//  PSMGLSlideShowView.h
//  NetScrape
//
//  Created by John Pannell on 12/10/04.
//  Copyright 2004 Positive Spin Media. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import "PSMGLImage.h"


typedef struct {
    GLdouble x,y,z;
} recVec;

typedef struct {
    recVec viewPos;                 // View position
    recVec viewDir;                 // View direction vector
    recVec viewUp;                  // View up direction
    recVec rotPoint;                // Point to rotate about
    GLdouble aperture;              // pContextInfo->camera aperture
    GLint viewWidth, viewHeight;    // current view height and width
} recCamera;

@interface PSMGLSlideShowView : NSOpenGLView {
    BOOL            fGLinit;
    NSTimer         *initiatorTimer;
    NSTimer         *effectTimer;
    PSMGLImage      *foregroundImage;
    PSMGLImage      *backgroundImage;
    NSMutableArray  *imageList;         // contains paths to various image files
    int             imageIndex;         // tracks the index of the FOREGROUND image
    BOOL            steveMo;            // slow motion (thanks for the idea, Steve!)
    
    // display strings
    GLString        *camStringTex;
    
    // camera handling
    recCamera       camera;
    
    // cube transition
    float           rotRadius;          // distance from origin to camera
    int             transitionSteps;    // number of steps in transition
    int             secondaryTransition;// pref value for transition direction
    
    // transition progress
    int             stepsCompleted;
}

// camera/view related
- (void)updateProjection;
- (void)updateCameraString;
- (void)updateModelView;
- (void)resetCamera;
- (void)resizeGL;
- (void)drawInfo;

// prefs update
-(void)triggerChange;

// transition related
-(void)startDissolve:(NSTimer *)theTimer;
-(void)performDissolve:(NSTimer *)theTimer;
-(void)startCube:(NSTimer *)theTimer;
-(void)performCube:(NSTimer *)theTimer;
-(void)placeBackgroundImage;

// the pictures!
-(void)setImageList:(NSMutableArray *)newImageList;

// management
-(void)killTimers;

@end
