//
//  PSMGLSlideShowView.m
//  NetScrape
//
//  Created by John Pannell on 12/10/04.
//  Copyright 2004 Positive Spin Media. All rights reserved.
//
#include <OpenGL/gl.h>
#include <GLUT/glut.h>

#import "PSMGLSlideShowView.h"
#define GL_PI 3.1415f

recVec gOrigin = {0.0, 0.0, 0.0};

// time and message info
CFAbsoluteTime gMsgPresistance = 10.0f;
static CFAbsoluteTime gStartTime = 0.0f;

// error output
GLString * gErrStringTex;
float gErrorTime;

// set app start time
static void setStartTime (void)
{
    gStartTime = CFAbsoluteTimeGetCurrent (); 
}

// return float elpased time in seconds since app start
static CFAbsoluteTime getElapsedTime (void)
{
    return CFAbsoluteTimeGetCurrent () - gStartTime;
}

// error reporting as both window message and debugger string
void reportError (char * strError)
{
    NSMutableDictionary *attribs = [NSMutableDictionary dictionary];
    [attribs setObject: [NSFont fontWithName: @"Monaco" size: 9.0f] forKey: NSFontAttributeName];
    [attribs setObject: [NSColor whiteColor] forKey: NSForegroundColorAttributeName];
    
    gErrorTime = getElapsedTime ();
    NSMutableString * errString = [NSMutableString stringWithFormat:@"Error: %s (at time: %0.1f secs).", strError, gErrorTime];
    NSLog (@"%@\n", errString);
    
    /*
    if (gErrStringTex)
        [gErrStringTex setString:errString];// withAttributes:attribs];
    else
        gErrStringTex = [[(GLString*)[GLString alloc] initWithString:errString] retain];
     */
}

GLenum glReportError (void)
{
    GLenum err = glGetError();
    if (GL_NO_ERROR != err)
        reportError ((char *) gluErrorString (err));
    return err;
}

@implementation PSMGLSlideShowView

- (id)initWithFrame:(NSRect)frame
{
    GLuint attribs[] = 
    {
        NSOpenGLPFANoRecovery,
        NSOpenGLPFAWindow,
        NSOpenGLPFAAccelerated,
        NSOpenGLPFADoubleBuffer,
        NSOpenGLPFAColorSize, 24,
        NSOpenGLPFAAlphaSize, 8,
        NSOpenGLPFADepthSize, 24,
        NSOpenGLPFAStencilSize, 8,
        NSOpenGLPFAAccumSize, 0,
        0
    };
    
    NSOpenGLPixelFormat* fmt = [[NSOpenGLPixelFormat alloc] initWithAttributes: (NSOpenGLPixelFormatAttribute*) attribs]; 
    
    if(!fmt){
        NSLog(@"No OpenGL pixel format");
    }
    
    // ready but not inited
    fGLinit = NO;
    imageList = [[NSMutableArray alloc] init];
    return self = [super initWithFrame:frame pixelFormat:[fmt autorelease]];
}

- (void) awakeFromNib
{
    setStartTime (); 
}

- (void) drawRect: (NSRect) rect
{
    if(!fGLinit){
        [self prepareOpenGL];
    }
    
    // setup viewport and prespective
    [self resizeGL]; // forces projection matrix update (does test for size changes)
    [self updateModelView];  // update model view matrix for object
        
    // clear the screen
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    
    // drawing code follows...
    [backgroundImage drawTexture];
    [foregroundImage drawTexture];
    
    [[self openGLContext] flushBuffer];
}

// handles resizing of GL need context update and if the window dimensions change, a
// a window dimension update, reseting of viewport and an update of the projection matrix
- (void)resizeGL
{
    NSRect rectView = [self bounds];
    
    // ensure camera knows size changed
    if ((camera.viewHeight != rectView.size.height) || (camera.viewWidth != rectView.size.width)) {
        camera.viewHeight = rectView.size.height;
        camera.viewWidth = rectView.size.width;
        
        glViewport (0, 0, camera.viewWidth, camera.viewHeight);
        
        // tell images that the frame has changed
        if(foregroundImage != nil){
            [foregroundImage setFrame:rectView];
        }
        if(backgroundImage != nil){
            [backgroundImage setFrame:rectView];
        }
        
        // reposition images
        [foregroundImage genQuad];
        [backgroundImage genQuad];
        [self placeBackgroundImage];
        
        // recalculate rotation radius
        rotRadius = [foregroundImage zOffset] - 150.0;
        
        //[self resetCamera];
        [self updateProjection];  // update projection matrix
    }
}

- (void)updateProjection
{
    GLdouble ratio, radians, wd2;
    GLdouble left, right, top, bottom, near, far;
    
    [[self openGLContext] makeCurrentContext];
    
    // set projection
    glMatrixMode (GL_PROJECTION);
    glLoadIdentity ();
    near = 50.0;
    if (near < 0.00001)
        near = 0.00001;
    far = -camera.viewPos.z * 1.5;
    radians = GL_PI / 180 * camera.aperture / 2; // half aperture degrees to radians 
    wd2 = near * tan(radians);
    ratio = camera.viewWidth / (float) camera.viewHeight;
    if (ratio >= 1.0) {
        left  = -wd2;
        right = wd2;
        top = wd2 / ratio;
        bottom = -wd2 / ratio;	
    } else {
        left  = -wd2 * ratio;
        right = wd2 * ratio;
        top = wd2;
        bottom = -wd2;	
    }
    glFrustum (left, right, bottom, top, near, far);
    [self updateCameraString];
}

// updates the contexts model view matrix for object and camera moves
- (void)updateModelView
{
    [[self openGLContext] makeCurrentContext];
    // move view
    glMatrixMode (GL_MODELVIEW);
    glLoadIdentity ();
    gluLookAt (camera.viewPos.x, camera.viewPos.y, camera.viewPos.z,
               camera.viewPos.x + camera.viewDir.x,
               camera.viewPos.y + camera.viewDir.y,
               camera.viewPos.z + camera.viewDir.z,
               camera.viewUp.x, camera.viewUp.y ,camera.viewUp.z);
    
    //[self updateCameraString];
}

- (void)updateCameraString
{
    /*
    NSMutableString *string = [NSMutableString stringWithFormat:@"Camera (%0.1f, %0.1f, %0.1f) \nlooking at (%0.1f, %0.1f, %0.1f) \nwith %0.1f aperture", camera.viewPos.x, camera.viewPos.y, camera.viewPos.z, camera.viewDir.x, camera.viewDir.y, camera.viewDir.z, camera.aperture];
    if (camStringTex)
        [camStringTex setString:string];// withAttributes:stanStringAttrib];
    else
        camStringTex = [[(GLString*)[GLString alloc] initWithString:string] retain];
     */
}

- (void)resetCamera
{
    // set aperture to fit image properly
    // major axis of image is 100 in glCoords, divide that by two, and 
    camera.aperture = 2.0*180.0*atan(50.0/150.0)/GL_PI;
    camera.rotPoint = gOrigin;
    
    camera.viewPos.x = 0.0;
    camera.viewPos.y = 0.0;
    camera.viewPos.z = rotRadius;
    camera.viewDir.x = -camera.viewPos.x; 
    camera.viewDir.y = -camera.viewPos.y; 
    camera.viewDir.z = -camera.viewPos.z;
    
    camera.viewUp.x = 0;  
    camera.viewUp.y = 1; 
    camera.viewUp.z = 0;
}

- (void)drawInfo
{	
    GLint matrixMode;
    GLboolean depthTest = glIsEnabled (GL_DEPTH_TEST);
    GLfloat height, width; //, messageTop = 10.0f
    
    height = camera.viewHeight;
    width = camera.viewWidth;
    
    glDisable (GL_DEPTH_TEST); // ensure text is not remove by depth buffer test.
    glEnable (GL_BLEND); // for text fading
    glBlendFunc (GL_ONE, GL_ONE_MINUS_SRC_ALPHA); // ditto
    glEnable (GL_TEXTURE_RECTANGLE_EXT);
    
    // set orthograhic 1:1  pixel transform in local view coords
    glGetIntegerv (GL_MATRIX_MODE, &matrixMode);
    glMatrixMode (GL_PROJECTION);
    glPushMatrix();
    glLoadIdentity ();
    glMatrixMode (GL_MODELVIEW);
    glPushMatrix();
    glLoadIdentity ();
    glScalef (2.0f / width, -2.0f /  height, 1.0f);
    glTranslatef (-width / 2.0f, -height / 2.0f, 0.0f);
    
    glColor4f (1.0f, 1.0f, 1.0f, 1.0f);
    //[infoStringTex drawAtPoint:NSMakePoint (10.0f, height - [infoStringTex frameSize].height - 10.0f)];
    //[camStringTex drawAtPoint:NSMakePoint (10.0f, messageTop)];
    
    
    // reset orginal martices
    glPopMatrix(); // GL_MODELVIEW
    glMatrixMode (GL_PROJECTION);
    glPopMatrix();
    glMatrixMode (matrixMode);
    
    glDisable (GL_TEXTURE_RECTANGLE_EXT);
    glDisable (GL_BLEND);
    if (depthTest)
        glEnable (GL_DEPTH_TEST);
    glReportError ();
}

-(void)prepareOpenGL
{
    // inits the openGL context, called once everything is ready
    // similar to "SetupRC" in the text
    // init GL stuff here
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glEnable(GL_DEPTH_TEST);
    glDepthFunc(GL_LEQUAL);
    glClearDepth(1.0);
    glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
    glEnable(GL_LINE_SMOOTH);
    glShadeModel(GL_SMOOTH);
    glFrontFace(GL_CCW);
    glPolygonOffset (1.0f, 1.0f);
    
    // testing for gl_texture_rectangle_ext support
    if(![[NSString stringWithCString:(char *)glGetString(GL_EXTENSIONS)] rangeOfString:@"texture_rectangle"].length){
        // not supported - toss up alert if I'm small
        if([self frame].size.width < 250){
            NSBeginAlertSheet(@"This might not work...", nil,
                              nil, nil, [self window],
                              nil, nil, nil, nil, @"The video hardware in this computer does not appear to support the fancy programming used to create these wonderful 3D slide shows. It might still work, but it might not... If you'd like to see support in future versions of Pandora, let me know: pandora@positivespinmedia.com");
        }
    }
    
    // set clear color
    glClearColor(0.0f,0.0f,0.0f,1.0f);
    
    // set drawing color
    glColor3f(0.0f,1.0f,0.0f);
    
    imageIndex = 0;
    
    // set swap to vbl sync (no tearing)
    long swapInt = 1;
    [[self openGLContext] setValues:&swapInt forParameter:NSOpenGLCPSwapInterval];
    
    float totalInterval = [[NSUserDefaults standardUserDefaults] integerForKey:@"Slideshow slide duration"] + [[NSUserDefaults standardUserDefaults] floatForKey:@"Slideshow transition speed"]/30.0;
    
    // set rotation variables based on preferences
    switch([[NSUserDefaults standardUserDefaults] integerForKey:@"Slideshow primary transition"]){
        case 0:
            // dissolve transition
            initiatorTimer = [NSTimer scheduledTimerWithTimeInterval:totalInterval target:self selector:@selector(startDissolve:) userInfo:nil repeats:YES];
            break;
            
        case 1:
            // cube transition
            initiatorTimer = [NSTimer scheduledTimerWithTimeInterval:totalInterval target:self selector:@selector(startCube:) userInfo:nil repeats:YES];
            break;
            
        case 2:
            // flip transition
            initiatorTimer = [NSTimer scheduledTimerWithTimeInterval:totalInterval target:self selector:@selector(startFlip:) userInfo:nil repeats:YES];
            break;
    }
    // add timers
    [[NSRunLoop currentRunLoop] addTimer:initiatorTimer forMode:NSEventTrackingRunLoopMode];
    [[NSRunLoop currentRunLoop] addTimer:initiatorTimer forMode:NSDefaultRunLoopMode];
    
    // make sure there are some photos in my list
    if([imageList count] == 0){
        // get images from bundle
        imageList = [[NSArray alloc] initWithObjects:[[[NSBundle mainBundle] bundlePath] stringByAppendingString:@"/Contents/Resources/example1.jpg"], [[[NSBundle mainBundle] bundlePath] stringByAppendingString:@"/Contents/Resources/example2.jpg"], nil];
    }
    
    if(!foregroundImage){
        foregroundImage = [[PSMGLImage alloc] initWithFrame:[self frame]];
        [foregroundImage loadTextureFromFile:[imageList objectAtIndex:imageIndex]];
        [foregroundImage setAlpha:1.0];
        imageIndex++;
    }
    if(!backgroundImage){
        backgroundImage = [[PSMGLImage alloc] initWithFrame:[self frame]];
        [backgroundImage loadTextureFromFile:[imageList objectAtIndex:imageIndex]];
        [backgroundImage setAlpha:1.0];
        imageIndex++;
    }
        
    
    //place background image if cube
    [self placeBackgroundImage];
    rotRadius = [foregroundImage zOffset] - 150.0;
    
    // set camera
    [self resetCamera];
    
    // note completed init
    fGLinit = YES;
}

- (void)mouseDown:(NSEvent *)theEvent // trigger the transition
{
    if ([theEvent modifierFlags] & NSShiftKeyMask){
        steveMo = TRUE;
    } else {
        steveMo = FALSE;
    }
    [self triggerChange];
}

-(void)startDissolve:(NSTimer *)theTimer
{
    // initiate the dissolve transition
    stepsCompleted = 0;
    if (steveMo == FALSE){
        transitionSteps = [[NSUserDefaults standardUserDefaults] floatForKey:@"Slideshow transition speed"];
    } else {
        transitionSteps = 3 * [[NSUserDefaults standardUserDefaults] floatForKey:@"Slideshow transition speed"];
    }
    steveMo = FALSE; // only doing this with mouse clicks on view
    effectTimer = [NSTimer scheduledTimerWithTimeInterval:(1.0/30.0) target:self selector:@selector(performDissolve:) userInfo:nil repeats:YES];
    [[NSRunLoop currentRunLoop] addTimer:effectTimer forMode:NSEventTrackingRunLoopMode];
    [[NSRunLoop currentRunLoop] addTimer:effectTimer forMode:NSDefaultRunLoopMode];
}

-(void)performDissolve:(NSTimer *)theTimer
{
    // work through execution of the dissolve transition
     float newAlpha = 0.5 * cos(GL_PI * stepsCompleted / transitionSteps) + 0.5;
    [foregroundImage setAlpha:newAlpha];
    [backgroundImage setAlpha:(1.0-newAlpha)];
    if(stepsCompleted >= transitionSteps){
        // transition is over
        [theTimer invalidate];
        effectTimer = nil;

        // move background image up to foreground
        [foregroundImage release];
        foregroundImage = backgroundImage;
        [foregroundImage genQuad];
        backgroundImage = [[PSMGLImage alloc] initWithFrame:[self frame]];
        [foregroundImage setAlpha:1.0];
        [backgroundImage setAlpha:0.0];
        
        // determine how to continue
        if((imageIndex) >= [imageList count]){
            // all slides have been shown
            if([[NSUserDefaults standardUserDefaults] boolForKey:@"Slideshow repeat show"]){
                imageIndex = 0;
                [backgroundImage loadTextureFromFile:[imageList objectAtIndex:imageIndex]];
                [backgroundImage placeFront];
                imageIndex++;
            } else {
                [initiatorTimer invalidate];
                initiatorTimer = nil;
                // note this for later - if bigger than count, show is over
                imageIndex++;
            }
        } else {
            // more images to go
            [backgroundImage loadTextureFromFile:[imageList objectAtIndex:imageIndex]];
            [backgroundImage placeFront];
            imageIndex++;
        }
    }
    [self setNeedsDisplay:YES];
    stepsCompleted++;
}

// prefs update
-(void)triggerChange
{
    NSTimer *immediateTimer;

    if((imageIndex) > [imageList count]){
        // show was over - I need to reload everything
        imageIndex = 0;
        [foregroundImage release];
        foregroundImage = nil;
        [backgroundImage release];
        backgroundImage = nil;
        
        foregroundImage = [[PSMGLImage alloc] initWithFrame:[self frame]];
        [foregroundImage loadTextureFromFile:[imageList objectAtIndex:imageIndex]];
        [foregroundImage setAlpha:1.0];
        imageIndex++;
    
        backgroundImage = [[PSMGLImage alloc] initWithFrame:[self frame]];
        [backgroundImage loadTextureFromFile:[imageList objectAtIndex:imageIndex]];
        [backgroundImage setAlpha:1.0];
        imageIndex++;
    }
    
    // tell images to regenerate their quads, as the prefs have just changed
    [foregroundImage genQuad];
    [backgroundImage genQuad];
    [self placeBackgroundImage];
    
    // recalculate rotation radius
    rotRadius = [foregroundImage zOffset] - 150.0;
    
    // put the camera back
    [self resetCamera];
    
    // call appropriate transition
    if([initiatorTimer isValid]){
        [initiatorTimer invalidate];
        initiatorTimer = nil;
    }
    if([effectTimer isValid]){
        [effectTimer invalidate];
        effectTimer = nil;
    }
    float totalInterval = [[NSUserDefaults standardUserDefaults] integerForKey:@"Slideshow slide duration"] + [[NSUserDefaults standardUserDefaults] floatForKey:@"Slideshow transition speed"]/30.0;
    switch([[NSUserDefaults standardUserDefaults] integerForKey:@"Slideshow primary transition"]){
        case 0:
            // dissolve transition
            immediateTimer = [NSTimer scheduledTimerWithTimeInterval:0.0 target:self selector:@selector(startDissolve:) userInfo:nil repeats:NO];
            initiatorTimer = [NSTimer scheduledTimerWithTimeInterval:totalInterval target:self selector:@selector(startDissolve:) userInfo:nil repeats:YES];
            [[NSRunLoop currentRunLoop] addTimer:initiatorTimer forMode:NSEventTrackingRunLoopMode];
            [[NSRunLoop currentRunLoop] addTimer:initiatorTimer forMode:NSDefaultRunLoopMode];
            break;
            
        case 1:
            // cube transition
            immediateTimer = [NSTimer scheduledTimerWithTimeInterval:0.0 target:self selector:@selector(startCube:) userInfo:nil repeats:NO];
            initiatorTimer = [NSTimer scheduledTimerWithTimeInterval:totalInterval target:self selector:@selector(startCube:) userInfo:nil repeats:YES];
            [[NSRunLoop currentRunLoop] addTimer:initiatorTimer forMode:NSEventTrackingRunLoopMode];
            [[NSRunLoop currentRunLoop] addTimer:initiatorTimer forMode:NSDefaultRunLoopMode];
            break;
            
        case 2:
            // flip transition
            immediateTimer = [NSTimer scheduledTimerWithTimeInterval:0.0 target:self selector:@selector(startFlip:) userInfo:nil repeats:NO];
            initiatorTimer = [NSTimer scheduledTimerWithTimeInterval:totalInterval target:self selector:@selector(startFlip:) userInfo:nil repeats:YES];
            [[NSRunLoop currentRunLoop] addTimer:initiatorTimer forMode:NSEventTrackingRunLoopMode];
            [[NSRunLoop currentRunLoop] addTimer:initiatorTimer forMode:NSDefaultRunLoopMode];
            break;
    }
    
    // redraw
    [self setNeedsDisplay:YES];
}

-(void)startCube:(NSTimer *)theTimer
{
    // initiate the cube transition
    stepsCompleted = 0;
    if (steveMo == FALSE){
        transitionSteps = [[NSUserDefaults standardUserDefaults] floatForKey:@"Slideshow transition speed"];
    } else {
        transitionSteps = 3 * [[NSUserDefaults standardUserDefaults] floatForKey:@"Slideshow transition speed"];
    }
    steveMo = FALSE; // only doing this with mouse clicks on view
    effectTimer = [NSTimer scheduledTimerWithTimeInterval:(1.0/30.0) target:self selector:@selector(performCube:) userInfo:nil repeats:YES];
    [[NSRunLoop currentRunLoop] addTimer:effectTimer forMode:NSEventTrackingRunLoopMode];
    [[NSRunLoop currentRunLoop] addTimer:effectTimer forMode:NSDefaultRunLoopMode];
    // place the background image
    [foregroundImage setAlpha:1.0];
    [backgroundImage setAlpha:1.0];
    // cache this to improve transition performance
    secondaryTransition = [[NSUserDefaults standardUserDefaults] integerForKey:@"Slideshow secondary transition"];
}

-(void)performCube:(NSTimer *)theTimer
{
    // work through execution of the cube transition
    // determine angle to move camera (between 0 and /2 in radians)
    float newAngle = (GL_PI/2.0) * (0.5 * cos(GL_PI * stepsCompleted / transitionSteps) + 0.5);
    // determine new camera position
    // bump out camera for effect while rotating
    float moveRadius = rotRadius - 30*sin(2*newAngle);
    camera.viewPos.z = sin(newAngle)*moveRadius;
    switch(secondaryTransition){
        case 0: // top to bottom
            camera.viewPos.y = -cos(newAngle)*moveRadius;
            break;
        case 1: // bottom to top
            camera.viewPos.y = cos(newAngle)*moveRadius;
            break;
        case 2: // left to right
            camera.viewPos.x = -cos(newAngle)*moveRadius;
            break;
        case 3: // right to left
            camera.viewPos.x = cos(newAngle)*moveRadius;
            break;
    }
    camera.viewDir.x = -camera.viewPos.x; 
    camera.viewDir.y = -camera.viewPos.y; 
    camera.viewDir.z = -camera.viewPos.z;
    
    if(stepsCompleted >= transitionSteps){
        // transition is over
        [theTimer invalidate];
        effectTimer = nil;
        
        // move background image up to foreground
        [foregroundImage release];
        foregroundImage = backgroundImage;
        [foregroundImage genQuad];
        backgroundImage = [[PSMGLImage alloc] initWithFrame:[self frame]];
        
        // put the camera back
        [self resetCamera];

        // determine how to continue
        if((imageIndex) >= [imageList count]){
            // all slides have been shown
            if([[NSUserDefaults standardUserDefaults] boolForKey:@"Slideshow repeat show"]){
                imageIndex = 0;
                [backgroundImage loadTextureFromFile:[imageList objectAtIndex:imageIndex]];
                imageIndex++;
                [self placeBackgroundImage];
            } else {
                [initiatorTimer invalidate];
                initiatorTimer = nil;
                // note this for later - if bigger than count, show is over
                imageIndex++;
            }
        } else {
            // more images to go
            [backgroundImage loadTextureFromFile:[imageList objectAtIndex:imageIndex]];
            imageIndex++;
            [self placeBackgroundImage];
        }
        
    }
    [self setNeedsDisplay:YES];
    stepsCompleted++;
}

-(void)placeBackgroundImage
{
    // position background image for cube rotation
    if([[NSUserDefaults standardUserDefaults] integerForKey:@"Slideshow primary transition"] == 1){
        switch([[NSUserDefaults standardUserDefaults] integerForKey:@"Slideshow secondary transition"]){
            case 0: // top to bottom
                [backgroundImage placeTop];
                break;
            case 1: // bottom to top
                [backgroundImage placeBottom];
                break;
            case 2: // left to right
                [backgroundImage placeLeft];
                break;
            case 3: // right to left
                [backgroundImage placeRight];
                break;
        }
    }
    
    // position for flip
    if([[NSUserDefaults standardUserDefaults] integerForKey:@"Slideshow primary transition"] == 2){
        [backgroundImage placeFlip];
    }
}

-(void)startFlip:(NSTimer *)theTimer
{
    // initiate the cube transition
    stepsCompleted = 0;
    if (steveMo == FALSE){
        transitionSteps = [[NSUserDefaults standardUserDefaults] floatForKey:@"Slideshow transition speed"];
    } else {
        transitionSteps = 3 * [[NSUserDefaults standardUserDefaults] floatForKey:@"Slideshow transition speed"];
    }
    steveMo = FALSE; // only doing this with mouse clicks on view
    effectTimer = [NSTimer scheduledTimerWithTimeInterval:(1.0/30.0) target:self selector:@selector(performFlip:) userInfo:nil repeats:YES];
    [[NSRunLoop currentRunLoop] addTimer:effectTimer forMode:NSEventTrackingRunLoopMode];
    [[NSRunLoop currentRunLoop] addTimer:effectTimer forMode:NSDefaultRunLoopMode];
    // place the background image
    [foregroundImage setAlpha:1.0];
    [backgroundImage setAlpha:1.0];
    // cache this to improve transition performance
    secondaryTransition = [[NSUserDefaults standardUserDefaults] integerForKey:@"Slideshow secondary transition"];
}

-(void)performFlip:(NSTimer *)theTimer
{
    // work through execution of the flip transition
    // determine angle to move camera (between 0 and  in radians)
    float newAngle = GL_PI * (0.5 * cos(GL_PI * stepsCompleted / transitionSteps + GL_PI) + 0.5);
    // determine new camera position
    // bump out camera for effect while rotating
    float moveRadius = rotRadius - 30*sin(newAngle);
    camera.viewPos.z = cos(newAngle)*moveRadius;
    switch(secondaryTransition){
        case 0: // top to bottom
            camera.viewPos.y = -sin(newAngle)*moveRadius;
            if(camera.viewPos.z > 0){
                camera.viewUp.y = -1;
            }
            break;
        case 1: // bottom to top
            camera.viewPos.y = sin(newAngle)*moveRadius;
            if(camera.viewPos.z > 0){
                camera.viewUp.y = -1;
            }
            break;
        case 2: // left to right
            camera.viewPos.x = -sin(newAngle)*moveRadius;
            break;
        case 3: // right to left
            camera.viewPos.x = sin(newAngle)*moveRadius;
            break;
    }
    camera.viewDir.x = -camera.viewPos.x; 
    camera.viewDir.y = -camera.viewPos.y; 
    camera.viewDir.z = -camera.viewPos.z;
    
    // if the camera gets past z = 0, I need to let it orient upside down
    
    
    if(stepsCompleted >= transitionSteps){
        // transition is over
        [theTimer invalidate];
        effectTimer = nil;
        
        // move background image up to foreground
        [foregroundImage release];
        foregroundImage = backgroundImage;
        [foregroundImage genQuad];
        backgroundImage = [[PSMGLImage alloc] initWithFrame:[self frame]];
        
        // put the camera back
        [self resetCamera];
        
        // determine how to continue
        if((imageIndex) >= [imageList count]){
            // all slides have been shown
            if([[NSUserDefaults standardUserDefaults] boolForKey:@"Slideshow repeat show"]){
                imageIndex = 0;
                [backgroundImage loadTextureFromFile:[imageList objectAtIndex:imageIndex]];
                imageIndex++;
                [self placeBackgroundImage];
            } else {
                [initiatorTimer invalidate];
                initiatorTimer = nil;
                // note this for later - if bigger than count, show is over
                imageIndex++;
            }
        } else {
            // more images to go
            [backgroundImage loadTextureFromFile:[imageList objectAtIndex:imageIndex]];
            imageIndex++;
            [self placeBackgroundImage];
        }
        
    }
    [self setNeedsDisplay:YES];
    stepsCompleted++;
}

// the pictures
-(void)setImageList:(NSMutableArray *)newImageList
{
    [newImageList retain];
    [imageList release];
    imageList = newImageList;
    
    // randomize if needed
    if([[NSUserDefaults standardUserDefaults] boolForKey:@"Slideshow randomize images"]){
        sranddev();
        int i, n = [imageList count];
        for(i = 0; i < n; i++) {
            // Swap the ith object with one randomly selected from [i,n).
            int destinationIndex = random() % (n - i) + i;
            [imageList exchangeObjectAtIndex:i withObjectAtIndex:destinationIndex];
        }
    }
}

-(void)killTimers
{
    [initiatorTimer invalidate];
    [effectTimer invalidate];
}

- (void)dealloc
{
    // clean up leftovers
    [foregroundImage release];
    [backgroundImage release];
    [imageList release];
    [super dealloc];
}


@end
