//
//  PSMToolbarMenuButtonCell.m
//  Pandora
//
//  Created by John Pannell on 7/12/06.
//  Copyright 2006 Positive Spin Media. All rights reserved.
//

#import "PSMToolbarMenuButtonCell.h"


@implementation PSMToolbarMenuButtonCell
- (id) init
{
    if (self = [super init])
    {
        _buttonCell = [[NSButtonCell alloc] initTextCell: @""];
        [_buttonCell setBordered:NO];
        [_buttonCell setHighlightsBy:NSContentsCellMask];
        [_buttonCell setImagePosition:NSImageOnly];
        
        _iconSize = NSMakeSize(32, 32);
    }
    
    return self;
}

- (void) dealloc
{
    [_buttonCell release];
    [_iconImage release];
    [super dealloc];
}


#pragma mark -
#pragma mark Accessors

- (NSSize)iconSize
{
    return _iconSize;
}


- (void)setIconSize:(NSSize)iconSize
{
    _iconSize = iconSize;
}

- (NSImage *)iconImage
{
    return _iconImage;
}


- (void)setIconImage:(NSImage *)iconImage
{
    [iconImage retain];
    [_iconImage release];
    _iconImage = iconImage;
    [_buttonCell setImage:_iconImage];
}

#pragma mark -
#pragma mark Mouse Tracking

- (NSPoint)menuPositionForFrame:(NSRect)cellFrame inView:(NSView *)controlView
{
    NSPoint result = [controlView convertPoint:cellFrame.origin toView:nil];
    result.x += 1.0;
    result.y -= [controlView frame].size.height;
    return result;
}

- (void)showMenuForEvent:(NSEvent *)theEvent controlView:(NSView *)controlView cellFrame:(NSRect)cellFrame
{
    NSPoint menuPosition = [self menuPositionForFrame:cellFrame inView:controlView];
    NSEvent *menuEvent;
    // create event for pop up menu with adjusted mouse position
    if([theEvent type] == NSKeyDown){
        menuEvent = [NSEvent keyEventWithType:[theEvent type] location:menuPosition modifierFlags:[theEvent modifierFlags] timestamp:[theEvent timestamp] windowNumber:[theEvent windowNumber] context:[theEvent context] characters:[theEvent characters] charactersIgnoringModifiers:[theEvent charactersIgnoringModifiers] isARepeat:[theEvent isARepeat] keyCode:[theEvent keyCode]];
    } else {
        menuEvent = [NSEvent mouseEventWithType:[theEvent type] location:menuPosition modifierFlags:[theEvent modifierFlags] timestamp:[theEvent timestamp] windowNumber:[theEvent windowNumber] context:[theEvent context] eventNumber:[theEvent eventNumber] clickCount:[theEvent clickCount] pressure:[theEvent pressure]];
    }
    [NSMenu popUpContextMenu:[self menu] withEvent:menuEvent forView:controlView];
}

- (BOOL)trackMouse:(NSEvent *)event inRect:(NSRect)cellFrame ofView:(NSView *)controlView untilMouseUp:(BOOL)untilMouseUp
{
    if ([event type] == NSKeyDown){// Keyboard event
        BOOL trackingResult = YES;
        
        unichar upAndDownArrowCharacters[2];
        upAndDownArrowCharacters[0] = NSUpArrowFunctionKey;
        upAndDownArrowCharacters[1] = NSDownArrowFunctionKey;
        NSString *upAndDownArrowString = [NSString stringWithCharacters: upAndDownArrowCharacters  length: 2];
        NSCharacterSet *upAndDownArrowCharacterSet = [NSCharacterSet characterSetWithCharactersInString: upAndDownArrowString];
        
        if ([[event characters] rangeOfCharacterFromSet: upAndDownArrowCharacterSet].location != NSNotFound){
            [self showMenuForEvent:event controlView:controlView cellFrame:cellFrame];
        } else if ([[event characters] rangeOfString: @" "].location != NSNotFound){
            [self performClick:controlView];
        }
        
        return trackingResult;
        
    } else {// Mouse event
        BOOL result = NO;
	NSDate *endDate;
	BOOL done = NO;
	// catch next mouse-dragged or mouse-up event until timeout
	NSEvent *upEvent;
	while (!done) { // loop ...
            endDate = [NSDate dateWithTimeIntervalSinceNow:0.5];
            upEvent = [NSApp nextEventMatchingMask:(NSLeftMouseUpMask) untilDate:endDate inMode:NSEventTrackingRunLoopMode dequeue:YES];
            if (upEvent) { // mouse event
                // check, if the mouse left our cell rect
                result = NSPointInRect([controlView convertPoint:[upEvent locationInWindow] fromView:nil], cellFrame);
                done = YES;
                
                if (done && result) {
                    [self performClick:controlView];
                }
            } else { // show menu
                done = YES;
                result = YES;
                [self showMenuForEvent:event controlView:controlView cellFrame:cellFrame];
            }
	} // while (!done)
	return result;
    }
}



- (NSView *)myParentView
{ 
    return [[myParentView retain] autorelease]; 
}

- (void)setMyParentView:(NSView *)newMyParentView 
{
    if (myParentView != newMyParentView) {
        [myParentView autorelease];
        myParentView = [newMyParentView retain];
    }
}

- (void) performClick: (id) sender
{    
    [_buttonCell performClick: sender];
    [super performClick: sender];
}

#pragma mark -
#pragma mark Drawing

- (void)drawWithFrame:(NSRect)cellFrame inView:(NSView *)controlView
{    
    if ([[controlView window] firstResponder] == controlView &&
        [controlView respondsToSelector: @selector(selectedCell)] &&
        [controlView performSelector: @selector(selectedCell)] == self)
    {
        //[_buttonCell setShowsFirstResponder: YES];
        [_buttonCell setShowsFirstResponder: NO]; // I don't like the appearance
    } else {
        [_buttonCell setShowsFirstResponder: NO];
    }
    
    [_buttonCell drawWithFrame:cellFrame inView:controlView];
}


- (void)highlight:(BOOL)flag  withFrame:(NSRect)cellFrame  inView:(NSView *)controlView
{  
    [_buttonCell highlight:flag  withFrame:cellFrame  inView:controlView];
    [super highlight:flag  withFrame:cellFrame  inView:controlView];
}

-(void)encodeWithCoder:(NSCoder *)coder {
    [super encodeWithCoder:coder];
    [coder encodeObject:[self iconImage] forKey:@"MCIcon"];
    [coder encodeSize:[self iconSize] forKey:@"MCIconSize"];
    
}

- (id)initWithCoder:(NSCoder *)coder {
    
    self = [super initWithCoder:coder];
    [self setIconImage: [coder decodeObjectForKey:@"MCIcon"]];
    [self setIconSize:[coder decodeSizeForKey:@"MCIconSize"]];
    
    _buttonCell = [[NSButtonCell alloc] initTextCell:@""];	
    [_buttonCell setBordered:NO];
    return self;
}

@end
