/*
        ScalingScrollView.m
        Copyright (c) 1995-2003 by Apple Computer, Inc., all rights reserved.
        Author: Mike Ferris
*/

#import <Cocoa/Cocoa.h>
#import "ScalingScrollView.h"
#import "PSMFlippedImageView.h"
#import "PSMPopUpButtonCell.h"
#import "PSMTexturedTextFieldCell.h"

   
static NSString *_NSDefaultScaleMenuLabels[] = {/* @"Set...", */ @"10%", @"25%", @"50%", @"75%", @"100%", @"128%", @"200%", @"400%", @"800%", @"1600%"};
static float _NSDefaultScaleMenuFactors[] = {/* 0.0, */ 0.1, 0.25, 0.5, 0.75, 1.0, 1.28, 2.0, 4.0, 8.0, 16.0};

@implementation ScalingScrollView

- (id)initWithFrame:(NSRect)rect {
    if ((self = [super initWithFrame:rect])) {
        scaleFactor = [[NSUserDefaults standardUserDefaults] floatForKey:@"Zoom scale"];
    }
    return self;
}

- (void)awakeFromNib
{
    scaleFactor = [[NSUserDefaults standardUserDefaults] floatForKey:@"Zoom scale"];
}

- (void)makeInfoTextField
{
    // create and configure info text field
    if(_infoTextField == nil){
        _infoTextField = [[NSTextField alloc] initWithFrame:NSMakeRect(0.0, 0.0, 200.0, 16.0)];
        [_infoTextField setCell:[[PSMTexturedTextFieldCell alloc] init]];
        [_infoTextField setStringValue:@""];
        [self addSubview:_infoTextField];
        [_infoTextField release];
    }
        
}

- (void)makeScalePopUpButton {
    if (_scalePopUpButton == nil) {
        unsigned cnt, numberOfDefaultItems = (sizeof(_NSDefaultScaleMenuLabels) / sizeof(NSString *));
        id curItem;

        // create it
        _scalePopUpButton = [[NSPopUpButton allocWithZone:[self zone]] initWithFrame:NSMakeRect(0.0, 0.0, 80.0, 16.0) pullsDown:NO];
        [_scalePopUpButton setCell:[[PSMPopUpButtonCell alloc] init]];
        [[_scalePopUpButton cell] setBezelStyle:NSShadowlessSquareBezelStyle];
        [[_scalePopUpButton cell] setArrowPosition:NSPopUpArrowAtBottom];
        [[_scalePopUpButton cell] setControlSize:NSSmallControlSize];
        
        // fill it
        [_scalePopUpButton addItemWithTitle:@"Scale to Fit"];
        [[_scalePopUpButton itemAtIndex:0] setRepresentedObject:[NSNumber numberWithFloat:-1.0]];
        [[_scalePopUpButton menu] addItem:[NSMenuItem separatorItem]];
        for (cnt = 0; cnt < numberOfDefaultItems; cnt++) {
            [_scalePopUpButton addItemWithTitle:NSLocalizedStringFromTable(_NSDefaultScaleMenuLabels[cnt], @"ZoomValues", nil)];
            curItem = [_scalePopUpButton itemAtIndex:cnt + 2];
            if (_NSDefaultScaleMenuFactors[cnt] != 0.0) {
                [curItem setRepresentedObject:[NSNumber numberWithFloat:_NSDefaultScaleMenuFactors[cnt]]];
            }
        }
        // set from default
        [_scalePopUpButton selectItemAtIndex:[_scalePopUpButton indexOfItemWithRepresentedObject:[NSNumber numberWithFloat:[[NSUserDefaults standardUserDefaults] floatForKey:@"Zoom scale"]]]];

        // hook it up
        [_scalePopUpButton setTarget:self];
        [_scalePopUpButton setAction:@selector(scalePopUpAction:)];

        // set a suitable font
        [_scalePopUpButton setFont:[NSFont toolTipsFontOfSize:10]];

	// don't let it become first responder
	[_scalePopUpButton setRefusesFirstResponder:YES];

        // put it in the scrollview
        [self addSubview:_scalePopUpButton];
        [_scalePopUpButton release];
    }
}

- (void)tile {
    // Let the superclass do most of the work.
    [super tile];

    NSScroller *horizScroller;
    NSRect horizScrollerFrame, buttonFrame, infoFrame;
    
    if (!_scalePopUpButton) [self makeScalePopUpButton];
    if (!_infoTextField) [self makeInfoTextField];

    horizScroller = [self horizontalScroller];
    horizScrollerFrame = [horizScroller frame];
    buttonFrame = [_scalePopUpButton frame];
    infoFrame = [_infoTextField frame];

    // Now we'll just adjust the horizontal scroller size and set the button size and location.
    horizScrollerFrame.size.width = horizScrollerFrame.size.width - buttonFrame.size.width - infoFrame.size.width;
    horizScrollerFrame.origin.x = horizScrollerFrame.origin.x + buttonFrame.size.width + infoFrame.size.width;
    [horizScroller setFrame:horizScrollerFrame];

    buttonFrame.origin.x = horizScrollerFrame.origin.x - buttonFrame.size.width - infoFrame.size.width;
    buttonFrame.size.height = horizScrollerFrame.size.height + 1.0;
    buttonFrame.origin.y = [self frame].size.height - buttonFrame.size.height;
    [_scalePopUpButton setFrame:buttonFrame];
    
    infoFrame.origin.x = horizScrollerFrame.origin.x - infoFrame.size.width;
    infoFrame.origin.y = buttonFrame.origin.y;
    [_infoTextField setFrame:infoFrame];
    
}

- (void)scalePopUpAction:(id)sender {
    NSNumber *selectedFactorObject = [[sender selectedCell] representedObject];
    
    if (selectedFactorObject == nil) {
        NSLog(@"Scale popup action: setting arbitrary zoom factors is not yet supported.");
        return;
    } else {
        [self setScaleFactor:[selectedFactorObject floatValue] adjustPopup:NO];
    }
    [[NSUserDefaults standardUserDefaults] setFloat:[selectedFactorObject floatValue] forKey:@"Zoom scale"];
    // set the menu as well
    [[[[[[[self menu] itemWithTitle:@"View"] submenu] itemWithTitle:@"Zoom"] submenu] itemArray] makeObjectsPerformSelector:@selector(setState:) withObject:NSOffState];
    [[[[[[[self menu] itemWithTitle:@"View"] submenu] itemWithTitle:@"Zoom"] submenu] itemAtIndex:[[[[[[self menu] itemWithTitle:@"View"] submenu] itemWithTitle:@"Zoom"] submenu] indexOfItemWithRepresentedObject:selectedFactorObject]] setState:NSOnState];
}

- (float)scaleFactor {
    return scaleFactor;
}

- (void)setScaleFactor:(float)newScaleFactor adjustPopup:(BOOL)flag {
    if (scaleFactor != newScaleFactor) {
	NSSize curDocFrameSize, newDocBoundsSize;
	NSView *clipView = [[self documentView] superview];
	
        if (flag) {
            [_scalePopUpButton selectItemAtIndex:[_scalePopUpButton indexOfItemWithRepresentedObject:[NSNumber numberWithFloat:newScaleFactor]]];
        }
        scaleFactor = newScaleFactor;
	
        if([[self documentView] isMemberOfClass:[NSTextView class]]){
            if(scaleFactor != -1.0){
                // Get the frame.  The frame must stay the same.
                curDocFrameSize = [clipView frame].size;
                
                // The new bounds will be frame divided by scale factor
                newDocBoundsSize.width = curDocFrameSize.width / scaleFactor;
                newDocBoundsSize.height = curDocFrameSize.height / scaleFactor;
                
                [clipView setBoundsSize:newDocBoundsSize];
            }
        } else {
            // image view
            [[self documentView] scaleFrameBy:scaleFactor];
        }
    }
}

- (void)setHasHorizontalScroller:(BOOL)flag {
    if (!flag) [self setScaleFactor:1.0 adjustPopup:NO];
    [super setHasHorizontalScroller:flag];
}

- (void)viewDidEndLiveResize
{
    if([[self documentView] isMemberOfClass:[NSTextView class]]){
        if(scaleFactor != -1.0){
            NSSize curDocFrameSize, newDocBoundsSize;
            NSView *clipView = [[self documentView] superview];
            // Get the frame.  The frame must stay the same.
            curDocFrameSize = [clipView frame].size;
            
            // The new bounds will be frame divided by scale factor
            newDocBoundsSize.width = curDocFrameSize.width / scaleFactor;
            newDocBoundsSize.height = curDocFrameSize.height / scaleFactor;
            
            [clipView setBoundsSize:newDocBoundsSize];
        }
    } else {
        // image view
        [[self documentView] scaleFrameBy:scaleFactor];
    }
}

- (NSTextField *)infoTextField
{
    return _infoTextField;
}

@end
