//
//  ThumbnailMatrix.h
//  NetScrape
//
//  Created by John Pannell on Fri Sep 27 2002.
//  Copyright (c) 2002 Positive Spin Media. All rights reserved.
//

#import <AppKit/AppKit.h>
#import "PSMWebFile.h"


@interface ThumbnailMatrix : NSMatrix
{
    int         _numberOfColumns;
    int         _currentRow;                // for placement of new images
    int         _currentColumn;             // for placement of new images
    int         _numImages;
    NSEvent     *_lastMouseDownEvent;       // keep this for dragging reference
    BOOL        drawForDrop;                // alter drawing if about to get dropped on
    NSPoint     dropMouseLocation;          // location of the mouse during drag/drop
    id          _engine;                    // the object feeding this matrix
}

// accessors
- (NSEvent *)lastMouseDownEvent;
- (void)setLastMouseDownEvent:(NSEvent *)event;
- (BOOL)drawForDrop;
- (NSPoint)dropMouseLocation;
- (id)engine;
- (void)setEngine:(id)engine;

// place an image
- (void)addScrapedFile:(id)object;
- (void)injectPastedFile:(PSMWebFile *)file atIndex:(int)index;
- (void)moveSelectedCellsToIndex:(int)injectionIndex;

// remove images
- (void)deleteSelectedCellsWithTrashing;
- (void)deleteSelectedCellsWithoutTrashing;
- (void)removeCellsWithFiles:(NSMutableSet *)sourceFiles;

// configure matrix
- (void)setNumberOfColumns:(int)number;
- (int)numberOfColumns;

// accessors
- (int)numImages;

- (unsigned long long)diskSpace;
- (NSString *)imageCountString;
- (NSMutableArray *)slideShowPaths;

// override so that I can resize the cells myself
- (void)resizeWithOldSuperviewSize:(NSSize)oldFrameSize;
- (void)viewDidEndLiveResizeWithWidth:(float)width;
- (void)adjustSize;

// drag
- (BOOL)acceptsFirstMouse:(NSEvent *)theEvent;
- (void)mouseDragged:(NSEvent *)theEvent;
- (unsigned int)draggingSourceOperationMaskForLocal:(BOOL)isLocal;
- (void)draggedImage:(NSImage *)anImage endedAt:(NSPoint)aPoint operation:(NSDragOperation)operation;
- (BOOL)shouldDelayWindowOrderingForEvent:(NSEvent *)theEvent;
// and drop
- (NSDragOperation)draggingEntered:(id <NSDraggingInfo>)sender;
- (NSDragOperation)draggingUpdated:(id <NSDraggingInfo>)sender;
- (void)draggingExited:(id <NSDraggingInfo>)sender;
- (BOOL)prepareForDragOperation:(id <NSDraggingInfo>)sender;
- (BOOL)performDragOperation:(id <NSDraggingInfo>)sender;
- (void)concludeDragOperation:(id <NSDraggingInfo>)sender;

// cut paste drag drop convenience
- (BOOL)readSlideFromPasteboard:(NSPasteboard *)pb;
- (void)writeSlideToPasteboard:(NSPasteboard *)pb;
- (NSString *)filePathForCell:(id)cell;
- (NSArray *)selectedFiles;
+ (NSSize)subtractPoints:(NSPoint)a_1 from:(NSPoint)a_2;
- (int)injectionIndexForPoint:(NSPoint)point;
- (void)reassignCellsFromArray:(NSMutableArray *)array;

// select all - override for behavior
- (void)selectAll:(id)sender;

// class convenience
+ (int)correctNumberOfColumns:(float)width;

@end
