//
//  PSMHTMLParser.m
//  Pandora
//
//  Created by John Pannell on 3/14/06.
//  Copyright 2006 Positive Spin Media. All rights reserved.
//

#import "PSMHTMLParser.h"
#import "PSMWebResource.h"
#import "TEC.h"


@implementation PSMHTMLParser

#pragma mark -
#pragma mark Creation/Destruction

- (id)initWithHTMLData:(NSMutableData *)data fromResource:(PSMWebResource *)resource;
{
    if(self = [super init]){
        [self setHTMLData:data];
        [self setResource:resource];
        
        _interestingTags = [[NSSet alloc] initWithObjects:@"a", @"img", @"area", @"frame", @"iframe", @"object", @"embed", @"applet", @"input", nil];
        _interestingAttributes = [[NSDictionary alloc] initWithObjectsAndKeys:@"href", @"a", @"src", @"img", @"href", @"area", @"src", @"frame", @"src", @"iframe", @"data", @"object", @"src", @"embed", @"code", @"applet", @"type", @"input", nil];
        
        [self setSourceURL:[resource sourceURL]];
    }
    
    return self;
}

- (void)dealloc
{
    [_htmldata release];
    [_resource release];
    [_interestingTags release];
    [_interestingAttributes release];
    [_sourceURL release];
    [super dealloc];
}
    
#pragma mark -
#pragma mark Accessors

- (NSMutableData *)htmldata
{
    return _htmldata;
}

- (void)setHTMLData:(NSMutableData *)data
{
    [data retain];
    [_htmldata release];
    _htmldata = data;
}

- (PSMWebResource *)resource
{
    return _resource;
}

- (void)setResource:(PSMWebResource *)resource
{
    [resource retain];
    [_resource release];
    _resource = resource;
}

- (NSString *)sourceURL
{
    return _sourceURL;
}

- (void)setSourceURL:(NSString *)string
{
    [string retain];
    [_sourceURL release];
    _sourceURL = string;
}

#pragma mark -
#pragma mark Functionality

- (void)parse
{
    // create a string from my downloaded data
    NSString *htmlString;
    
    // old default unicode way, so as not to break the way it currently works
    htmlString = [[NSString alloc] initWithData:_htmldata encoding:NSUTF8StringEncoding];
    
    if(!htmlString){
        // use hint from the server, if we have one
        if([[_resource textEncoding] length] > 0){
            [htmlString release];
            //NSLog(@"%@", [_resource textEncoding]);
            CFStringEncoding enc = CFStringConvertIANACharSetNameToEncoding((CFStringRef)[_resource textEncoding]);
            if(enc != kCFStringEncodingInvalidId){
                UInt32 nsEnc = CFStringConvertEncodingToNSStringEncoding(enc);
                htmlString = [[NSString alloc] initWithData:_htmldata encoding:nsEnc];
            }
        }
    }
    
    if(!htmlString){
        // STILL didn't work?  Try to sniff out encoding
        [htmlString release];
        int i = 0;
        TECSniffer *sniffer = [[TECSniffer alloc] initWithWebTextEncodings];
        NSArray *results = [sniffer sniff:_htmldata];
        while(!htmlString && i < [results count]){
            htmlString = [[NSString alloc] initWithData:_htmldata encoding:[[results objectAtIndex:i] unsignedIntValue]];
        }
    }
        
        
    //NSLog(@"%@", htmlString);
    // what I'm looking for
    NSCharacterSet *tagStart = [NSCharacterSet characterSetWithCharactersInString:@"<"];
    NSMutableCharacterSet *temp = [[NSCharacterSet whitespaceAndNewlineCharacterSet] mutableCopy];
    [temp addCharactersInString:@"/>"];
    NSCharacterSet *tagEnd = [temp copy];
    [temp release];
    NSCharacterSet *tagEndBracket = [NSCharacterSet characterSetWithCharactersInString:@">"];
    
    // storage
    NSString *tagBuffer = [[[NSMutableString alloc] init] autorelease];
    NSString *attributeBuffer = [[[NSMutableString alloc] init] autorelease];
    
    // scanner
    NSScanner *scanner = [[NSScanner alloc] initWithString:htmlString];
    [scanner setCaseSensitive:NO];
    [scanner setCharactersToBeSkipped:nil];
    
    while(![scanner isAtEnd]){
        [scanner scanUpToCharactersFromSet:tagStart intoString:nil];
        [scanner scanUpToCharactersFromSet:[NSCharacterSet letterCharacterSet] intoString:nil];
        [scanner scanUpToCharactersFromSet:tagEnd intoString:&tagBuffer];
        
        if([tagBuffer length] > 0){
            if([_interestingTags containsObject:[tagBuffer lowercaseString]]){
                [scanner scanUpToCharactersFromSet:tagEndBracket intoString:&attributeBuffer];
                NSRange attrRange = [attributeBuffer rangeOfString:[_interestingAttributes objectForKey:[tagBuffer lowercaseString]] options:NSCaseInsensitiveSearch];
                if(attrRange.length > 0){
                    // this tag has an attribute we are interested in... let's get it
                    NSString *value = [[self findValueForAttribute:[_interestingAttributes objectForKey:[tagBuffer lowercaseString]] inString:attributeBuffer] retain];
                    if(value != nil){
                        NSURL *formedURL = [NSURL URLWithString:[value stringByAddingPercentEscapesUsingEncoding:NSASCIIStringEncoding] relativeToURL:[NSURL URLWithString:_sourceURL]];
                        if(([[formedURL scheme] isEqualToString:@"http"]) || ([[formedURL scheme] isEqualToString:@"https"])){
                            //NSLog(@"%@ %@ %@", tagBuffer, [_interestingAttributes objectForKey:[tagBuffer lowercaseString]], [formedURL absoluteString]);
                            if([tagBuffer isEqualToString:@"img"]){
                                [_resource proposeFoundImage:[formedURL absoluteString]];
                            } else {
                                [_resource proposeFoundLink:[formedURL absoluteString]];
                            }
                        }
                    }
                    
                    attributeBuffer = @"";
                    [value release];
                }
            }
        }
        tagBuffer = @"";
    }
    
    [scanner release];
    [htmlString release];
    [tagEnd release];
    [_resource doneParsing];
    [self release];
}

- (NSString *)findValueForAttribute:(NSString *)attribute inString:(NSString *)tagContents
{
    // scanner
    NSScanner *scanner = [[NSScanner alloc] initWithString:tagContents];
    [scanner setCaseSensitive:NO];
    [scanner setCharactersToBeSkipped:nil];
    
    // characters
    NSMutableCharacterSet *temp = [[NSCharacterSet whitespaceAndNewlineCharacterSet] mutableCopy];
    [temp addCharactersInString:@"'=\"\\"];
    NSCharacterSet *valueStartMarkers = [temp copy];
    [temp release];
    
    temp = [[NSCharacterSet whitespaceAndNewlineCharacterSet] mutableCopy];
    [temp addCharactersInString:@"'\"\\"];
    NSCharacterSet *valueEndMarkers = [temp copy];
    [temp release];
    
    NSString *valueString = [NSString stringWithString:@""];
    [scanner scanUpToString:attribute intoString:nil];
    [scanner scanUpToCharactersFromSet:valueStartMarkers intoString:nil];
    [scanner scanCharactersFromSet:valueStartMarkers intoString:nil];
    [scanner scanUpToCharactersFromSet:valueEndMarkers intoString:&valueString];
    
    // authentication form?
    if(([attribute isEqualToString:@"type"]) && ([valueString caseInsensitiveCompare:@"password"] == NSOrderedSame)){
        //NSLog(@"found auth form");
        [_resource foundAuthForm];
        return nil;
    }

    [scanner release];
    [valueStartMarkers release];
    [valueEndMarkers release];
    return valueString;
}
    

@end
