//
//  PSMWebFile.m
//  Pandora
//
//  Created by John Pannell on 5/25/06.
//  Copyright 2006 Positive Spin Media. All rights reserved.
//

#import "PSMWebFile.h"
#import "PasteController.h"


@implementation PSMWebFile

#pragma mark -
#pragma mark Creation/Destruction

- (id)initWithResourceID:(int)value
{
    if(self = [super init]){
        _resourceID = value;
    }
    return self;
}

- (id)initWithSourceURL:(NSString *)url
{
    if(self = [super init]){
        [self setSourceURL:url];
        _resourceID = 0;
        _actualContentLength = 0;
    }
    return self;
}

- (void)dealloc
{
    [_sourceURL release];
    [_parentWebPage release];
    [_MIMEType release];
    [_thumbnail release];
    [_storagePath release];
    [_aliasData release];
    [super dealloc];
}

#pragma mark -
#pragma mark Accessors

- (int)resourceID
{
    return _resourceID;
}

- (void)setResourceID:(int)value
{
    _resourceID = value;
}

- (NSString *)sourceURL
{
    return _sourceURL;
}

- (void)setSourceURL:(NSString *)string
{
    [string retain];
    [_sourceURL release];
    _sourceURL = string;
}

- (NSString *)parentWebPage
{
    return _parentWebPage;
}

- (void)setParentWebPage:(NSString *)string
{
    [string retain];
    [_parentWebPage release];
    _parentWebPage = string;
}

- (NSString *)MIMEType
{
    return _MIMEType;
}

- (void)setMIMEType:(NSString *)string
{
    [string retain];
    [_MIMEType release];
    _MIMEType = string;
}

- (long long)actualContentLength
{
    return _actualContentLength;
}

- (void)setActualContentLength:(long long)value
{
    _actualContentLength = value;
}

- (NSImage *)thumbnail
{
    return _thumbnail;
}

- (void)setThumbnail:(NSImage *)image
{
    [image retain];
    [_thumbnail release];
    _thumbnail = image;
}

- (NSString *)storagePath
{
    return _storagePath;
}

- (void)setStoragePath:(NSString *)path
{
    [path retain];
    [_storagePath release];
    _storagePath = path;
}

- (NSData *)aliasData
{
    return _aliasData;
}

- (void)setAliasData:(NSData *)data
{
    [data retain];
    [_aliasData release];
    _aliasData = data;
}

#pragma mark -
#pragma mark Archiving

- (void)encodeWithCoder:(NSCoder *)aCoder {
    [aCoder encodeValueOfObjCType:@encode(int) at:&_resourceID];
    [aCoder encodeObject:_sourceURL];
    [aCoder encodeObject:_parentWebPage];
    [aCoder encodeValueOfObjCType:@encode(long long) at:&_actualContentLength];
    [aCoder encodeObject:_MIMEType];
    [aCoder encodeObject:[_thumbnail TIFFRepresentation]];
    [aCoder encodeValueOfObjCType:@encode(int) at:&_pixelsWide];
    [aCoder encodeValueOfObjCType:@encode(int) at:&_pixelsHigh];
    [aCoder encodeObject:_storagePath];
    [aCoder encodeObject:_aliasData];
   
}

- (id)initWithCoder:(NSCoder *)aDecoder {
    [aDecoder decodeValueOfObjCType:@encode(int) at:&_resourceID];
    _sourceURL = [[aDecoder decodeObject] retain];
    _parentWebPage = [[aDecoder decodeObject] retain];
    [aDecoder decodeValueOfObjCType:@encode(long long) at:&_actualContentLength];
    _MIMEType = [[aDecoder decodeObject] retain];
    _thumbnail = [[NSImage alloc] initWithData:[aDecoder decodeObject]];
    [aDecoder decodeValueOfObjCType:@encode(int) at:&_pixelsWide];
    [aDecoder decodeValueOfObjCType:@encode(int) at:&_pixelsHigh];
    _storagePath = [[aDecoder decodeObject] retain];
    _aliasData = [[aDecoder decodeObject] retain];

    return self;
}

- (id)replacementObjectForPortCoder:(NSPortCoder *)encoder {
    if ([encoder isByref]){
        return [NSDistantObject proxyWithLocal:self connection:[encoder connection]];
    } else {
        return self;
    }
}

#pragma mark -
#pragma mark Copy/Paste Support

-(void)loadFileData:(id)sender
{
    // operates in separate thread
    NSAutoreleasePool *thePool = [[NSAutoreleasePool alloc] init];
    
    // retrieve info from disk given URL
    NSFileManager *fileManager = [NSFileManager defaultManager];
    NSDictionary *fileAttributes = [fileManager fileAttributesAtPath:[self storagePath] traverseLink:YES];
    if(fileAttributes != nil){
        [self setActualContentLength:[[fileAttributes objectForKey:NSFileSize] longLongValue]];
    }
    
    // make thumbnail
    if([self actualContentLength] > 0){
        NSString *pathToFile = [self storagePath];
        NSBitmapImageRep *sourceBitmapImageRep = [[NSBitmapImageRep alloc] initWithData:[NSData dataWithContentsOfFile:pathToFile]];
        _pixelsHigh = [sourceBitmapImageRep pixelsHigh];
        _pixelsWide = [sourceBitmapImageRep pixelsWide];
        
        CFURLRef imageLocation = CFURLCreateWithFileSystemPath(kCFAllocatorDefault, (CFStringRef)pathToFile, kCFURLPOSIXPathStyle, false);
        CGImageRef thumbRef = PSMCreateWebFileThumbnailFromImageSource(imageLocation);
        CFRelease(imageLocation);
        
        if(thumbRef != NULL){
            NSImage *slideImage;
            float scaleFactor;
            if(_pixelsHigh > _pixelsWide){
                slideImage = [NSImage imageNamed:@"vert_slide"];
                scaleFactor = 114.0/_pixelsHigh;
            } else {
                slideImage = [NSImage imageNamed:@"horiz_slide"];
                scaleFactor = 114.0/_pixelsWide;
            }
            NSRect rect = NSMakeRect(67.0-scaleFactor*_pixelsWide/2.0,67.0-scaleFactor*_pixelsHigh/2.0,scaleFactor*_pixelsWide,scaleFactor*_pixelsHigh);
            NSImage *thumbImage = [[NSImage alloc] initWithSize:NSMakeSize(134.0, 134.0)];
            [thumbImage lockFocus];
            CGContextDrawImage([[NSGraphicsContext currentContext] graphicsPort], *(CGRect*)&rect, thumbRef);
            [[NSGraphicsContext currentContext] setImageInterpolation:NSImageInterpolationHigh];
            [slideImage compositeToPoint:NSMakePoint(3.0,3.0) fromRect:NSMakeRect(0.0, 0.0, 128.0, 128.0) operation:NSCompositeSourceOver];
            [thumbImage unlockFocus];
            
            _thumbnail = [[NSImage alloc] initWithData:[thumbImage TIFFRepresentation]];
        
            [thumbImage release];
            [sender performSelectorOnMainThread:@selector(addScrapedFile:) withObject:self waitUntilDone:NO];
        } else {
            [sender performSelectorOnMainThread:@selector(addScrapedFile:) withObject:nil waitUntilDone:NO];
            [self release];
        }
            
    } else {
        [sender performSelectorOnMainThread:@selector(addScrapedFile:) withObject:nil waitUntilDone:NO];
        [self release];
    }
    
    // bye bye pool
    [thePool release];
    
    // kill thread (necessary?)
    [NSThread exit]; 
}

#pragma mark -
#pragma mark C Functions

CGImageRef PSMCreateWebFileThumbnailFromImageSource(CFURLRef url)
{
    // from page 230 of Programming With Quartz
    CGImageRef thumb = NULL;
    CGImageSourceRef imageSource = NULL;
    int maxThumbSize = 126;
    CFNumberRef maxThumbSizeRef;
    CFStringRef keys[3];
    CFTypeRef values[3];
    CFDictionaryRef options = NULL;
    
    imageSource = CGImageSourceCreateWithURL(url, NULL);
    if(imageSource == NULL)
        return NULL;
    
    maxThumbSizeRef = CFNumberCreate(NULL, kCFNumberIntType, &maxThumbSize);
    
    // options for thumb creation
    keys[0] = kCGImageSourceCreateThumbnailFromImageAlways;
    values[0] = (CFTypeRef)kCFBooleanTrue;
    
    keys[1] = kCGImageSourceThumbnailMaxPixelSize;
    values[1] = (CFTypeRef)maxThumbSizeRef;
    
    keys[2] = kCGImageSourceCreateThumbnailWithTransform;
    values[2] = (CFTypeRef)kCFBooleanTrue;
    
    options = CFDictionaryCreate(NULL, (const void **)keys, (const void **)values, 3, &kCFTypeDictionaryKeyCallBacks, &kCFTypeDictionaryValueCallBacks);
    CFRelease(maxThumbSizeRef);
    
    // make thumb
    thumb = CGImageSourceCreateThumbnailAtIndex(imageSource, 0, options);
    
    CFRelease(options);
    CFRelease(imageSource);
    
    if(thumb == NULL)
        return NULL;
    
    return thumb;
}


@end
