//
//  PSMWebResource.h
//  Pandora
//
//  Created by John Pannell on 2/14/06.
//  Copyright 2006 Positive Spin Media. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <Webkit/Webkit.h>

@class PSMWebSpider;

@interface PSMWebResource : NSObject {
    
    // beahvior
    BOOL            _isLoading;
    BOOL            _isSubmittingForm;
    BOOL            _formCompleted;
    
    // the bosses
    PSMWebSpider    *_spider;
    id              _plugin;
    
    // represented content
    int             _resourceID;        // unique ID number - for logging
    NSString        *_sourceURL;
    NSString        *_parentWebPage;
    long long       _expectedContentLength;
    long long       _actualContentLength;
    NSString        *_MIMEType;
    NSString        *_textEncoding;
    NSMutableData   *_content;
    NSImage         *_thumbnail;
    
    // size info
    int             _pixelsWide;
    int             _pixelsHigh;
    
    // location on disk
    NSString        *_storagePath;
    NSData          *_aliasData;
    
    // declogger
    NSTimer         *_clogTimer;
    NSURLConnection *myConnection;
    
    // link depth
    int             _linkDepth;
    
    // timing
    NSDate          *_identifyStartDate;
    NSDate          *_loadStartDate;
    
}

// creation/destruction
- (id)initWithSourceURL:(NSString *)url fromSpider:(PSMWebSpider *)spider withPlugin:(id)plugin;
- (void)dispose;
- (void)dealloc;

// accessors
- (int)resourceID;
- (void)setResourceID:(int)value;
- (BOOL)isLoading;
- (void)setIsLoading:(BOOL)value;
- (BOOL)isSubmittingForm;
- (void)setIsSubmittingForm:(BOOL)value;
- (PSMWebSpider *)spider;
- (void)setSpider:(PSMWebSpider *)spider;
- (id)plugin;
- (void)setPlugin:(id)object;
- (NSString *)sourceURL;
- (void)setSourceURL:(NSString *)string;
- (NSString *)parentWebPage;
- (void)setParentWebPage:(NSString *)string;
- (long long)expectedContentLength;
- (void)setExpectedContentLength:(long long)value;
- (NSString *)MIMEType;
- (void)setMIMEType:(NSString *)string;
- (NSString *)textEncoding;
- (void)setTextEncoding:(NSString *)string;
- (long long)actualContentLength;
- (void)setActualContentLength:(long long)value;
- (NSMutableData *)content;
- (void)setContent:(NSMutableData *)content;
- (NSImage *)thumbnail;
- (void)setThumbnail:(NSImage *)image;
- (NSString *)storagePath;
- (void)setStoragePath:(NSString *)path;
- (NSData *)aliasData;
- (void)setAliasData:(NSData *)data;
- (int)linkDepth;
- (void)setLinkDepth:(int)value;

// functionality
- (void)identify;
- (void)parse;
- (void)load;
- (void)createThumbnail;
- (void)saveDataToFile;
- (void)declog:(NSTimer *)timer;

// NSURLConnection delegate methods - used for identifying, parsing, and loading resources
- (void)connection:(NSURLConnection *)connection didReceiveAuthenticationChallenge:(NSURLAuthenticationChallenge *)challenge;
- (void)connection:(NSURLConnection *)connection didCancelAuthenticationChallenge:(NSURLAuthenticationChallenge *)challenge;

-(NSURLRequest *)connection:(NSURLConnection *)connection willSendRequest:(NSURLRequest *)request redirectResponse:(NSURLResponse *)redirectResponse;
-(NSCachedURLResponse *)connection:(NSURLConnection *)connection willCacheResponse:(NSCachedURLResponse *)cachedResponse;

- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response;
- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data;
- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error;
- (void)connectionDidFinishLoading:(NSURLConnection *)connection;

// WebView frameload delegate methods - used to autofill and submit auth forms
- (void)webView:(WebView *)sender didFinishLoadForFrame:(WebFrame *)frame;

// parser
- (void)parseHTML;
- (void)proposeFoundLink:(NSString *)url;
- (void)proposeFoundImage:(NSString *)url;
- (void)doneParsing;
- (void)foundAuthForm;

// thumbnailing
- (void)successfulThumbnail;
- (void)failedThumbnail;
- (void)createThumbnailForBitmap;
- (void)createThumbnailForPDF;
- (void)createThumbnailForVideo;
- (void)createThumbnailForAudio;
- (void)createThumbnailDefault;

// C Functions
/*
CGImageRef PSMCreateThumbnailFromImageSource(CFURLRef url);
 */

@end
