//
//  PSMWebResource.m
//  Pandora
//
//  Created by John Pannell on 2/14/06.
//  Copyright 2006 Positive Spin Media. All rights reserved.
//
#import <Carbon/Carbon.h>
#import "PSMWebResource.h"
#import "PSMWebSpider.h"
#import "PSMWebSpiderPlugin.h"
#import "PSMHTMLParser.h"
#import "IconFamily.h"
#import "BDAlias.h"


@implementation PSMWebResource

#pragma mark -
#pragma mark Creation/Destruction

- (id)initWithSourceURL:(NSString *)url fromSpider:(PSMWebSpider *)spider withPlugin:(id)plugin
{
    if(self == [super init]){
        [self setSpider:spider];
        [self setPlugin:plugin];
        [self setSourceURL:url];
        [self setIsLoading:NO];
        [self setIsSubmittingForm:NO];
        _formCompleted = NO;
        _content = [[NSMutableData alloc] init];
        myConnection = nil;
    }
    return self;
}

- (NSString *)description
{
    return [NSString stringWithFormat:@"%@, %@", _sourceURL, _MIMEType];
}

- (void)dispose
{
    // user paused in process - toss in-process resources
    if(myConnection)
        [myConnection cancel];

    [_clogTimer invalidate];
    _clogTimer = nil;
    [_content release];
    _content = nil;
}

- (void)dealloc
{
    [_spider release];
    [_plugin release];
    [_sourceURL release];
    [_MIMEType release];
    [_textEncoding release];
    // [_content release]; already being done in process
    [_thumbnail release];
    [_storagePath release];
    [_aliasData release];
    [super dealloc];
}

#pragma mark -
#pragma mark Accessors

- (int)resourceID
{
    return _resourceID;
}

- (void)setResourceID:(int)value
{
    _resourceID = value;
}

- (BOOL)isLoading
{
    return _isLoading;
}

- (void)setIsLoading:(BOOL)value
{
    _isLoading = value;
}

- (BOOL)isSubmittingForm
{
    return _isSubmittingForm;
}

- (void)setIsSubmittingForm:(BOOL)value
{
    _isSubmittingForm = value;
}

- (PSMWebSpider *)spider
{
    return _spider;
}

- (void)setSpider:(PSMWebSpider *)object
{
    [object retain];
    [_spider release];
    _spider = object;
}

- (id)plugin
{
    return _plugin;
}

- (void)setPlugin:(id)object
{
    [object retain];
    [_plugin release];
    _plugin = object;
}

- (NSString *)sourceURL
{
    return _sourceURL;
}

- (void)setSourceURL:(NSString *)string
{
    [string retain];
    [_sourceURL release];
    _sourceURL = string;
}

- (NSString *)parentWebPage
{
    return _parentWebPage;
}

- (void)setParentWebPage:(NSString *)string
{
    [string retain];
    [_parentWebPage release];
    _parentWebPage = string;
}

- (long long)expectedContentLength
{
    return _expectedContentLength;
}

- (void)setExpectedContentLength:(long long)value
{
    _expectedContentLength = value;
}

- (NSString *)MIMEType
{
    return _MIMEType;
}

- (void)setMIMEType:(NSString *)string
{
    [string retain];
    [_MIMEType release];
    _MIMEType = string;
}

- (NSString *)textEncoding
{
    return _textEncoding;
}

- (void)setTextEncoding:(NSString *)string
{
    [string retain];
    [_textEncoding release];
    _textEncoding = string;
}

- (long long)actualContentLength
{
    return _actualContentLength;
}

- (void)setActualContentLength:(long long)value
{
    _actualContentLength = value;
}

- (NSMutableData *)content
{
    return _content;
}

- (void)setContent:(NSMutableData *)content
{
    [content retain];
    [_content release];
    _content = content;
}

- (NSImage *)thumbnail
{
    return _thumbnail;
}

- (void)setThumbnail:(NSImage *)image
{
    [image retain];
    [_thumbnail release];
    _thumbnail = image;
}

- (NSString *)storagePath
{
    return _storagePath;
}

- (void)setStoragePath:(NSString *)path
{
    [path retain];
    [_storagePath release];
    _storagePath = path;
}

- (NSData *)aliasData
{
    return _aliasData;
}

- (void)setAliasData:(NSData *)data
{
    [data retain];
    [_aliasData release];
    _aliasData = data;
}

- (int)linkDepth
{ 
    return _linkDepth;
}

- (void)setLinkDepth:(int)value
{
    _linkDepth = value;
}

#pragma mark -
#pragma mark Functionality

-(void)identify
{
	//NSLog(@"start identify");
    _identifyStartDate = [[NSDate date] retain];
    [self setIsLoading:NO];
    NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:[NSURL URLWithString:_sourceURL]];
    [request setHTTPMethod:@"HEAD"];
    [request setValue:[_spider userAgent] forHTTPHeaderField:@"User-Agent"];
    [request setTimeoutInterval:15.0];
    [request setCachePolicy:NSURLRequestReloadIgnoringCacheData];
    myConnection = [[NSURLConnection alloc] initWithRequest:request delegate:self];
    
    // declogger
    _clogTimer = [NSTimer scheduledTimerWithTimeInterval:30.0 target:self selector:@selector(declog:) userInfo:nil repeats:NO];
}

- (void)parse
{
    [self load];
}

- (void)load
{
    _loadStartDate = [[NSDate date] retain];
    [self setIsLoading:YES];
    NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:[NSURL URLWithString:_sourceURL]];
    [request setValue:[_spider userAgent] forHTTPHeaderField:@"User-Agent"];
    [request setTimeoutInterval:15.0];
    [request setCachePolicy:NSURLRequestReloadIgnoringCacheData];
    myConnection = [[NSURLConnection alloc] initWithRequest:request delegate:self];
    
    // declogger
    _clogTimer = [NSTimer scheduledTimerWithTimeInterval:30.0 target:self selector:@selector(declog:) userInfo:nil repeats:NO];
}

- (void)createThumbnail
{
    NSDate *startDate = [NSDate date];
    if([[NSBitmapImageRep imageUnfilteredFileTypes] containsObject:[[self MIMEType] lastPathComponent]]){
        NSBitmapImageRep *sourceBitmapImageRep = [[NSBitmapImageRep alloc] initWithData:_content];
        _pixelsHigh = [sourceBitmapImageRep pixelsHigh];
        _pixelsWide = [sourceBitmapImageRep pixelsWide];
        [sourceBitmapImageRep release];
        
         // filter for file dimensions
        NSDictionary *appPrefs = [[NSUserDefaults standardUserDefaults] persistentDomainForName:@"com.positivespinmedia.Pandora"];
        if([[appPrefs objectForKey:@"Skip for width"] boolValue]){
            if([[appPrefs objectForKey:@"Skip width condition"] intValue] == 0){ // skip if over
                if(_pixelsWide > [[appPrefs objectForKey:@"Skip width amount"] intValue]){
                    [[_spider session] logStatus:@"Skipped" forResourceID:[self resourceID]];
                    [[_spider session] logComment:@"Image too wide" forResourceID:[self resourceID]];
                    [[NSFileManager defaultManager] removeFileAtPath:_storagePath handler:nil];
                    [self release];
                    return;
                }
            } else { // skip if under
                if(_pixelsWide < [[appPrefs objectForKey:@"Skip width amount"] intValue]){
                    [[_spider session] logStatus:@"Skipped" forResourceID:[self resourceID]];
                    [[_spider session] logComment:@"Image not wide enough" forResourceID:[self resourceID]];
                    [[NSFileManager defaultManager] removeFileAtPath:_storagePath handler:nil];
                    [self release];
                    return;
                }
            }
        }
        
        if([[appPrefs objectForKey:@"Skip for height"] boolValue]){
            if([[appPrefs objectForKey:@"Skip height condition"] intValue] == 0){ // skip if over
                if(_pixelsHigh > [[appPrefs objectForKey:@"Skip height amount"] intValue]){
                    [[_spider session] logStatus:@"Skipped" forResourceID:[self resourceID]];
                    [[_spider session] logComment:@"Image too tall" forResourceID:[self resourceID]];
                    [[NSFileManager defaultManager] removeFileAtPath:_storagePath handler:nil];
                    [self release];
                    return;
                }
            } else { // skip if under
                if(_pixelsHigh < [[appPrefs objectForKey:@"Skip height amount"] intValue]){
                    [[_spider session] logStatus:@"Skipped" forResourceID:[self resourceID]];
                    [[_spider session] logComment:@"Image not tall enough" forResourceID:[self resourceID]];
                    [[NSFileManager defaultManager] removeFileAtPath:_storagePath handler:nil];
                    [self release];
                    return;
                }
            }
        }
                    
        
        [self createThumbnailForBitmap];
    } else {
    
    /*
    if([[[self MIMEType] lastPathComponent] isEqualToString:@"pdf"]){
        [self createThumbnailForPDF];
        return;
    }
    
    NSRange slashChar = [[self MIMEType] rangeOfString:@"/"];
    
    if([[[self MIMEType] substringToIndex:slashChar.index] isEqualToString:@"video"]){
        [self createThumbnailForVideo];
        return;
    }
    
    if([[[self MIMEType] substringToIndex:slashChar.index] isEqualToString:@"audio"]){
        [self createThumbnailForAudio];
        return;
    }

     */
        [self createThumbnailDefault];
    }
    NSDate *endDate = [NSDate date];
    [[_spider session] logThumbnailTime:[NSString stringWithFormat:@"%f", [endDate timeIntervalSinceDate:startDate]] forResourceID:[self resourceID]];
    
}

- (void)saveDataToFile
{
    NSDictionary *appPrefs = [[NSUserDefaults standardUserDefaults] persistentDomainForName:@"com.positivespinmedia.Pandora"];
    NSString *storageName;
    if([[appPrefs objectForKey:@"Enumerate file names"] boolValue]){
        // must tack on number in front of name
        NSString *numberString = [NSString stringWithFormat:@"0000%d-",[_spider savedFileNumber]];
        numberString = [numberString substringWithRange:NSMakeRange([numberString length]-5,5)];
        storageName = [NSString stringWithFormat:@"%@%@", numberString, [_sourceURL lastPathComponent]];
    } else {
        storageName = [_sourceURL lastPathComponent];
    }
        
    [self setStoragePath:[_spider createSafeFilePathFromBase:[NSString stringWithFormat:@"%@%@", [_spider storageLocationForMIMEType:[self MIMEType]], storageName]]];
    [_content writeToFile:[self storagePath] atomically:YES];
    
    // create alias
    BDAlias *alias = [BDAlias aliasWithPath:[self storagePath]];
    _aliasData = [[alias aliasData] retain];
}

- (void)declog:(NSTimer *)timer
{
    // resource has had no response in over 30 seconds
    [[_spider session] logComment:@"No response for over 30 seconds" forResourceID:[self resourceID]];
    [myConnection cancel];
    myConnection = nil;
    [_spider resourceConnectionFailed:self];
}


#pragma mark -
#pragma mark Parsing

- (void)parseHTML
{
    PSMHTMLParser *parser = [[PSMHTMLParser alloc] initWithHTMLData:_content fromResource:self];
    [parser parse]; // I'll hear back from this - and it releases itself when finished
}
    
- (void)proposeFoundLink:(NSString *)url
{
    NSDictionary *returnedURL = [_plugin postprocessURL:url];
    if(returnedURL != nil){
        if([returnedURL objectForKey:@"parent"] != nil){
            [_spider createWebResourceForURL:[returnedURL objectForKey:@"source"] action:@"identify" withParent:[returnedURL objectForKey:@"parent"] fromResource:self];
        } else {
            [_spider createWebResourceForURL:[returnedURL objectForKey:@"source"] action:@"identify" withParent:_sourceURL fromResource:self];
        }
    }
}

- (void)proposeFoundImage:(NSString *)url
{
    NSDictionary *returnedURL = [_plugin postprocessURL:url];
    if(returnedURL != nil){
        if([returnedURL objectForKey:@"parent"] != nil){
            [_spider createWebResourceForURL:[returnedURL objectForKey:@"source"] action:@"load" withParent:[returnedURL objectForKey:@"parent"] fromResource:self];
        } else {
            [_spider createWebResourceForURL:[returnedURL objectForKey:@"source"] action:@"load" withParent:_sourceURL fromResource:self];
        }
    }
}

- (void)foundAuthForm
{
    // load up resource in webview, fill out form, and submit
    if([_plugin authenticationInfo] != nil){
        [[_spider session] logStatus:@"Authenticating via Web Form" forResourceID:[self resourceID]];
        [self setIsSubmittingForm:YES];
        WebView *formLoader = [[WebView alloc] initWithFrame:NSMakeRect(0.0,0.0,200.0,200.0) frameName:@"loadFrame" groupName:@"loadGroup"];
        [formLoader setFrameLoadDelegate:self];
        NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:[NSURL URLWithString:_sourceURL]];
        [request setValue:[_spider userAgent] forHTTPHeaderField:@"User-Agent"];
        [request setTimeoutInterval:15.0];
        [request setCachePolicy:NSURLRequestReloadIgnoringCacheData];
        [[formLoader mainFrame] loadRequest:request];
    }
}

- (void)doneParsing
{
    [[_spider session] logParseTime:[NSString stringWithFormat:@"%f", [[NSDate date] timeIntervalSinceDate:_loadStartDate]] forResourceID:[self resourceID]];
    [_content release];
    _content = nil;
    if(!_isSubmittingForm)
        [_spider resourceParsed:self];
}

#pragma mark -
#pragma mark Thumbnailing

- (void)successfulThumbnail
{
    [_content release];
    _content = nil;
    [_spider resourceReady:self];
}

- (void)failedThumbnail
{
    [_content release];
    _content = nil;
    [[_spider session] logComment:@"Could not thumbnail file" forResourceID:[self resourceID]];
    [[_spider session] logStatus:@"Failed" forResourceID:[self resourceID]];
    [[NSFileManager defaultManager] removeFileAtPath:_storagePath handler:nil];
    [self release];
}

- (void)createThumbnailForBitmap
{
    NSString *pathToFile = [self storagePath];
    
    CFURLRef imageLocation = CFURLCreateWithFileSystemPath(kCFAllocatorDefault, (CFStringRef)pathToFile, kCFURLPOSIXPathStyle, false);
    CGImageSourceRef imageSource = NULL;
    int maxThumbSize = 126;
    CFNumberRef maxThumbSizeRef;
    CFStringRef keys[3];
    CFTypeRef values[3];
    CFDictionaryRef options = NULL;
    
    imageSource = CGImageSourceCreateWithURL(imageLocation, NULL);
    if(imageSource == NULL){
        NSLog(@"no image source");
        [self failedThumbnail];
        return;
    }
    
    if([(NSString *)CGImageSourceGetType(imageSource) rangeOfString:@"jpeg"].length > 0){
        // trying to read/write metadata - only for jpegs
        CGImageDestinationRef imageDest = CGImageDestinationCreateWithURL(imageLocation, CGImageSourceGetType(imageSource), CGImageSourceGetCount(imageSource), NULL);
        if(imageDest == NULL) NSLog(@"no image dest");
        
        CFDictionaryRef sourceMetadata = CGImageSourceCopyPropertiesAtIndex(imageSource, 0, NULL);
        if(sourceMetadata == NULL) NSLog(@"no source metadata");
        NSMutableDictionary *metadata = [(NSDictionary *)sourceMetadata mutableCopy];
        NSMutableDictionary *iptc;
        if([metadata objectForKey:@"{IPTC}"]){
                iptc = [(NSDictionary *)[metadata objectForKey:@"{IPTC}"] mutableCopy];
        } else {
                iptc = [[NSMutableDictionary alloc] init];
        }
        
        NSMutableArray *keywords;
        if([iptc objectForKey:@"Keywords"]){
            keywords = (NSMutableArray *)[[iptc objectForKey:@"Keywords"] mutableCopy];
            [keywords addObject:[_spider rootString]];
            [keywords addObject:[self sourceURL]];
        } else {
            keywords = [[NSMutableArray alloc] init];
            [keywords addObject:[_spider rootString]];
            [keywords addObject:[self sourceURL]];
        }
        
        [iptc setObject:[self parentWebPage] forKey:(NSString *)kCGImagePropertyIPTCSource];
        
        [iptc setObject:keywords forKey:@"Keywords"];
        NSDictionary *newMetadata = [[NSDictionary alloc] initWithObjectsAndKeys:iptc, @"{IPTC}", nil];
        int imageCount = CGImageSourceGetCount(imageSource);
            
        int i;
        for (i = 0; i < imageCount; i++)
        {
            CGImageDestinationAddImageFromSource(imageDest,
                                                 imageSource,
                                                 i,
                                                 (CFDictionaryRef)newMetadata);
        }
        if(!CGImageDestinationFinalize(imageDest)) NSLog(@"could not finalize");

        // clean up
        if(sourceMetadata != NULL)
            CFRelease(sourceMetadata);
        [keywords release];
        [iptc release];
        [metadata release];
        [newMetadata release];
        if(imageDest != NULL)
            CFRelease(imageDest);
        // end new metadata code
    }
    
    maxThumbSizeRef = CFNumberCreate(NULL, kCFNumberIntType, &maxThumbSize);
    
    // options for thumb creation
    keys[0] = kCGImageSourceCreateThumbnailFromImageAlways;
    values[0] = (CFTypeRef)kCFBooleanTrue;
    
    keys[1] = kCGImageSourceThumbnailMaxPixelSize;
    values[1] = (CFTypeRef)maxThumbSizeRef;
    
    keys[2] = kCGImageSourceCreateThumbnailWithTransform;
    values[2] = (CFTypeRef)kCFBooleanTrue;
    
    options = CFDictionaryCreate(NULL, (const void **)keys, (const void **)values, 3, &kCFTypeDictionaryKeyCallBacks, &kCFTypeDictionaryValueCallBacks);
    if(maxThumbSizeRef != NULL)
        CFRelease(maxThumbSizeRef);
    
    // make thumb
    CGImageRef thumbRef = CGImageSourceCreateThumbnailAtIndex(imageSource, 0, options);
    
    if(options != NULL)
        CFRelease(options);
    if(imageSource != NULL)
        CFRelease(imageSource);
    if(imageLocation != NULL)
        CFRelease(imageLocation);
    
    if(thumbRef != NULL){
        NSImage *slideImage;
        float scaleFactor;
        if(_pixelsHigh > _pixelsWide){
            slideImage = [NSImage imageNamed:@"vert_slide"];
            scaleFactor = 114.0/_pixelsHigh;
        } else {
            slideImage = [NSImage imageNamed:@"horiz_slide"];
            scaleFactor = 114.0/_pixelsWide;
        }
        NSRect rect = NSMakeRect(67.0-scaleFactor*_pixelsWide/2.0,67.0-scaleFactor*_pixelsHigh/2.0,scaleFactor*_pixelsWide,scaleFactor*_pixelsHigh);
        NSImage *thumbImage = [[NSImage alloc] initWithSize:NSMakeSize(134.0, 134.0)];
        [thumbImage lockFocus];
        CGContextDrawImage([[NSGraphicsContext currentContext] graphicsPort], *(CGRect*)&rect, thumbRef);
        [[NSGraphicsContext currentContext] setImageInterpolation:NSImageInterpolationHigh];
        [slideImage compositeToPoint:NSMakePoint(3.0,3.0) fromRect:NSMakeRect(0.0, 0.0, 128.0, 128.0) operation:NSCompositeSourceOver];
        [thumbImage unlockFocus];
        
        _thumbnail = [[NSImage alloc] initWithData:[thumbImage TIFFRepresentation]];
        
        // custom icon
        [[NSUserDefaults standardUserDefaults] synchronize];
        NSDictionary *appPrefs = [[NSUserDefaults standardUserDefaults] persistentDomainForName:@"com.positivespinmedia.Pandora"];
        if([[appPrefs objectForKey:@"Use custom icons"] boolValue]){
            IconFamily* iconFamily = [IconFamily iconFamilyWithThumbnailsOfImage:_thumbnail];
            [iconFamily setAsCustomIconForFile:_storagePath];
        }
        
        // write spotlight comment
		/*
        NSDictionary *errors = [NSDictionary dictionary];
        NSAppleEventDescriptor *firstParameter = [NSAppleEventDescriptor descriptorWithString:_storagePath];
        NSAppleEventDescriptor *secondParameter = [NSAppleEventDescriptor descriptorWithString:[NSString stringWithFormat:@"%@, %@, %@", [_spider rootString], _sourceURL, _parentWebPage]];
        // create and populate the list of parameters
        // note that the array starts at index 1
        NSAppleEventDescriptor *parameters = [NSAppleEventDescriptor listDescriptor];
        [parameters insertDescriptor:firstParameter atIndex:1];
        [parameters insertDescriptor:secondParameter atIndex:2];
        // create the AppleEvent target
        ProcessSerialNumber psn = { 0, kCurrentProcess };
        NSAppleEventDescriptor *target = [NSAppleEventDescriptor descriptorWithDescriptorType:typeProcessSerialNumber bytes:&psn length:sizeof(ProcessSerialNumber)];
        // create an NSAppleEventDescriptor with the method name
        // note that the name must be lowercase (even if it is uppercase in AppleScript)
        NSAppleEventDescriptor *handler = [NSAppleEventDescriptor descriptorWithString:[@"setSpotlightComment" lowercaseString]];
        // last but not least, create the event for an AppleScript subroutine
        // set the method name and the list of parameters
        NSAppleEventDescriptor *event = [NSAppleEventDescriptor appleEventWithEventClass:kASAppleScriptSuite eventID:kASSubroutineEvent targetDescriptor:target returnID:kAutoGenerateReturnID transactionID:kAnyTransactionID];
        [event setParamDescriptor:handler forKeyword:keyASSubroutineName];
        [event setParamDescriptor:parameters forKeyword:keyDirectObject];
        // at last, call the event in AppleScript
        if(![[_spider commentScript] executeAppleEvent:event error:&errors]) //NSLog(@"%@", errors);
            //[[_spider session] logComment:@"Could not set Finder Spotlight comment" forResourceID:[self resourceID]];
		 */
        
        [thumbImage release];
        [self successfulThumbnail];
    } else {
        // no thumb
        [self failedThumbnail];
    }    
}

- (void)createThumbnailForPDF
{
    
}

- (void)createThumbnailForVideo
{
    
}

- (void)createThumbnailForAudio
{
    
}

- (void)createThumbnailDefault
{
    NSImage *theIcon = [[NSWorkspace sharedWorkspace] iconForFile:_storagePath];
    [theIcon setSize:NSMakeSize(128.0,128.0)];
    NSImage *thumbImage = [[NSImage alloc] initWithSize:NSMakeSize(134.0, 134.0)];
    [thumbImage lockFocus];
    [theIcon compositeToPoint:NSMakePoint(3.0,3.0) fromRect:NSMakeRect(0.0, 0.0, 128.0, 128.0) operation:NSCompositeSourceOver];
    [thumbImage unlockFocus];
    
    _thumbnail = [[NSImage alloc] initWithData:[thumbImage TIFFRepresentation]];
    
    // custom icon
    /*
    [[NSUserDefaults standardUserDefaults] synchronize];
    NSDictionary *appPrefs = [[NSUserDefaults standardUserDefaults] persistentDomainForName:@"com.positivespinmedia.Pandora"];
    if([[appPrefs objectForKey:@"Use custom icons"] boolValue]){
        IconFamily* iconFamily = [IconFamily iconFamilyWithThumbnailsOfImage:_thumbnail];
        [iconFamily setAsCustomIconForFile:_storagePath];
    }
     */
    
    // write spotlight comment
	/*
    NSDictionary *errors = [NSDictionary dictionary];
    NSAppleEventDescriptor *firstParameter = [NSAppleEventDescriptor descriptorWithString:_storagePath];
    NSAppleEventDescriptor *secondParameter = [NSAppleEventDescriptor descriptorWithString:[NSString stringWithFormat:@"%@, %@, %@", [_spider rootString], _sourceURL, _parentWebPage]];
    // create and populate the list of parameters
    // note that the array starts at index 1
    NSAppleEventDescriptor *parameters = [NSAppleEventDescriptor listDescriptor];
    [parameters insertDescriptor:firstParameter atIndex:1];
    [parameters insertDescriptor:secondParameter atIndex:2];
    // create the AppleEvent target
    ProcessSerialNumber psn = { 0, kCurrentProcess };
    NSAppleEventDescriptor *target = [NSAppleEventDescriptor descriptorWithDescriptorType:typeProcessSerialNumber bytes:&psn length:sizeof(ProcessSerialNumber)];
    // create an NSAppleEventDescriptor with the method name
    // note that the name must be lowercase (even if it is uppercase in AppleScript)
    NSAppleEventDescriptor *handler = [NSAppleEventDescriptor descriptorWithString:[@"setSpotlightComment" lowercaseString]];
    // last but not least, create the event for an AppleScript subroutine
    // set the method name and the list of parameters
    NSAppleEventDescriptor *event = [NSAppleEventDescriptor appleEventWithEventClass:kASAppleScriptSuite eventID:kASSubroutineEvent targetDescriptor:target returnID:kAutoGenerateReturnID transactionID:kAnyTransactionID];
    [event setParamDescriptor:handler forKeyword:keyASSubroutineName];
    [event setParamDescriptor:parameters forKeyword:keyDirectObject];
    // at last, call the event in AppleScript
    if(![[_spider commentScript] executeAppleEvent:event error:&errors]) //NSLog(@"%@", errors);
        //[[_spider session] logComment:@"Could not set Finder Spotlight comment" forResourceID:[self resourceID]];
	 */
    
    [thumbImage release];
    [self successfulThumbnail];
    
}

#pragma mark -
#pragma mark NSURLConnection Delegate

- (void)connection:(NSURLConnection *)connection didReceiveAuthenticationChallenge:(NSURLAuthenticationChallenge *)challenge
{
    if([challenge previousFailureCount] > 0){
        [[challenge sender] cancelAuthenticationChallenge:challenge];
        return;
    }
    
    [[_spider session] logStatus:@"Authenticating via HTTP" forResourceID:[self resourceID]];
    if([challenge previousFailureCount] == 0){
        if([_plugin authenticationInfo] == nil){
            // I've got nothing
            [[challenge sender] cancelAuthenticationChallenge:challenge];
        } else {
            // create credential
            NSURLCredential *newCredential = [NSURLCredential credentialWithUser:[[[_plugin authenticationInfo] objectAtIndex:1] objectAtIndex:0] password:[[[_plugin authenticationInfo] objectAtIndex:1] objectAtIndex:1] persistence:NSURLCredentialPersistencePermanent];
            [[challenge sender] useCredential:newCredential forAuthenticationChallenge:challenge];
        }
    }
}

- (void)connection:(NSURLConnection *)connection didCancelAuthenticationChallenge:(NSURLAuthenticationChallenge *)challenge
{
    [[_spider session] logComment:@"Failed HTTP Authentication" forResourceID:[self resourceID]];
    [connection cancel];
    [connection release];
    myConnection = nil;
    [_clogTimer invalidate];
    _clogTimer = nil;
    [_spider resourceConnectionFailed:self];
}

-(NSURLRequest *)connection:(NSURLConnection *)connection willSendRequest:(NSURLRequest *)request redirectResponse:(NSURLResponse *)redirectResponse
{
    // if nothing has changed, return the request - for some reason, Leopard is issuing a willSendRequest: right off the bat?
    if([[[request URL] absoluteString] isEqualToString:[self sourceURL]]){
        return request;
    }
	
    // since the redirect is NOT the same as the original request, I'm going to trash the original connection and load the new asset
    [[_spider session] logComment:[NSString stringWithFormat:@"Redirected to %@", [[request URL] absoluteString]] forResourceID:[self resourceID]];
    [self setSourceURL:[[request URL] absoluteString]];
    [connection cancel];
    [connection release];
    myConnection = nil;
    [_clogTimer invalidate];
    _clogTimer = nil;
    if(_isLoading){
        [self load];
    } else {
        [self identify];
    }
    return nil;
}

-(NSCachedURLResponse *)connection:(NSURLConnection *)connection willCacheResponse:(NSCachedURLResponse *)cachedResponse
{
    return cachedResponse;
}

- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response
{
    // check if this is OK (not a 404 or the like)
    if([response respondsToSelector:@selector(statusCode)]){
        int statusCode = [(NSHTTPURLResponse *)response statusCode];
        if(statusCode >= 400){
            [[_spider session] logComment:[NSHTTPURLResponse localizedStringForStatusCode:statusCode] forResourceID:[self resourceID]];
            [connection cancel];
            [connection release];
            myConnection = nil;
            [_clogTimer invalidate];
            _clogTimer = nil;
            [_spider resourceConnectionFailed:self];
            return;
        }
    }
    
    // examine the response and note specifics - this could get called a number of times (FYI)
    [self setExpectedContentLength:[response expectedContentLength]];
    [self setMIMEType:[response MIMEType]];
    [self setTextEncoding:[response textEncodingName]]; // keep this hint for later
    [_content setLength:0];
    [_clogTimer setFireDate:[NSDate dateWithTimeIntervalSinceNow:30.0]];
}

- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data
{
    [_content appendData:data];
    //NSLog(@"%@", [[[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding] autorelease]);
    [[_spider session] logProgress:[NSString stringWithFormat:@"%@ of %@", [_spider fileSizeStringForUnsigned:[_content length]], [_spider fileSizeStringForLongLong:_expectedContentLength]] forResourceID:[self resourceID]];
    [_clogTimer setFireDate:[NSDate dateWithTimeIntervalSinceNow:30.0]];
}

- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error
{
    [connection cancel];
    [connection release];
    myConnection = nil;
    [_clogTimer invalidate];
    _clogTimer = nil;
    [[_spider session] logComment:[error localizedFailureReason] forResourceID:[self resourceID]];
    [_spider resourceConnectionFailed:self];
}

- (void)connectionDidFinishLoading:(NSURLConnection *)connection
{
    [connection cancel];
    [connection release];
    myConnection = nil;
    [_clogTimer invalidate];
    _clogTimer = nil;
    
    if(![self isLoading]){
        [[_spider session] logIdentifyTime:[NSString stringWithFormat:@"%f", [[NSDate date] timeIntervalSinceDate:_identifyStartDate]] forResourceID:[self resourceID]];
        [_spider resourceIdentified:self];
        return;
    } 
    
    // it is possible that something in a img tag will come up as HTML for MIME...
    if(([[self MIMEType] isEqualToString:@"text/html"]) && ([[_spider parseArray] containsObject:self])){
        [self setActualContentLength:[_content length]];
        [self parseHTML];
    } else {
        [self setActualContentLength:[_content length]];
        [[_spider session] logLoadTime:[NSString stringWithFormat:@"%f", [[NSDate date] timeIntervalSinceDate:_loadStartDate]] forResourceID:[self resourceID]];
        [_spider resourceLoaded:self];
    }
}

#pragma mark -
#pragma mark WebView: frameLoad delegate

- (void)webView:(WebView *)sender didFailProvisionalLoadWithError:(NSError *)error forFrame:(WebFrame *)frame
{
    [sender release];
    [[_spider session] logComment:[error localizedFailureReason] forResourceID:[self resourceID]];
    [_spider resourceParsed:self];
}

- (void)webView:(WebView *)sender serverRedirectedForDataSource:(WebFrame *)frame
{
    [[_spider session] logComment:[NSString stringWithFormat:@"Redirected to %@", [[[[frame dataSource] request] URL] absoluteString]] forResourceID:[self resourceID]];
}

- (void)webView:(WebView *)sender didFinishLoadForFrame:(WebFrame *)frame
{
    // if I haven't reached the successful end of the login process, submit again
    if([[[[[frame dataSource] request] URL] absoluteString] rangeOfString:[_plugin authSuccessURLFragment]].length == 0){
        [[_spider session] logComment:@"Found Web Form" forResourceID:[self resourceID]];
        // ask plugin for key value combos needed to fill out form
        NSArray *authInfo = [[_plugin authenticationInfo] retain];
        NSArray *inputKeys = [authInfo objectAtIndex:0];
        NSArray *inputValues = [authInfo objectAtIndex:1];
        
        // search form for same key value pairs and set values
        DOMDocument *document = [frame DOMDocument];
        DOMNodeList *formNodeList = [document getElementsByTagName:@"FORM"];
        int i;
        
        for(i=0; i < [formNodeList length]; i++){
            BOOL authForm = NO;
            DOMHTMLFormElement *formNode = (DOMHTMLFormElement *)[formNodeList item:i];
            // get inputs from form
            DOMNodeList *inputNodeList = [formNode getElementsByTagName:@"INPUT"];
            int j;
            
            for(j = 0; j < [inputNodeList length]; j++){
                DOMHTMLInputElement *inputNode = (DOMHTMLInputElement *)[inputNodeList item:j];
                if([inputKeys containsObject:[inputNode name]]){
                    [inputNode setValue:[inputValues objectAtIndex:[inputKeys indexOfObject:[inputNode name]]] forKey:@"value"];
                    authForm = YES;
                }
            }
            
            if(authForm){
                // submit this form
                [[_spider session] logComment:@"Submitted Web Form" forResourceID:[self resourceID]];
                [formNode submit];
                _formCompleted = YES;
            }
        }
    } else {
        // are we done?  ask plugin for fragment of URL that signifies success
        // some auth schemes redirect quite a bit
        if([[[[[frame dataSource] request] URL] absoluteString] rangeOfString:[_plugin authSuccessURLFragment]].length == 0)
            return;

        // this is the content on the backside of the form submission
        [[_spider session] logComment:@"Completed Web Form" forResourceID:[self resourceID]];
        if([_plugin parseAuthSuccessPage])
            [self proposeFoundLink:[[[[[sender mainFrame] dataSource] request] URL] absoluteString]];
        [sender release];
        [_spider resourceParsed:self];
    }
}

- (void)webView:(WebView *)sender didFailLoadWithError:(NSError *)error forFrame:(WebFrame *)frame
{
    [sender release];
    [[_spider session] logComment:[error localizedFailureReason] forResourceID:[self resourceID]];
}

/*
#pragma mark -
#pragma mark C Functions

CGImageRef PSMCreateThumbnailFromImageSource(CFURLRef url)
{
    // from page 230 of Programming With Quartz
    CGImageRef thumb = NULL;
    CGImageSourceRef imageSource = NULL;
    int maxThumbSize = 126;
    CFNumberRef maxThumbSizeRef;
    CFStringRef keys[3];
    CFTypeRef values[3];
    CFDictionaryRef options = NULL;
    
    imageSource = CGImageSourceCreateWithURL(url, NULL);
    if(imageSource == NULL)
        return NULL;
	
	// trying to read metadata
	CFURLRef newLocation = CFURLCreateWithFileSystemPath(kCFAllocatorDefault, CFSTR("/Users/johnp/Desktop/file.jpg"), kCFURLPOSIXPathStyle, false);
	CGImageDestinationRef imageDest = CGImageDestinationCreateWithURL(newLocation, CGImageSourceGetType(imageSource), CGImageSourceGetCount(imageSource), NULL);
	NSMutableDictionary *metadata = [(NSDictionary *)CGImageSourceCopyPropertiesAtIndex(imageSource, 0, NULL) mutableCopy];
	NSMutableDictionary *iptc;
	if([metadata objectForKey:@"{IPTC}"]){
		iptc = [(NSDictionary *)[metadata objectForKey:@"{IPTC}"] mutableCopy];
	} else {
		iptc = [NSMutableDictionary dictionary];
	}
	
	NSMutableString *keywords;
	if([iptc objectForKey:@"Keywords"]){
		keywords = (NSMutableString *)[[metadata objectForKey:@"Keywords"] mutableCopy];
		keywords = (NSMutableString *)[keywords stringByAppendingString:@", Important!"];
	} else {
		keywords = [NSMutableString string];
		keywords = (NSMutableString *)[keywords stringByAppendingString:@"Important"];
	}
	
	[iptc setObject:keywords forKey:@"Keywords"];
	[metadata setObject:iptc forKey:@"{IPTC}"];
	int imageCount = CGImageSourceGetCount(imageSource);
    int i;
    for (i = 0; i < imageCount; i++)
    {
        CGImageDestinationAddImageFromSource(imageDest,
                                             imageSource,
                                             i,
                                             (CFDictionaryRef)metadata);
    }
    //CGImageDestinationSetProperties(imageDest, (CFDictionaryRef)metadata);
    CGImageDestinationFinalize(imageDest);
	
	//NSLog(@"%s", CFDictionaryGetValue(CFDictionaryGetValue(metadata, kCGImagePropertyIPTCDictionary), CFSTR("Keywords")));
	NSLog(@"%@", (NSDictionary *)metadata);
	
	// end new metadata code
    
    maxThumbSizeRef = CFNumberCreate(NULL, kCFNumberIntType, &maxThumbSize);
    
    // options for thumb creation
    keys[0] = kCGImageSourceCreateThumbnailFromImageAlways;
    values[0] = (CFTypeRef)kCFBooleanTrue;
    
    keys[1] = kCGImageSourceThumbnailMaxPixelSize;
    values[1] = (CFTypeRef)maxThumbSizeRef;
    
    keys[2] = kCGImageSourceCreateThumbnailWithTransform;
    values[2] = (CFTypeRef)kCFBooleanTrue;
    
    options = CFDictionaryCreate(NULL, (const void **)keys, (const void **)values, 3, &kCFTypeDictionaryKeyCallBacks, &kCFTypeDictionaryValueCallBacks);
    CFRelease(maxThumbSizeRef);
    
    // make thumb
    thumb = CGImageSourceCreateThumbnailAtIndex(imageSource, 0, options);
    
    CFRelease(options);
    CFRelease(imageSource);
    
    if(thumb == NULL)
        return NULL;
    
    return thumb;
}

*/

@end
