//
//  PSMWebSpider.h
//  Pandora
//
//  Created by John Pannell on 2/10/06.
//  Copyright 2006 Positive Spin Media. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <Webkit/Webkit.h>
#import "PandoraSession.h"
#import "PandoraWebSpider.h"

@class PSMWebResource;

@interface PSMWebSpider : NSObject <PandoraWebSpider>{
    
    // pending process storage
    NSMutableArray      *_identifyQueue;
    NSMutableArray      *_parseQueue;
    NSMutableArray      *_loadQueue;
    
    // in-process storage
    NSMutableArray      *_identifyArray;
    NSMutableArray      *_parseArray;
    NSMutableArray      *_loadArray;
    
    // list of previously found URLs
    NSMutableSet        *_foundURLs;
    
    // my assigned plug in
    id                  _plugin;
    
    // my corresponding session in the Pandora application
    id<PandoraSession>  _session;
    
    // guidelines
    int                 _identifyDepth;         // number of resources identifying at a given moment
    int                 _parseDepth;            // number of resources parsing at a given moment
    int                 _loadDepth;             // number of resources loading at a given moment
    NSString            *_rootString;           // user entered string that started it all
    NSString            *_storageLocation;      // path to folder holding found resources
    NSString            *_customImageLocation;
    NSString            *_customAudioLocation;
    NSString            *_customVideoLocation;
    NSString            *_customDocumentLocation;
    NSMutableDictionary *_storageLookupTable;
    int                 _resourceNumber;        // for uniqueing resources
    NSString            *_userAgent;
    int                 _savedFileNumber;       // for enumerating file names
    
    // for writing spotlight comments
    NSAppleScript       *_commentScript;
    
    // pause/resume
    BOOL                _paused;
    int                 _successfulFiles;
    
}

// creation/destruction
- (id)init;
- (void)dealloc;

// accessors
- (NSString *)storageLocation;
- (NSString *)storageLocationForMIMEType:(NSString *)type;
- (void)setStorageLocation:(NSString *)path;
- (NSString *)rootString;
- (void)setRootString:(NSString *)value;
- (id)plugin;
- (int)resourceNumber;
- (NSMutableArray *)parseArray;
- (NSAppleScript *)commentScript;
- (oneway void)setSessionObject:(in byref id <PandoraSession>)session;
- (NSString *)userAgent;
- (id)session;
- (int)savedFileNumber;

// functionality
- (BOOL)loadPluginNamed:(NSString *)bundleName;
- (void)startProcessing;
- (void)pauseProcessing;
- (void)resumeProcessing;
- (void)stopProcessing;
- (void)createWebResourceForURL:(NSString *)url action:(NSString *)action withParent:(NSString *)parentURL fromResource:(PSMWebResource *)resource;

// verify that parent app is still present
- (void)parentAppCheck:(NSTimer *)timer;

// queue management
- (void)advanceIdentifyQueue;
- (void)advanceParseQueue;
- (void)advanceLoadQueue;

// mesages from resources
- (void)resourceIdentified:(PSMWebResource *)resource;
- (void)resourceParsed:(PSMWebResource *)resource;
- (void)resourceLoaded:(PSMWebResource *)resource;
- (void)resourceReady:(PSMWebResource *)resource;
- (void)resourceConnectionFailed:(PSMWebResource *)resource;

// convenience
- (NSString *)createSafeFilePathFromBase:(NSString *)creationPath;
- (NSString *)createSafeFolderPathFromBase:(NSString *)creationPath;
- (NSString *)fileSizeStringForLongLong:(long long)size;
- (NSString *)fileSizeStringForUnsigned:(unsigned)size;
- (BOOL)finishedProcessing;

@end
