//
//  PSMWebSpider.m
//  Pandora
//
//  Created by John Pannell on 2/10/06.
//  Copyright 2006 Positive Spin Media. All rights reserved.
//

#import "PSMWebSpider.h"
#import "PSMWebSpiderPlugin.h"
#import "PSMWebResource.h"
#import "PSMWebFile.h"


@implementation PSMWebSpider

#pragma mark -
#pragma mark Creation/Destruction

- (id)init
{
    if(self = [super init]){
        _identifyQueue = [[NSMutableArray alloc] init];
        _identifyArray = [[NSMutableArray alloc] init];
        _identifyDepth = 3;
        
        _parseQueue = [[NSMutableArray alloc] init];
        _parseArray = [[NSMutableArray alloc] init];
        _parseDepth = 1;
        
        _loadQueue = [[NSMutableArray alloc] init];
        _loadArray = [[NSMutableArray alloc] init];
        _loadDepth = 3;
        
        _resourceNumber = 0;
        _savedFileNumber = 0;
        _successfulFiles = 0;
        _paused = NO;
        
        _foundURLs = [[NSMutableSet alloc] init];
        
        _plugin = nil;
        _customImageLocation = nil;
        _customAudioLocation = nil;
        _customVideoLocation = nil;
        _customDocumentLocation = nil;
        _storageLookupTable = [[NSMutableDictionary alloc] init];
        
        _rootString = [[[[NSProcessInfo processInfo] arguments] objectAtIndex:1] retain];
        
        NSDictionary *errors = [NSDictionary dictionary];
        NSString *path = [[NSBundle mainBundle] pathForResource:@"setSpotlightComment" ofType:@"scpt"];
        if(path){
            NSURL *url = [NSURL fileURLWithPath:path];
            if(url){
                _commentScript = [[NSAppleScript alloc] initWithContentsOfURL:url error:&errors];
            }
        }
        
        
        // load and configure plugin
        if(![self loadPluginNamed:[[[NSProcessInfo processInfo] arguments] objectAtIndex:2]])
            NSLog(@"Could not load plugin!");
        [[NSUserDefaults standardUserDefaults] synchronize];
        NSDictionary *appPrefs = [[NSUserDefaults standardUserDefaults] persistentDomainForName:@"com.positivespinmedia.Pandora"];
        [_plugin setMIMETypes:[NSSet setWithArray:[appPrefs objectForKey:@"MIME types"]]];
        if([[appPrefs objectForKey:@"Use maximum file size"] boolValue])
            [_plugin setMaxFileSize:[[appPrefs objectForKey:@"Maximum file size"] intValue]];
        if([[appPrefs objectForKey:@"Use minimum file size"] boolValue])
            [_plugin setMinFileSize:[[appPrefs objectForKey:@"Minimum file size"] intValue]];
        
        if([_plugin contentFromDifferentServers]){
            _loadDepth = 7;
            _identifyDepth = 7;
        }
        
        // storage - strip out slashes for path info
        NSMutableString *storageFolder = [NSMutableString stringWithString:_rootString];
        if([storageFolder rangeOfString:@"://"].length != 0){
            int start = [storageFolder rangeOfString:@"://"].location + [storageFolder rangeOfString:@"://"].length;
            [storageFolder replaceCharactersInRange:NSMakeRange(0, start) withString:@""];
        }
        
        if([storageFolder rangeOfString:@"/"].length != 0){
            int start = [storageFolder rangeOfString:@"/"].location;
            int length = [storageFolder length] - [storageFolder rangeOfString:@"/"].location;
            [storageFolder replaceCharactersInRange:NSMakeRange(start, length) withString:@""];
        }
        
        NSFileManager *manager = [NSFileManager defaultManager];
        BOOL isDir;
        
        // default storage location
        if([appPrefs objectForKey:@"Default storage location"]){
            NSMutableString *storagePath = [[appPrefs objectForKey:@"Default storage location"] mutableCopy];
            NSString *machineLicenseFile = [[NSSearchPathForDirectoriesInDomains(NSApplicationSupportDirectory, NSLocalDomainMask, YES) objectAtIndex:0] stringByAppendingString:@"/Pandora/.com.positivespinmedia.Pandora.license"];
            if (!([manager fileExistsAtPath:storagePath isDirectory:&isDir] && isDir) || ![[NSFileManager defaultManager] fileExistsAtPath:machineLicenseFile]){
                [storagePath setString:NSTemporaryDirectory()];
            }
            
            _storageLocation = [self createSafeFolderPathFromBase:[[NSString alloc] initWithFormat:@"%@/%@/", storagePath, storageFolder]];
            [manager createDirectoryAtPath:_storageLocation attributes:nil];
        }
        
        // make folders for custom storage locations if needed
        if([[appPrefs objectForKey:@"Use custom image location"] boolValue]){
            NSMutableString *storagePath = [[appPrefs objectForKey:@"Default image location"] mutableCopy];
            if (!([manager fileExistsAtPath:storagePath isDirectory:&isDir] && isDir)){
                [storagePath setString:NSTemporaryDirectory()];
            }
            _customImageLocation = [self createSafeFolderPathFromBase:[NSString stringWithFormat:@"%@/%@/", storagePath, storageFolder]];
            [manager createDirectoryAtPath:_customImageLocation attributes:nil];
        }
        
        if([[appPrefs objectForKey:@"Use custom audio location"] boolValue]){
            NSMutableString *storagePath = [[appPrefs objectForKey:@"Default audio location"] mutableCopy];
            if (!([manager fileExistsAtPath:storagePath isDirectory:&isDir] && isDir)){
                [storagePath setString:NSTemporaryDirectory()];
            }
            _customAudioLocation = [self createSafeFolderPathFromBase:[NSString stringWithFormat:@"%@/%@/", storagePath, storageFolder]];
            [manager createDirectoryAtPath:_customAudioLocation attributes:nil];
        }
        
        if([[appPrefs objectForKey:@"Use custom video location"] boolValue]){
            NSMutableString *storagePath = [[appPrefs objectForKey:@"Default video location"] mutableCopy];
            if (!([manager fileExistsAtPath:storagePath isDirectory:&isDir] && isDir)){
                [storagePath setString:NSTemporaryDirectory()];
            }
            _customVideoLocation = [self createSafeFolderPathFromBase:[NSString stringWithFormat:@"%@/%@/", storagePath, storageFolder]];
            [manager createDirectoryAtPath:_customVideoLocation attributes:nil];
        }
        
        if([[appPrefs objectForKey:@"Use custom document location"] boolValue]){
            NSMutableString *storagePath = [[appPrefs objectForKey:@"Default document location"] mutableCopy];
            if (!([manager fileExistsAtPath:storagePath isDirectory:&isDir] && isDir)){
                [storagePath setString:NSTemporaryDirectory()];
            }
            _customDocumentLocation = [self createSafeFolderPathFromBase:[NSString stringWithFormat:@"%@/%@/", storagePath, storageFolder]];
            [manager createDirectoryAtPath:_customDocumentLocation attributes:nil];
        }
        
        // build storage lookup table from mime list
        NSMutableArray *mimeList = [[NSMutableArray alloc] initWithContentsOfFile:[NSString stringWithFormat:@"%@/Pandora/mime.plist", [NSSearchPathForDirectoriesInDomains(NSApplicationSupportDirectory, NSLocalDomainMask,YES) objectAtIndex:0]]];
        if(!mimeList)
            mimeList = [[NSMutableArray alloc] initWithContentsOfFile:[NSString stringWithFormat:@"%@/Pandora/mime.plist", [NSSearchPathForDirectoriesInDomains(NSApplicationSupportDirectory, NSUserDomainMask,YES) objectAtIndex:0]]];
        NSEnumerator *e = [mimeList objectEnumerator];
        NSMutableDictionary *listing;
        while(listing = [e nextObject]){
            if([[listing objectForKey:@"description"] isEqualToString:@"Pictures"] && [[appPrefs objectForKey:@"Use custom image location"] boolValue]){
                NSEnumerator *c = [[listing objectForKey:@"children"] objectEnumerator];
                NSDictionary *child;
                while(child = [c nextObject]){
                    [_storageLookupTable setObject:_customImageLocation forKey:[child objectForKey:@"MIME type"]];
                }
            }
            
            if([[listing objectForKey:@"description"] isEqualToString:@"Music"] && [[appPrefs objectForKey:@"Use custom audio location"] boolValue]){
                NSEnumerator *c = [[listing objectForKey:@"children"] objectEnumerator];
                NSDictionary *child;
                while(child = [c nextObject]){
                    [_storageLookupTable setObject:_customAudioLocation forKey:[child objectForKey:@"MIME type"]];
                }
            }
            
            if([[listing objectForKey:@"description"] isEqualToString:@"Movies"] && [[appPrefs objectForKey:@"Use custom video location"] boolValue]){
                NSEnumerator *c = [[listing objectForKey:@"children"] objectEnumerator];
                NSDictionary *child;
                while(child = [c nextObject]){
                    [_storageLookupTable setObject:_customVideoLocation forKey:[child objectForKey:@"MIME type"]];
                }
            }
            
            if([[listing objectForKey:@"description"] isEqualToString:@"Documents"] && [[appPrefs objectForKey:@"Use custom document location"] boolValue]){
                NSEnumerator *c = [[listing objectForKey:@"children"] objectEnumerator];
                NSDictionary *child;
                while(child = [c nextObject]){
                    [_storageLookupTable setObject:_customDocumentLocation forKey:[child objectForKey:@"MIME type"]];
                }
            }
        }
        
        // user agent
        _userAgent = [appPrefs objectForKey:@"User agent"];
        
        NSTimer *parentCheckTimer = [NSTimer scheduledTimerWithTimeInterval:10.0 target:self selector:@selector(parentAppCheck:) userInfo:nil repeats:YES];
        [parentCheckTimer retain];
    }
    
    return self;
}

- (void)dealloc
{
    [_identifyQueue release];
    [_identifyArray release];
    [_parseQueue release];
    [_parseArray release];
    [_loadQueue release];
    [_loadArray release];
    [_foundURLs release];
    [_plugin release];
    [_rootString release];
    [_storageLocation release];
    [super dealloc];
}

#pragma mark -
#pragma mark Accessors

- (NSString *)storageLocation
{
    return _storageLocation;
}

- (NSString *)storageLocationForMIMEType:(NSString *)type
{
    if([_storageLookupTable objectForKey:type]){
        return [_storageLookupTable objectForKey:type];
    }
    
    return _storageLocation;
}

- (void)setStorageLocation:(NSString *)path
{
    [path retain];
    [_storageLocation release];
    _storageLocation = path;
}

- (NSString *)rootString
{
    return _rootString;
}

- (void)setRootString:(NSString *)value
{
    [value retain];
    [_rootString release];
    _rootString = value;
}

- (id)plugin
{
    return _plugin;
}

- (int)resourceNumber
{
    _resourceNumber++;
    return _resourceNumber;
}

- (NSMutableArray *)parseArray
{
    return _parseArray;
}

- (NSAppleScript *)commentScript
{
    return _commentScript;
}

- (oneway void)setSessionObject:(in byref id <PandoraSession>)session
{
    [session retain];
    [_session release];
    _session = session;
    [(NSDistantObject *)_session setProtocolForProxy:@protocol(PandoraSession)];
}

- (NSString *)userAgent
{
    return _userAgent;
}

- (id)session
{
    return _session;
}

- (int)savedFileNumber
{
    return _savedFileNumber++;
}

#pragma mark -
#pragma mark Functionality

- (BOOL)loadPluginNamed:(NSString *)bundleName
{
    // preferentially load external plugins
    NSString *appSupportSubpath = [NSString stringWithFormat:@"Application Support/Pandora/PlugIns/%@", bundleName];
    NSMutableArray *librarySearchPaths = (NSMutableArray *)NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSAllDomainsMask - NSSystemDomainMask, YES);
    NSString *currentPath;
    NSEnumerator *e = [librarySearchPaths objectEnumerator];
    while(currentPath = [e nextObject]){
        NSString *currentDirectory = [currentPath stringByAppendingPathComponent:appSupportSubpath];
        if([[NSFileManager defaultManager] fileExistsAtPath:currentDirectory]){
            // load
            NSBundle *bundle = [NSBundle bundleWithPath:currentDirectory];
            Class principalClass = [bundle principalClass];
            _plugin = [[principalClass alloc] init];
            return YES;
        }
    }
    
    // internal plugins
    NSString *localPlugin = [[[[[NSBundle mainBundle] bundlePath] stringByDeletingLastPathComponent] stringByDeletingLastPathComponent] stringByAppendingFormat:@"/PlugIns/%@", bundleName];
    if([[NSFileManager defaultManager] fileExistsAtPath:localPlugin]){
        // load
        NSBundle *bundle = [NSBundle bundleWithPath:localPlugin];
        Class principalClass = [bundle principalClass];
        _plugin = [[principalClass alloc] init];
        return YES;
    }
    
    NSLog(@"Failed to load plugin \"%@\"", bundleName);
    return NO; // failed to load named plugin
}

- (void)startProcessing
{
    //NSLog(@"startProcessing called - sending \"%@\" to plugin", [self rootString]);
    NSArray *initialStart = [[_plugin preprocessURLOrKeyword:[self rootString]] retain];
    //NSLog(@"startProcessing. returned array: %@", initialStart);
    NSEnumerator *e = [initialStart objectEnumerator];
    NSDictionary *currentURL;
    while(currentURL = [e nextObject]){
        [self createWebResourceForURL:[currentURL objectForKey:@"URL"] action:[currentURL objectForKey:@"action"] withParent:nil fromResource:nil];
    }
    [initialStart release];
    [_session logConsoleMessageWithURL:[NSString stringWithFormat:@"%@ processing for \"%@\"", [[_plugin class] name], [self rootString]] status:@"Processing" andComment:[[NSDate date] description]];
}

- (void)pauseProcessing
{
    // must hold things up until later (i.e. for sleep or loss of connection)
    _paused = YES;
    
    // identification
    NSEnumerator *ia = [_identifyArray reverseObjectEnumerator];
    PSMWebResource *iaObject;
    while(iaObject = [ia nextObject]){
        PSMWebResource *newResource = [[PSMWebResource alloc] initWithSourceURL:[iaObject sourceURL] fromSpider:self withPlugin:_plugin];
        [newResource setParentWebPage:[iaObject parentWebPage]];
        [newResource setResourceID:[iaObject resourceID]];
        [newResource setLinkDepth:[iaObject linkDepth]];
        if([_identifyQueue count] == 0){
            [_identifyQueue addObject:newResource];
        } else {
            [_identifyQueue insertObject:newResource atIndex:0];
        }
    }
    [_identifyArray makeObjectsPerformSelector:@selector(dispose)];
    [_identifyArray removeAllObjects];
    
    // parsing
    NSEnumerator *pa = [_parseArray reverseObjectEnumerator];
    PSMWebResource *paObject;
    while(paObject = [pa nextObject]){
        PSMWebResource *newResource = [[PSMWebResource alloc] initWithSourceURL:[paObject sourceURL] fromSpider:self withPlugin:_plugin];
        [newResource setParentWebPage:[paObject parentWebPage]];
        [newResource setResourceID:[paObject resourceID]];
        [newResource setLinkDepth:[paObject linkDepth]];
        if([_parseQueue count] == 0){
            [_parseQueue addObject:newResource];
        } else {
            [_parseQueue insertObject:newResource atIndex:0];
        }
    }
    [_identifyArray makeObjectsPerformSelector:@selector(dispose)];
    [_parseArray removeAllObjects];
    
    // loading
    NSEnumerator *la = [_loadArray reverseObjectEnumerator];
    PSMWebResource *laObject;
    while(laObject = [la nextObject]){
        PSMWebResource *newResource = [[PSMWebResource alloc] initWithSourceURL:[laObject sourceURL] fromSpider:self withPlugin:_plugin];
        [newResource setParentWebPage:[laObject parentWebPage]];
        [newResource setResourceID:[laObject resourceID]];
        [newResource setLinkDepth:[laObject linkDepth]];
        if([_loadQueue count] == 0){
            [_loadQueue addObject:newResource];
        } else {
            [_loadQueue insertObject:newResource atIndex:0];
        }
    }
    [_identifyArray makeObjectsPerformSelector:@selector(dispose)];
    [_loadArray removeAllObjects];
    
    // everything has been pushed back into the queues, and will not advance until a resume message
    [_session logConsoleMessageWithURL:[NSString stringWithFormat:@"Spider paused for \"%@\"", [self rootString]] status:@"Paused" andComment:[[NSDate date] description]];
}

- (void)resumeProcessing
{
    _paused = NO;
    
    // refill queues
    // identify
    int i;
    for(i = 0; i < _identifyDepth; i++){
        [self advanceIdentifyQueue];
    }
    
    // parse
    for(i = 0; i < _parseDepth; i++){
        [self advanceParseQueue];
    }
    
    // load
    for(i = 0; i < _loadDepth; i++){
        [self advanceLoadQueue];
    }
    
    [_session logConsoleMessageWithURL:[NSString stringWithFormat:@"Spider processing for \"%@\"", [self rootString]] status:@"Processing" andComment:[[NSDate date] description]];
}

- (void)stopProcessing
{
    // clean up if desired
    NSDictionary *appPrefs = [[NSUserDefaults standardUserDefaults] persistentDomainForName:@"com.positivespinmedia.Pandora"];
    if([[appPrefs objectForKey:@"Delete files when finished"] boolValue]){
        // nuke my storage location
        [[NSFileManager defaultManager] removeFileAtPath:_storageLocation handler:nil];
        if([[appPrefs objectForKey:@"Use custom image location"] boolValue])
            [[NSFileManager defaultManager] removeFileAtPath:_customImageLocation handler:nil];
        if([[appPrefs objectForKey:@"Use custom audio location"] boolValue])
            [[NSFileManager defaultManager] removeFileAtPath:_customAudioLocation handler:nil];
        if([[appPrefs objectForKey:@"Use custom video location"] boolValue])
            [[NSFileManager defaultManager] removeFileAtPath:_customVideoLocation handler:nil];
        if([[appPrefs objectForKey:@"Use custom document location"] boolValue])
            [[NSFileManager defaultManager] removeFileAtPath:_customDocumentLocation handler:nil];
    }
        
    [NSApp terminate:self];
}

- (void)dumpSnapshot
{
    // log the contents of the queues
    NSLog(@"identify: %@", _identifyArray);
    NSLog(@"parse: %@", _parseArray);
    NSLog(@"load: %@", _loadArray);
}

- (void)createWebResourceForURL:(NSString *)url action:(NSString *)action withParent:(NSString *)parentURL fromResource:(PSMWebResource *)resource
{
    // trim any named anchors from URL
    if([url rangeOfString:@"#"].length > 0)
        url = [url substringToIndex:[url rangeOfString:@"#"].location];
    
    if([_foundURLs containsObject:url])
        return;
    
    int linkDepthPreference = [_plugin linkDepth];
    if((linkDepthPreference >= 0) && (resource != nil)){
        if([resource linkDepth] > linkDepthPreference)
            return; // no adding if beyond desired link depth
    }
    
    [_foundURLs addObject:url];
    
    // create and configure
    PSMWebResource *newResource = [[PSMWebResource alloc] initWithSourceURL:url fromSpider:self withPlugin:_plugin];
    [newResource setParentWebPage:parentURL];
    [newResource setResourceID:[self resourceNumber]];
    if(resource != nil){
        if([action isEqualToString:@"load"]){
            [newResource setLinkDepth:[resource linkDepth]]; // set images to same depth as parent page
        } else {
            [newResource setLinkDepth:([resource linkDepth] + 1)];
        }
    } else {
        [newResource setLinkDepth:0];
    }
    
	//NSLog(@"resource created: %@, %@", newResource, action);
    // place in process
    if([action isEqualToString:@"identify"]){
        [_identifyQueue addObject:newResource];
        [self advanceIdentifyQueue];
    }
    
    if([action isEqualToString:@"parse"]){
        [_parseQueue addObject:newResource];
        [self advanceParseQueue];
    }
    
    if([action isEqualToString:@"load"]){
        [_loadQueue addObject:newResource];
        [self advanceLoadQueue];
    }
    
    // logging
    [_session logNewResourceID:[newResource resourceID] withURL:[newResource sourceURL]];
}

#pragma mark -
#pragma mark Parent App Verification

- (void)parentAppCheck:(NSTimer *)timer
{
    NSArray *appList = [[NSWorkspace sharedWorkspace] launchedApplications];
    BOOL parentIsRunning = NO;
    NSEnumerator *e = [appList objectEnumerator];
    NSString *thisBundleID;
    while(thisBundleID = [[e nextObject] objectForKey:@"NSApplicationBundleIdentifier"]){
        if([thisBundleID isEqualToString:@"com.positivespinmedia.Pandora"])
            parentIsRunning = YES;
    }
    if(!parentIsRunning){
        [self stopProcessing];
    }
}

#pragma mark -
#pragma mark Queue Management

- (void)advanceIdentifyQueue
{
    if(_paused)
        return;
    if([_identifyArray count] < _identifyDepth){
        if([_identifyQueue count] > 0){
            PSMWebResource *nextObject = [_identifyQueue objectAtIndex:0];
            [_identifyArray addObject:nextObject];
            [_identifyQueue removeObject:nextObject];
            [_session logStatus:@"Identifying" forResourceID:[nextObject resourceID]];
            [nextObject identify];
        }
    }
    if([self finishedProcessing])
        [_session setIsDone:YES];
}

- (void)advanceParseQueue
{
    if(_paused)
        return;
    if([_parseArray count] < _parseDepth){
        if([_parseQueue count] > 0){
            PSMWebResource *nextObject = [_parseQueue objectAtIndex:0];
            [_parseArray addObject:nextObject];
            [_parseQueue removeObject:nextObject];
            [_session logStatus:@"Parsing" forResourceID:[nextObject resourceID]];
            [nextObject parse];
			//NSLog(@"resource: %@", nextObject);
        }
    }
    if([self finishedProcessing])
        [_session setIsDone:YES];
}

- (void)advanceLoadQueue
{
    if(_paused)
        return;
    if([_loadArray count] < _loadDepth){
        if([_loadQueue count] > 0){
            PSMWebResource *nextObject = [_loadQueue objectAtIndex:0];
            [_loadArray addObject:nextObject];
            [_loadQueue removeObject:nextObject];
            [_session logStatus:@"Loading" forResourceID:[nextObject resourceID]];
            [nextObject load];
        }
    }
    if([self finishedProcessing])
        [_session setIsDone:YES];
}

#pragma mark -
#pragma mark Resource Management

- (void)resourceIdentified:(PSMWebResource *)resource
{
    if(_paused)
        return;
    
    [_identifyArray removeObject:resource];
    [self advanceIdentifyQueue];
    
    [_session logStatus:@"Identified" forResourceID:[resource resourceID]];
    [_session logMIMEType:[resource MIMEType] forResourceID:[resource resourceID]];
    [_session logProgress:[NSString stringWithFormat:@"%@ reported", [self fileSizeStringForLongLong:[resource expectedContentLength]]] forResourceID:[resource resourceID]];
    
    // always follow HTML
    if([[resource MIMEType] isEqualToString:@"text/html"]){
        [_parseQueue addObject:resource];
        [self advanceParseQueue];
        return;
    }
    
    NSString *reason = nil;
    if(![_plugin shouldPursueResourceWithURL:[resource sourceURL] MIMEType:[resource MIMEType] andExpectedFileSize:[resource expectedContentLength] withReason:&reason]){
        [_session logStatus:@"Skipped" forResourceID:[resource resourceID]];
        if(reason)
            [_session logComment:reason forResourceID:[resource resourceID]];
        [resource release]; // toss it
        return;
    }
    
    // default - load the resource
    [_loadQueue addObject:resource];
    [self advanceLoadQueue];
        
}

- (void)resourceParsed:(PSMWebResource *)resource
{
    if(_paused)
        return;
    
    [_parseArray removeObject:resource];
    [self advanceParseQueue];
    
    [_session logStatus:@"Parsed" forResourceID:[resource resourceID]];
    [_session logProgress:[NSString stringWithFormat:@"%@",[self fileSizeStringForLongLong:[resource actualContentLength]]] forResourceID:[resource resourceID]];
    
    [resource release];
}

- (void)resourceLoaded:(PSMWebResource *)resource
{
    if(_paused)
        return;
    
    if([_loadArray containsObject:resource]){
        [_loadArray removeObject:resource];
        [self advanceLoadQueue];
    }
    
    // object could have erroneously been placed in parse queue, due to bad HTTP headers
    if([_parseArray containsObject:resource]){
        [_parseArray removeObject:resource];
        [self advanceParseQueue];
    }
    
    [_session logStatus:@"Loaded" forResourceID:[resource resourceID]];
    [_session logMIMEType:[resource MIMEType] forResourceID:[resource resourceID]];
    [_session logProgress:[NSString stringWithFormat:@"%@",[self fileSizeStringForLongLong:[resource actualContentLength]]] forResourceID:[resource resourceID]];
    
    NSString *reason = nil;
    if(![_plugin shouldPursueResourceWithURL:[resource sourceURL] MIMEType:[resource MIMEType] andExpectedFileSize:[resource actualContentLength] withReason:&reason]){
        [_session logStatus:@"Skipped" forResourceID:[resource resourceID]];
        if(reason)
            [_session logComment:reason forResourceID:[resource resourceID]];
        [resource release]; // toss it
        return;
    } else {
        [resource saveDataToFile];
        [resource createThumbnail];
    }
}

- (void)resourceReady:(PSMWebResource *)resource
{
	//NSLog(@"called");
    if(_paused)
        return;
    
    [_session logStatus:@"Completed" forResourceID:[resource resourceID]];
    
    // create webfile to send
    PSMWebFile *webFile = [[PSMWebFile alloc] initWithResourceID:[resource resourceID]];
    [webFile setSourceURL:[resource sourceURL]];
    [webFile setParentWebPage:[resource parentWebPage]];
    [webFile setMIMEType:[resource MIMEType]];
    [webFile setActualContentLength:[resource actualContentLength]];
    [webFile setThumbnail:[resource thumbnail]];
    [webFile setStoragePath:[resource storagePath]];
    [webFile setAliasData:[resource aliasData]];
    [_session addWebFile:webFile];
    
    // clean up
    [webFile release];
    [resource release];
    
    // collection limit?
    _successfulFiles++;
    NSDictionary *appPrefs = [[NSUserDefaults standardUserDefaults] persistentDomainForName:@"com.positivespinmedia.Pandora"];
    if([[appPrefs objectForKey:@"Stop at limit"] boolValue]){
        if((_successfulFiles % [[appPrefs objectForKey:@"Stop limit amount"] intValue]) == 0){
            _paused = YES;
            [_session pauseEngine];
        }
    }
}

- (void)resourceConnectionFailed:(PSMWebResource *)resource
{
    if(_paused)
        return;
    
    [_session logStatus:@"Failed" forResourceID:[resource resourceID]];
    
    // clean it out of processing
    if([_identifyArray containsObject:resource]){
        [_identifyArray removeObject:resource];
        [self advanceIdentifyQueue];
    }
    if([_parseArray containsObject:resource]){
        [_parseArray removeObject:resource];
        [self advanceParseQueue];
    }
    if([_loadArray containsObject:resource]){
        [_loadArray removeObject:resource];
        [self advanceLoadQueue];
    }
    
    [resource release];
}

#pragma mark -
#pragma mark Convenience

- (NSString *)createSafeFilePathFromBase:(NSString *)creationPath
{
    int numberWithName = 1;
    BOOL isDir;
    NSString *safePath;
    safePath = [creationPath retain];
    
    while ([[NSFileManager defaultManager] fileExistsAtPath:safePath isDirectory:&isDir]){
        [safePath release];
        safePath = [[NSString alloc] initWithFormat:@"%@-%d.%@", [creationPath stringByDeletingPathExtension], numberWithName, [creationPath pathExtension]];
        numberWithName++;
    }
    return [safePath autorelease];
}

- (NSString *)createSafeFolderPathFromBase:(NSString *)creationPath
{
    int numberWithName = 1;
    BOOL isDir;
    NSString *safePath;
    safePath = [creationPath retain];
    
    while ([[NSFileManager defaultManager] fileExistsAtPath:safePath isDirectory:&isDir]){
        [safePath release];
        safePath = [[NSString alloc] initWithFormat:@"%@/%@-%d/", [creationPath stringByDeletingLastPathComponent], [creationPath lastPathComponent], numberWithName];
        numberWithName++;
    }
    return [safePath autorelease];
}

- (NSString *)fileSizeStringForLongLong:(long long)size
{
    if(size < 0)
        return @"???";
    
    NSString *sizeString;
    
    if((size/1024) < 1000){
        // measure in K
        int thisSize = floor(size/1024);
        sizeString = [NSString stringWithFormat:@"%d K",thisSize];
    } else {
        // measure in M
        float thisSize = size/(1024.0*1024.0);
        sizeString = [NSString stringWithFormat:@"%0.2f M",thisSize];
    }
    
    return sizeString;
}

- (NSString *)fileSizeStringForUnsigned:(unsigned)size
{
    if(size < 0)
        return @"???";
    
    NSString *sizeString;
    
    if((size/1024) < 1000){
        // measure in K
        int thisSize = floor(size/1024);
        sizeString = [NSString stringWithFormat:@"%d K",thisSize];
    } else {
        // measure in M
        float thisSize = size/(1024.0*1024.0);
        sizeString = [NSString stringWithFormat:@"%0.2f M",thisSize];
    }
    
    return sizeString;
}

- (BOOL)finishedProcessing
{
    if([_identifyArray count] > 0)
        return NO;
    if([_identifyQueue count] > 0)
        return NO;
    if([_parseArray count] > 0)
        return NO;
    if([_parseQueue count] > 0)
        return NO;
    if([_loadArray count] > 0)
        return NO;
    if([_loadQueue count] > 0)
        return NO;
    
    return YES;
}

@end
